@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: Environment.java,v 1.142 2004/08/21 02:22:11 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: Environment.java,v 1.171 2006/01/03 21:55:37 bostic Exp $
+ */
 
 package com.sleepycat.je;
 
@@ -27,6 +27,7 @@ import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.DbEnvPool;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.txn.Locker;
+import com.sleepycat.je.txn.LockerFactory;
 import com.sleepycat.je.utilint.Tracer;
 
 /**
@@ -41,7 +42,7 @@ public class Environment {
      */
     private static final String PROPFILE_NAME = "je.properties";
 
-    private EnvironmentImpl environmentImpl;
+    protected EnvironmentImpl environmentImpl;
     private TransactionConfig defaultTxnConfig;
     private EnvironmentMutableConfig handleConfig;
 
@@ -62,7 +63,7 @@ public class Environment {
         referringDbTxns = Collections.synchronizedSet(new HashSet());
         valid = false;
 
-        DatabaseUtil.checkForNullParam(envHome, "dbEnvHome");
+        DatabaseUtil.checkForNullParam(envHome, "envHome");
 
         /* If the user specified a null object, use the default */
         EnvironmentConfig baseConfig =
@@ -73,7 +74,7 @@ public class Environment {
         /* Make a copy, apply je.properties, and init the handle config. */
         EnvironmentConfig useConfig = baseConfig.cloneConfig();
         applyFileConfig(envHome, useConfig);
-        copyToHandleConfig(useConfig);
+        copyToHandleConfig(useConfig, useConfig);
 
         /* Look in the shared pool for this environment. */
         DbEnvPool.EnvironmentImplInfo envInfo  =
@@ -89,21 +90,18 @@ public class Environment {
             synchronized (environmentImpl) {
 
                 /* 
-                 * If a non-null configuration parameter was passed in
-                 * and this is not the handle that created the
-                 * underlying EnvironmentImpl, check that the
-                 * configuration parameters specified for this open
-                 * match those of the currently open environment. An
-                 * exception is thrown if the check fails.
+                 * If a non-null configuration parameter was passed in and this
+                 * is not the handle that created the underlying
+                 * EnvironmentImpl, check that the configuration parameters
+                 * specified for this open match those of the currently open
+                 * environment. An exception is thrown if the check fails.
                  *
-                 * Don't do this check if this handle created the
-                 * environment because the creation might have
-                 * modified the parameters, which would create a
-                 * Catch-22 in terms of validation.  For example,
-                 * je.maxMemory will be overridden if the JVM's -mx
-                 * flag is less than that setting, so the initial
-                 * handle's config object won't be the same as the
-                 * passed in config.
+                 * Don't do this check if this handle created the environment
+                 * because the creation might have modified the parameters,
+                 * which would create a Catch-22 in terms of validation.  For
+                 * example, je.maxMemory will be overridden if the JVM's -mx
+                 * flag is less than that setting, so the initial handle's
+                 * config object won't be the same as the passed in config.
                  */
                 environmentImpl.checkImmutablePropsForEquality(useConfig);
             }
@@ -117,6 +115,48 @@ public class Environment {
         environmentImpl.incReferenceCount();
     }
 
+    /**
+     * Get an Environment for an existing EnvironmentImpl. Used by utilities
+     * such as the JMX MBean which don't want to open the environment or be
+     * reference counted. The calling application must take care not to retain
+     * the the doc templates in the doc_src directory.
+     */
+    Environment(File envHome) 
+        throws DatabaseException {
+
+        environmentImpl = null;
+        valid = false;
+
+        /* Look in the shared pool for this environment. */
+        DbEnvPool.EnvironmentImplInfo envInfo  =
+            DbEnvPool.getInstance().getExistingEnvironment(envHome);
+        
+        EnvironmentImpl foundImpl = envInfo.envImpl;
+        if (foundImpl != null) {
+            /* Check if the environmentImpl is valid. */
+            foundImpl.checkIfInvalid();
+
+            /* Successful, increment reference count */
+            environmentImpl = foundImpl;
+            environmentImpl.incReferenceCount();
+
+            /* 
+             * Initialize the handle's environment config, so that it's valid
+             * to call setConfig and getConfig against this handle.  Make a
+             * copy, apply je.properties, and init the handle config.
+             */
+            EnvironmentConfig useConfig =
+                EnvironmentConfig.DEFAULT.cloneConfig();
+            applyFileConfig(envHome, useConfig);
+            copyToHandleConfig(useConfig, useConfig);
+
+            /* Need this in order to open database handles. */
+            referringDbs = Collections.synchronizedSet(new HashSet());
+
+            valid = true;
+        }
+    }
+
     /**
      * Apply the configurations specified in the je.properties file to override
      * any programatically set configurations.
@@ -131,8 +171,10 @@ public class Environment {
             try {
                 paramFile = new File(envHome, PROPFILE_NAME);
                 Properties fileProps = new Properties();	 
-                fileProps.load(new FileInputStream(paramFile));	 
-                
+		FileInputStream fis = new FileInputStream(paramFile);
+                fileProps.load(fis);
+                fis.close();
+
                 /* Validate the existing file. */
                 useConfig.validateProperties(fileProps);
                 
@@ -144,9 +186,12 @@ public class Environment {
                     String value = (String) propPair.getValue();
                     useConfig.setConfigParam(name, value);
                 }
-                
             } catch (FileNotFoundException e) {	 
-                // eat the exception, okay if the file doesn't exist	 
+
+		/* 
+		 * Klockwork - ok
+                 * Eat the exception, okay if the file doesn't exist.
+		 */
             } catch (IOException e) {	 
                 IllegalArgumentException e2 = new IllegalArgumentException
                     ("An error occurred when reading " + paramFile);
@@ -167,6 +212,7 @@ public class Environment {
         try {
             checkEnv();
         } catch (RunRecoveryException e) {
+
             /* 
              * We're trying to close on an environment that has seen a fatal
              * exception. Try to do the minimum, such as closing file
@@ -181,68 +227,72 @@ public class Environment {
 
         StringBuffer errors = new StringBuffer();
         try {
-            int nDbs = referringDbs.size();
-            if (nDbs != 0) {
-                errors.append("There ");
-                if (nDbs == 1) {
-                    errors.append
-                        ("is 1 open Database in the Environment.\n");
-                } else {
-                    errors.append("are ");
-                    errors.append(nDbs);
-                    errors.append
-                        (" open Database in the Environment.\n");
-                }
-                errors.append("Closing the following databases:\n");
+            if (referringDbs != null) {
+                int nDbs = referringDbs.size();
+                if (nDbs != 0) {
+                    errors.append("There ");
+                    if (nDbs == 1) {
+                        errors.append
+                            ("is 1 open Database in the Environment.\n");
+                    } else {
+                        errors.append("are ");
+                        errors.append(nDbs);
+                        errors.append
+                            (" open Database in the Environment.\n");
+                    }
+                    errors.append("Closing the following databases:\n");
 
-                Iterator iter = referringDbs.iterator();
-                while (iter.hasNext()) {
-                    Database db = (Database) iter.next();
-                    /*
-                     * Save the db name before we attempt the close, it's 
-                     * unavailable after the close.
-                     */
-                    String dbName = db.getDatabaseName(); 
-                    errors.append(dbName).append(" ");
-                    try {
-                        db.close();
-                    } catch (RunRecoveryException e) {
-                        throw e;
-                    } catch (DatabaseException DBE) {
-                        errors.append("\nWhile closing Database ");
-                        errors.append(dbName);
-                        errors.append(" encountered exception: ");
-                        errors.append(DBE).append("\n");
+                    Iterator iter = referringDbs.iterator();
+                    while (iter.hasNext()) {
+                        Database db = (Database) iter.next();
+                        /*
+                         * Save the db name before we attempt the close, it's 
+                         * unavailable after the close.
+                         */
+                        String dbName = db.getDebugName(); 
+                        errors.append(dbName).append(" ");
+                        try {
+                            db.close();
+                        } catch (RunRecoveryException e) {
+                            throw e;
+                        } catch (DatabaseException DBE) {
+                            errors.append("\nWhile closing Database ");
+                            errors.append(dbName);
+                            errors.append(" encountered exception: ");
+                            errors.append(DBE).append("\n");
+                        }
                     }
                 }
             }
 
-            int nTxns = referringDbTxns.size();
-            if (nTxns != 0) {
-                Iterator iter = referringDbTxns.iterator();
-                errors.append("There ");
-                if (nTxns == 1) {
-                    errors.append("is 1 existing transaction opened against");
-                    errors.append(" the Environment.\n");
-                } else {
-                    errors.append("are ");
-                    errors.append(nTxns);
-                    errors.append(" existing transactions opened against");
-                    errors.append(" the Environment.\n");
-                }
-                errors.append("Aborting open transactions ...\n");
+            if (referringDbTxns != null) {
+                int nTxns = referringDbTxns.size();
+                if (nTxns != 0) {
+                    Iterator iter = referringDbTxns.iterator();
+                    errors.append("There ");
+                    if (nTxns == 1) {
+                        errors.append("is 1 existing transaction opened against");
+                        errors.append(" the Environment.\n");
+                    } else {
+                        errors.append("are ");
+                        errors.append(nTxns);
+                        errors.append(" existing transactions opened against");
+                        errors.append(" the Environment.\n");
+                    }
+                    errors.append("Aborting open transactions ...\n");
 
-                while (iter.hasNext()) {
-                    Transaction txn = (Transaction) iter.next();
-                    try {
-                        txn.abort();
-                    } catch (RunRecoveryException e) {
-                        throw e;
-                    } catch (DatabaseException DBE) {
-                        errors.append("\nWhile aborting transaction ");
-                        errors.append(txn.getId());
-                        errors.append(" encountered exception: ");
-                        errors.append(DBE).append("\n");
+                    while (iter.hasNext()) {
+                        Transaction txn = (Transaction) iter.next();
+                        try {
+                            txn.abort();
+                        } catch (RunRecoveryException e) {
+                            throw e;
+                        } catch (DatabaseException DBE) {
+                            errors.append("\nWhile aborting transaction ");
+                            errors.append(txn.getId());
+                            errors.append(" encountered exception: ");
+                            errors.append(DBE).append("\n");
+                        }
                     }
                 }
             }
@@ -264,11 +314,7 @@ public class Environment {
             }
         }
     }
-
-    /*
-     * Manage databases.
-     */
-
+    
     /**
      * Javadoc for this public method is generated via
      * the doc templates in the doc_src directory.
@@ -306,7 +352,9 @@ public class Environment {
         return db;
     }
 
-    private void openDb(Transaction txn, Database newDb, String databaseName,
+    private void openDb(Transaction txn,
+			Database newDb,
+			String databaseName,
                         DatabaseConfig dbConfig,
                         boolean needWritableLockerForInit)
         throws DatabaseException {
@@ -322,12 +370,13 @@ public class Environment {
          * Check that the open configuration doesn't conflict with the
          * environmentImpl configuration.
          */
-        validateDbConfigAgainstEnv(dbConfig,
-                                   databaseName);
+        validateDbConfigAgainstEnv(dbConfig, databaseName);
 
         Locker locker = null;
         boolean operationOk = false;
+	boolean dbIsClosing = false;
         try {
+
             /* 
              * Does this database exist? Get a transaction to use. If the
              * database exists already, we really only need a readable locker.
@@ -339,48 +388,56 @@ public class Environment {
              */
             boolean isWritableLocker;
             if (needWritableLockerForInit) {
-                locker = DatabaseUtil.getWritableLocker
-                            (this, txn,
-                             dbConfig.getTransactional(),
-                             true /*retainNonTxnLocks*/);
+                locker = LockerFactory.getWritableLocker
+                    (this,
+                     txn,
+                     dbConfig.getTransactional(),
+                     true,  // retainNonTxnLocks
+                     null);
                 isWritableLocker = true;
             } else {
-                locker = DatabaseUtil.getReadableLocker
-                            (this, txn,
-                             dbConfig.getTransactional(),
-                             true /*retainNonTxnLocks*/);
+                locker = LockerFactory.getReadableLocker
+                    (this, txn,
+                     dbConfig.getTransactional(),
+                     true,   // retainNonTxnLocks
+                     false); // readCommittedIsolation
                 isWritableLocker = !dbConfig.getTransactional() ||
-                                   (txn != null && locker == txn.getLocker());
+                                   locker.isTransactional();
             }
+
             DatabaseImpl database = environmentImpl.getDb(locker,
                                                           databaseName,
                                                           newDb);
             boolean databaseExists =
-                (database == null) ? false: 
-                ((database.getIsDeleted())? false : true);
+                (database == null) ? false : 
+                ((database.isDeleted()) ? false : true);
 
             if (databaseExists) {
                 if (dbConfig.getAllowCreate() &&
                     dbConfig.getExclusiveCreate()) {
                     /* We intended to create this, but it already exists. */
+		    dbIsClosing = true;
                     throw new DatabaseException
-                        ("Database " + database + " already exists");
+                        ("Database " + databaseName + " already exists");
                 }
 
                 newDb.initExisting(this, locker, database, dbConfig);
             } else {
                 /* No database. Create if we're allowed to. */
                 if (dbConfig.getAllowCreate()) {
+
                     /* 
                      * We're going to have to do some writing. Switch to a
                      * writable locker if we don't already have one.
                      */
                     if (!isWritableLocker) {
                         locker.operationEnd(OperationStatus.SUCCESS);
-                        locker = DatabaseUtil.getWritableLocker
-                                    (this, txn,
-                                     dbConfig.getTransactional(),
-                                     true /*retainNonTxnLocks*/);
+                        locker = LockerFactory.getWritableLocker
+                            (this,
+                             txn,
+                             dbConfig.getTransactional(),
+                             true,  // retainNonTxnLocks
+                             null);
                         isWritableLocker  = true;
                     }
 
@@ -396,15 +453,15 @@ public class Environment {
             operationOk = true;
             addReferringHandle(newDb);
         } finally {
+
             /*
-             * Tell the transaction that this operation is over. Some
-             * types of transactions (BasicLocker and AutoTxn) will actually
-             * finish. The transaction can decide if it is finishing and 
-             * if it needs to transfer the db handle lock it owns to someone
-             * else.
+             * Tell the transaction that this operation is over. Some types of
+             * transactions (BasicLocker and AutoTxn) will actually finish. The
+             * transaction can decide if it is finishing and if it needs to
+             * transfer the db handle lock it owns to someone else.
              */
             if (locker != null) {
-                locker.setHandleLockOwner(operationOk, newDb, false);
+                locker.setHandleLockOwner(operationOk, newDb, dbIsClosing);
                 locker.operationEnd(operationOk);
             }
         }
@@ -413,6 +470,7 @@ public class Environment {
     private void validateDbConfigAgainstEnv(DatabaseConfig dbConfig,
                                             String databaseName)
         throws DatabaseException {
+
         /* Check operation's transactional status against the Environment */
         if (dbConfig.getTransactional() &&
             !(environmentImpl.isTransactional())) {
@@ -445,14 +503,16 @@ public class Environment {
         Locker locker = null;
         boolean operationOk = false;
         try {
+
             /* 
              * Note: use env level isTransactional as proxy for the db
              * isTransactional.
              */
-            locker = DatabaseUtil.getWritableLocker
+            locker = LockerFactory.getWritableLocker
                         (this, txn,
                          environmentImpl.isTransactional(),
-                         true /*retainNonTxnLocks*/);
+                         true /*retainNonTxnLocks*/,
+                         null);
             environmentImpl.dbRemove(locker, databaseName);
             operationOk = true;
         } finally {
@@ -480,14 +540,16 @@ public class Environment {
         Locker locker = null;
         boolean operationOk = false;
         try {
-            /* 
+
+            /*
              * Note: use env level isTransactional as proxy for the db
              * isTransactional.
              */
-            locker = DatabaseUtil.getWritableLocker
+            locker = LockerFactory.getWritableLocker
                         (this, txn,
                          environmentImpl.isTransactional(),
-                         true /*retainNonTxnLocks*/);
+                         true /*retainNonTxnLocks*/,
+                         null);
             environmentImpl.dbRename(locker, databaseName, newName);
             operationOk = true;
         } finally {
@@ -497,6 +559,47 @@ public class Environment {
         }
     }
     
+    /**
+     * Javadoc for this public method is generated via
+     * the doc templates in the doc_src directory.
+     */
+    public long truncateDatabase(Transaction txn,
+                                 String databaseName,
+                                 boolean returnCount) 
+        throws DatabaseException {
+
+        checkHandleIsValid();
+        checkEnv();
+        DatabaseUtil.checkForNullParam(databaseName, "databaseName");
+
+        Locker locker = null;
+        boolean operationOk = false;
+        long count = 0;
+        try {
+
+            /*
+             * Note: use env level isTransactional as proxy for the db
+             * isTransactional.
+             */
+            locker = LockerFactory.getWritableLocker
+                        (this, txn,
+                         environmentImpl.isTransactional(),
+                         true /*retainNonTxnLocks*/,
+                         null);
+            
+            count = environmentImpl.truncate(locker,
+                                             databaseName,
+                                             returnCount);
+
+            operationOk = true;
+        } finally {
+            if (locker != null) {
+                locker.operationEnd(operationOk);
+            }
+        }
+        return count;
+    }
+
     /**
      * Returns the current memory usage in bytes for all btrees in the
      * environmentImpl.
@@ -536,7 +639,9 @@ public class Environment {
      * Copies the handle properties out of the config properties, and
      * initializes the default transaction config.
      */
-    private void copyToHandleConfig(EnvironmentMutableConfig useConfig) {
+    private void copyToHandleConfig(EnvironmentMutableConfig useConfig,
+                                    EnvironmentConfig initStaticConfig)
+        throws DatabaseException {
 
         /*
          * Create the new objects, initialize them, then change the instance
@@ -550,6 +655,18 @@ public class Environment {
         TransactionConfig newTxnConfig =
             TransactionConfig.DEFAULT.cloneConfig();
         newTxnConfig.setNoSync(handleConfig.getTxnNoSync());
+        newTxnConfig.setWriteNoSync(handleConfig.getTxnWriteNoSync());
+        if (initStaticConfig != null) {
+            newTxnConfig.setSerializableIsolation
+                (initStaticConfig.getTxnSerializableIsolation());
+            newTxnConfig.setReadCommitted
+                (initStaticConfig.getTxnReadCommitted());
+        } else {
+            newTxnConfig.setSerializableIsolation
+                (defaultTxnConfig.getSerializableIsolation());
+            newTxnConfig.setReadCommitted
+                (defaultTxnConfig.getReadCommitted());
+        }
         this.defaultTxnConfig = newTxnConfig;
     }
 
@@ -570,27 +687,73 @@ public class Environment {
 		 "environment");
         }
 
+        if (txnConfig != null &&
+            ((txnConfig.getSerializableIsolation() &&
+              txnConfig.getReadUncommitted()) ||
+             (txnConfig.getSerializableIsolation() &&
+              txnConfig.getReadCommitted()) ||
+             (txnConfig.getReadUncommitted() &&
+              txnConfig.getReadCommitted()))) {
+            throw new IllegalArgumentException
+                ("Only one may be specified: SerializableIsolation, " +
+                 "ReadCommitted or ReadUncommitted");
+        }
+
         /*
-         * Apply txn config defaults.  We don't clone unless we have to apply
-         * the env default, since we don't hold onto a txn config reference.
+	 * Apply txn config defaults.  We don't need to clone unless we have to
+	 * apply the env default, since we don't hold onto a txn config
+	 * reference.
          */
-        TransactionConfig useConfig;
+        TransactionConfig useConfig = null;
         if (txnConfig == null) {
             useConfig = defaultTxnConfig;
         } else {
-            if (defaultTxnConfig.getNoSync() &&
-                !txnConfig.getNoSync() &&
-                !txnConfig.getSync()) {
-                useConfig = txnConfig.cloneConfig();
-                useConfig.setNoSync(true);
-            } else {
+            if (defaultTxnConfig.getNoSync() ||
+                defaultTxnConfig.getWriteNoSync()) {
+
+                /* 
+                 * The environment sync settings have been set, check if any
+                 * were set in the user's txn config. If none were set in the
+                 * user's config, apply the environment defaults
+                 */
+                if (!txnConfig.getNoSync() &&
+                    !txnConfig.getSync() &&
+                    !txnConfig.getWriteNoSync()) {
+		    useConfig = txnConfig.cloneConfig();
+                    if (defaultTxnConfig.getWriteNoSync()) {
+                        useConfig.setWriteNoSync(true);
+                    } else {
+                        useConfig.setNoSync(true);
+                    }
+                }
+            }
+
+            /* Apply isolation level default. */
+            if (!txnConfig.getSerializableIsolation() &&
+                !txnConfig.getReadCommitted() &&
+                !txnConfig.getReadUncommitted()) {
+                if (defaultTxnConfig.getSerializableIsolation()) {
+                    if (useConfig == null) {
+                        useConfig = txnConfig.cloneConfig();
+                    }
+                    useConfig.setSerializableIsolation(true);
+                } else if (defaultTxnConfig.getReadCommitted()) {
+                    if (useConfig == null) {
+                        useConfig = txnConfig.cloneConfig();
+                    }
+                    useConfig.setReadCommitted(true);
+                }
+            }
+            
+            /* No environment level defaults applied. */
+            if (useConfig == null) {
                 useConfig = txnConfig;
             }
         }
 
         Transaction txn =
             new Transaction
-	    (this, environmentImpl.txnBegin(parent, useConfig), null);
+	    (this, environmentImpl.txnBegin(parent, useConfig));
         addReferringHandle(txn);
         return txn;
     }
@@ -609,7 +772,6 @@ public class Environment {
 
         environmentImpl.invokeCheckpoint(useConfig,
                                          false, // flushAll
-                                         false, // deleteAllCleanedFiles
                                          "api");
     }
 
@@ -626,7 +788,6 @@ public class Environment {
         config.setForce(true);
         environmentImpl.invokeCheckpoint(config,
                                          true,  // flushAll
-                                         false, // deleteAllCleanedFiles
                                          "sync");
     }
 
@@ -697,7 +858,7 @@ public class Environment {
         environmentImpl.setMutableConfig(mutableConfig);
 
         /* Reset the handle config properties. */
-        copyToHandleConfig(mutableConfig);
+        copyToHandleConfig(mutableConfig, null);
     }
 
     /**
@@ -798,6 +959,25 @@ public class Environment {
         return environmentImpl.verify(useConfig, out);
     }
 
+    /**
+     * Javadoc for this public method is generated via
+     * the doc templates in the doc_src directory.
+     */
+    public Transaction getThreadTransaction()
+	throws DatabaseException {
+
+	return (Transaction) environmentImpl.getTxnManager().getTxnForThread();
+    }
+
+    /**
+     * Javadoc for this public method is generated via
+     * the doc templates in the doc_src directory.
+     */
+    public void setThreadTransaction(Transaction txn) {
+
+	environmentImpl.getTxnManager().setTxnForThread(txn);
+    }
+
     /*
      * Non public api -- helpers
      */
@@ -841,7 +1021,7 @@ public class Environment {
         return environmentImpl;
     }
     
-    private void checkHandleIsValid()
+    protected void checkHandleIsValid()
         throws DatabaseException {
 
         if (!valid) {
@@ -853,7 +1033,7 @@ public class Environment {
     /**
      * Throws if the environmentImpl is invalid.
      */
-    private void checkEnv()
+    protected void checkEnv()
         throws DatabaseException, RunRecoveryException {
 
         if (environmentImpl == null) {
