@@ -1,22 +1,27 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: SecondaryDatabase.java,v 1.32 2004/07/17 22:27:18 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: SecondaryDatabase.java,v 1.45 2006/01/04 21:49:53 linda Exp $
+ */
 
 package com.sleepycat.je;
 
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Set;
 import java.util.logging.Level;
+import java.util.logging.Logger;
 
 import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.GetMode;
 import com.sleepycat.je.dbi.PutMode;
 import com.sleepycat.je.dbi.CursorImpl.SearchMode;
 import com.sleepycat.je.txn.Locker;
-import com.sleepycat.je.utilint.Tracer;
+import com.sleepycat.je.txn.LockerFactory;
 
 /**
  * Javadoc for this public class is generated via
@@ -32,7 +37,8 @@ public class SecondaryDatabase extends Database {
     /**
      * Creates a secondary database but does not open or fully initialize it.
      */
-    SecondaryDatabase(Environment env, SecondaryConfig secConfig,
+    SecondaryDatabase(Environment env,
+		      SecondaryConfig secConfig,
                       Database primaryDatabase)
         throws DatabaseException {
 
@@ -42,27 +48,55 @@ public class SecondaryDatabase extends Database {
         if (primaryDatabase.configuration.getSortedDuplicates()) {
             throw new IllegalArgumentException
                 ("Duplicates must not be allowed for a primary database: " +
-                 primaryDatabase.getDatabaseName());
+                 primaryDatabase.getDebugName());
         }
         if (env.getEnvironmentImpl() !=
                 primaryDatabase.getEnvironment().getEnvironmentImpl()) {
             throw new IllegalArgumentException
                 ("Primary and secondary databases must be in the same" +
-                 " environment.");
+                 " environment");
+        }
+        if (secConfig.getKeyCreator() != null &&
+            secConfig.getMultiKeyCreator() != null) {
+            throw new IllegalArgumentException
+                ("secConfig.getKeyCreator() and getMultiKeyCreator() may not" +
+                 " both be non-null");
         }
         if (!primaryDatabase.configuration.getReadOnly() &&
-            secConfig.getKeyCreator() == null) {
+            secConfig.getKeyCreator() == null &&
+            secConfig.getMultiKeyCreator() == null) {
             throw new NullPointerException
-                ("secConfig and secConfig.getKeyCreator() may be null" +
-                 " only if the primary database is read-only");
+                ("secConfig and getKeyCreator()/getMultiKeyCreator()" +
+                 " may be null only if the primary database is read-only");
         }
-        if (secConfig.getForeignKeyDatabase() != null &&
-            secConfig.getForeignKeyDeleteAction() ==
-                         ForeignKeyDeleteAction.NULLIFY &&
-            secConfig.getForeignKeyNullifier() == null) {
+        if (secConfig.getForeignKeyNullifier() != null &&
+            secConfig.getForeignMultiKeyNullifier() != null) {
             throw new IllegalArgumentException
-                ("ForeignKeyNullifier must be non-null when " +
-                 " ForeignKeyDeleteAction is NULLIFY.");
+                ("secConfig.getForeignKeyNullifier() and" +
+                 " getForeignMultiKeyNullifier() may not both be non-null");
+        }
+        if (secConfig.getForeignKeyDeleteAction() ==
+                         ForeignKeyDeleteAction.NULLIFY &&
+            secConfig.getForeignKeyNullifier() == null &&
+            secConfig.getForeignMultiKeyNullifier() == null) {
+            throw new NullPointerException
+                ("ForeignKeyNullifier or ForeignMultiKeyNullifier must be" +
+                 " non-null when ForeignKeyDeleteAction is NULLIFY");
+        }
+        if (secConfig.getForeignKeyNullifier() != null &&
+            secConfig.getMultiKeyCreator() != null) {
+            throw new IllegalArgumentException
+                ("ForeignKeyNullifier may not be used with" +
+                 " SecondaryMultiKeyCreator -- use" +
+                 " ForeignMultiKeyNullifier instead");
+        }
+        if (secConfig.getForeignKeyDatabase() != null) {
+            Database foreignDb = secConfig.getForeignKeyDatabase();
+            if (foreignDb.getDatabaseImpl().getSortedDuplicates()) {
+                throw new IllegalArgumentException
+                    ("Duplicates must not be allowed for a foreign key " +
+                     " database: " + foreignDb.getDebugName());
+            }
         }
         primaryDb = primaryDatabase;
         secondaryTrigger = new SecondaryTrigger(this);
@@ -93,6 +127,15 @@ public class SecondaryDatabase extends Database {
                       DatabaseConfig dbConfig)
         throws DatabaseException {
 
+        /* Disallow one secondary associated with two different primaries. */
+        Database otherPriDb = database.findPrimaryDatabase();
+        if (otherPriDb != null &&
+            otherPriDb.getDatabaseImpl() != primaryDb.getDatabaseImpl()) {
+            throw new IllegalArgumentException
+                ("Secondary is already associated with a different primary: " +
+                 otherPriDb.getDebugName());
+        }
+
         super.initExisting(env, locker, database, dbConfig);
         init(locker);
     }
@@ -103,10 +146,7 @@ public class SecondaryDatabase extends Database {
     private void init(Locker locker)
         throws DatabaseException {
 
-        Tracer.trace(Level.FINEST,
-                     envHandle.getEnvironmentImpl(),
-                     "SecondaryDatabase open: name=" + getDatabaseName() +
-                     " primary=" + primaryDb.getDatabaseName());
+        trace(Level.FINEST, "SecondaryDatabase open");
 
         secondaryConfig = (SecondaryConfig) configuration;
 
@@ -209,6 +249,13 @@ public class SecondaryDatabase extends Database {
         return (SecondaryConfig) getConfig();
     }
 
+    /**
+     * Returns the secondary config without cloning, for internal use.
+     */
+    public SecondaryConfig getPrivateSecondaryConfig() {
+        return secondaryConfig;
+    }
+
     /**
      * Javadoc for this public method is generated via
      * the doc templates in the doc_src directory.
@@ -227,7 +274,7 @@ public class SecondaryDatabase extends Database {
                           CursorConfig cursorConfig)
         throws DatabaseException {
 
-        return new SecondaryCursor(this, txn, cursorConfig, secondaryConfig);
+        return new SecondaryCursor(this, txn, cursorConfig);
     }
 
     /**
@@ -249,9 +296,8 @@ public class SecondaryDatabase extends Database {
 
         OperationStatus commitStatus = OperationStatus.NOTFOUND;
         try {
-            locker =
-                DatabaseUtil.getWritableLocker(envHandle,
-                                               txn, isTransactional());
+            locker = LockerFactory.getWritableLocker
+                (envHandle, txn, isTransactional());
 
             /* Read the primary key (the data of a secondary). */
             cursor = new Cursor(this, locker, null);
@@ -305,10 +351,15 @@ public class SecondaryDatabase extends Database {
         checkRequiredDbState(OPEN, "Can't call SecondaryDatabase.get:");
         trace(Level.FINEST, "SecondaryDatabase.get", txn, key, null, lockMode);
 
+        CursorConfig cursorConfig = CursorConfig.DEFAULT;
+        if (lockMode == LockMode.READ_COMMITTED) {
+            cursorConfig = CursorConfig.READ_COMMITTED;
+            lockMode = null;
+        }
+
         SecondaryCursor cursor = null;
         try {
-	    cursor = new SecondaryCursor(this, txn, CursorConfig.DEFAULT,
-                                         secondaryConfig);
+	    cursor = new SecondaryCursor(this, txn, cursorConfig);
             return cursor.search(key, pKey, data, lockMode, SearchMode.SET);
         } finally {
             if (cursor != null) {
@@ -344,16 +395,21 @@ public class SecondaryDatabase extends Database {
         checkEnv();
         DatabaseUtil.checkForNullDbt(key, "key", true);
         DatabaseUtil.checkForNullDbt(pKey, "pKey", true);
-        DatabaseUtil.checkForNullDbt(data, "data", true);
+        DatabaseUtil.checkForNullDbt(data, "data", false);
         checkRequiredDbState(OPEN,
                              "Can't call SecondaryDatabase.getSearchBoth:");
         trace(Level.FINEST, "SecondaryDatabase.getSearchBoth", txn, key, data,
               lockMode);
 
+        CursorConfig cursorConfig = CursorConfig.DEFAULT;
+        if (lockMode == LockMode.READ_COMMITTED) {
+            cursorConfig = CursorConfig.READ_COMMITTED;
+            lockMode = null;
+        }
+
         SecondaryCursor cursor = null;
         try {
-	    cursor = new SecondaryCursor(this, txn, CursorConfig.DEFAULT,
-                                         secondaryConfig);
+	    cursor = new SecondaryCursor(this, txn, cursorConfig);
             return cursor.search(key, pKey, data, lockMode, SearchMode.BOTH);
         } finally {
             if (cursor != null) {
@@ -411,6 +467,7 @@ public class SecondaryDatabase extends Database {
     /**
      * Javadoc for this public method is generated via
      * the doc templates in the doc_src directory.
+     * @deprecated
      */
     public int truncate(Transaction txn, boolean countRecords)
         throws DatabaseException {
@@ -422,7 +479,7 @@ public class SecondaryDatabase extends Database {
      * Updates a single secondary when a put() or delete() is performed on
      * the primary.
      *
-     * @param txn the user transaction.
+     * @param locker the internal locker.
      *
      * @param cursor secondary cursor to use, or null if this method should
      * open and close a cursor if one is needed.
@@ -442,99 +499,189 @@ public class SecondaryDatabase extends Database {
                          DatabaseEntry newData)
         throws DatabaseException {
 
+        /*
+         * If we're updating the primary and the secondary key cannot be
+         * changed, optimize for that case by doing nothing.
+         */
+        if (secondaryConfig.getImmutableSecondaryKey() &&
+            oldData != null && newData != null) {
+            return;
+        }
+
         SecondaryKeyCreator keyCreator = secondaryConfig.getKeyCreator();
+        if (keyCreator != null) {
+            /* Each primary record may have a single secondary key. */
+            assert secondaryConfig.getMultiKeyCreator() == null;
 
-        /* Get old and new secondary keys. */
-        DatabaseEntry oldSecKey = null;
-        if (oldData != null) {
-            oldSecKey = new DatabaseEntry();
-            if (!keyCreator.createSecondaryKey(this, priKey, oldData,
-                                               oldSecKey)) {
-                oldSecKey = null;
+            /* Get old and new secondary keys. */
+            DatabaseEntry oldSecKey = null;
+            if (oldData != null) {
+                oldSecKey = new DatabaseEntry();
+                if (!keyCreator.createSecondaryKey(this, priKey, oldData,
+                                                   oldSecKey)) {
+                    oldSecKey = null;
+                }
             }
-        }
-        DatabaseEntry newSecKey = null;
-        if (newData != null) {
-            newSecKey = new DatabaseEntry();
-            if (!keyCreator.createSecondaryKey(this, priKey, newData,
-                                               newSecKey)) {
-                newSecKey = null;
+            DatabaseEntry newSecKey = null;
+            if (newData != null) {
+                newSecKey = new DatabaseEntry();
+                if (!keyCreator.createSecondaryKey(this, priKey, newData,
+                                                   newSecKey)) {
+                    newSecKey = null;
+                }
             }
-        }
 
-        /* Update secondary if old and new keys are unequal. */
-        if ((oldSecKey != null && !oldSecKey.dataEquals(newSecKey)) ||
-            (newSecKey != null && !newSecKey.dataEquals(oldSecKey))) {
+            /* Update secondary if old and new keys are unequal. */
+            if ((oldSecKey != null && !oldSecKey.equals(newSecKey)) ||
+                (newSecKey != null && !newSecKey.equals(oldSecKey))) {
 
-            boolean localCursor = (cursor == null);
-            if (localCursor) {
-                cursor = new Cursor(this, locker, null);
+                boolean localCursor = (cursor == null);
+                if (localCursor) {
+                    cursor = new Cursor(this, locker, null);
+                }
+                try {
+                    /* Delete the old key. */
+                    if (oldSecKey != null) {
+                        deleteKey(cursor, priKey, oldSecKey);
+                    }
+                    /* Insert the new key. */
+                    if (newSecKey != null) {
+                        insertKey(locker, cursor, priKey, newSecKey);
+                    }
+                } finally {
+                    if (localCursor && cursor != null) {
+                        cursor.close();
+                    }
+                }
             }
-            try {
-                /* Insert new secondary key. */
-                if (newSecKey != null) {
-                    Database foreignDb =
-                        secondaryConfig.getForeignKeyDatabase();
-                    if (foreignDb != null) {
-                        Cursor foreignCursor = null;
-                        try {
-                            foreignCursor = new Cursor(foreignDb, locker,
-                                                       null);
-                            DatabaseEntry tmpData = new DatabaseEntry();
-                            OperationStatus status =
-                                foreignCursor.search(newSecKey, tmpData,
-                                                     LockMode.DEFAULT,
-                                                     SearchMode.SET);
-                            if (status != OperationStatus.SUCCESS) {
-                                throw new DatabaseException
-                                    ("Secondary " + getDatabaseName() +
-                                     " foreign key not allowed: it is not" +
-                                     " present in the foreign database");
-                            }
-                        } finally {
-                            if (foreignCursor != null) {
-                                foreignCursor.close();
-                            }
+        } else {
+            /* Each primary record may have multiple secondary keys. */
+            SecondaryMultiKeyCreator multiKeyCreator =
+                secondaryConfig.getMultiKeyCreator();
+            assert multiKeyCreator != null;
+
+            /* Get old and new secondary keys. */
+            Set oldKeys = Collections.EMPTY_SET;
+            Set newKeys = Collections.EMPTY_SET;
+            if (oldData != null) {
+                oldKeys = new HashSet();
+                multiKeyCreator.createSecondaryKeys(this, priKey,
+                                                    oldData, oldKeys);
+            }
+            if (newData != null) {
+                newKeys = new HashSet();
+                multiKeyCreator.createSecondaryKeys(this, priKey,
+                                                    newData, newKeys);
+            }
+
+            /* Update the secondary if there is a difference. */
+            if (!oldKeys.equals(newKeys)) {
+
+                boolean localCursor = (cursor == null);
+                if (localCursor) {
+                    cursor = new Cursor(this, locker, null);
+                }
+                try {
+                    /* Delete old keys that are no longer present. */
+                    Set oldKeysCopy = oldKeys;
+                    if (oldKeys != Collections.EMPTY_SET) {
+                        oldKeysCopy = new HashSet(oldKeys);
+                        oldKeys.removeAll(newKeys);
+                        for (Iterator i = oldKeys.iterator(); i.hasNext();) {
+                            DatabaseEntry oldKey = (DatabaseEntry) i.next();
+                            deleteKey(cursor, priKey, oldKey);
                         }
                     }
-                    OperationStatus status;
-                    if (getConfig().getSortedDuplicates()) {
-                        status = cursor.putInternal(newSecKey, priKey,
-                                                    PutMode.NODUP);
-                    } else {
-                        status = cursor.putInternal(newSecKey, priKey,
-                                                    PutMode.NOOVERWRITE);
+                    /* Insert new keys that were not present before. */
+                    if (newKeys != Collections.EMPTY_SET) {
+                        newKeys.removeAll(oldKeysCopy);
+                        for (Iterator i = newKeys.iterator(); i.hasNext();) {
+                            DatabaseEntry newKey = (DatabaseEntry) i.next();
+                            insertKey(locker, cursor, priKey, newKey);
+                        }
                     }
-                    if (status != OperationStatus.SUCCESS) {
-                        throw new DatabaseException
-                            ("Could not insert secondary key in " +
-                             getDatabaseName() + ' ' + status);
+                } finally {
+                    if (localCursor && cursor != null) {
+                        cursor.close();
                     }
                 }
-
-                /* Delete old secondary key. */
-                if (oldSecKey != null) {
-                    OperationStatus status =
-                        cursor.search(oldSecKey, priKey,
-                                      LockMode.RMW,
-                                      SearchMode.BOTH);
-                    if (status == OperationStatus.SUCCESS) {
-                        cursor.deleteInternal();
-                    } else {
-                        throw new DatabaseException
-                            ("Secondary " + getDatabaseName() +
-                            " is corrupt: the primary record contains a key" +
-                            " that is not present in the secondary");
-                    }
-                }
-            } finally {
-                if (localCursor && cursor != null) {
-                    cursor.close();
-                }
             }
         }
     }
 
+    /**
+     * Deletes an old secondary key.
+     */
+    private void deleteKey(Cursor cursor,
+                           DatabaseEntry priKey,
+                           DatabaseEntry oldSecKey)
+        throws DatabaseException {
+
+        OperationStatus status =
+            cursor.search(oldSecKey, priKey,
+                          LockMode.RMW,
+                          SearchMode.BOTH);
+        if (status == OperationStatus.SUCCESS) {
+            cursor.deleteInternal();
+        } else {
+            throw new DatabaseException
+                ("Secondary " + getDebugName() +
+                " is corrupt: the primary record contains a key" +
+                " that is not present in the secondary");
+        }
+    }
+
+    /**
+     * Inserts a new secondary key.
+     */
+    private void insertKey(Locker locker,
+                           Cursor cursor,
+                           DatabaseEntry priKey,
+                           DatabaseEntry newSecKey)
+        throws DatabaseException {
+
+        /* Check for the existence of a foreign key. */
+        Database foreignDb =
+            secondaryConfig.getForeignKeyDatabase();
+        if (foreignDb != null) {
+            Cursor foreignCursor = null;
+            try {
+                foreignCursor = new Cursor(foreignDb, locker,
+                                           null);
+                DatabaseEntry tmpData = new DatabaseEntry();
+                OperationStatus status =
+                    foreignCursor.search(newSecKey, tmpData,
+                                         LockMode.DEFAULT,
+                                         SearchMode.SET);
+                if (status != OperationStatus.SUCCESS) {
+                    throw new DatabaseException
+                        ("Secondary " + getDebugName() +
+                         " foreign key not allowed: it is not" +
+                         " present in the foreign database");
+                }
+            } finally {
+                if (foreignCursor != null) {
+                    foreignCursor.close();
+                }
+            }
+        }
+
+        /* Insert the new key. */
+        OperationStatus status;
+        if (configuration.getSortedDuplicates()) {
+            status = cursor.putInternal(newSecKey, priKey,
+                                        PutMode.NODUP);
+        } else {
+            status = cursor.putInternal(newSecKey, priKey,
+                                        PutMode.NOOVERWRITE);
+        }
+        if (status != OperationStatus.SUCCESS) {
+            throw new DatabaseException
+                ("Could not insert secondary key in " +
+                 getDebugName() + ' ' + status);
+        }
+    }
+
     /**
      * Called by the ForeignKeyTrigger when a record in the foreign database is
      * deleted.
@@ -572,7 +719,7 @@ public class SecondaryDatabase extends Database {
                      * the transaction.
                      */
                     throw new DatabaseException
-                        ("Secondary " + getDatabaseName() +
+                        ("Secondary " + getDebugName() +
                          " refers to a foreign key that has been deleted" +
                          " (ForeignKeyDeleteAction.ABORT)");
 
@@ -611,10 +758,20 @@ public class SecondaryDatabase extends Database {
                         status = priCursor.search(priKey, data, LockMode.RMW,
                                                   SearchMode.SET);
                         if (status == OperationStatus.SUCCESS) {
-                            ForeignKeyNullifier nullifier =
-                                secondaryConfig.getForeignKeyNullifier();
-                            if (nullifier.nullifyForeignKey(this, data)) {
-                                priCursor.putCurrent(data);
+                            ForeignMultiKeyNullifier multiNullifier =
+                                secondaryConfig.getForeignMultiKeyNullifier();
+                            if (multiNullifier != null) {
+                                if (multiNullifier.nullifyForeignKey
+                                        (this, priKey, data, secKey)) {
+                                    priCursor.putCurrent(data);
+                                }
+                            } else {
+                                ForeignKeyNullifier nullifier =
+                                    secondaryConfig.getForeignKeyNullifier();
+                                if (nullifier.nullifyForeignKey
+                                        (this, data)) {
+                                    priCursor.putCurrent(data);
+                                }
                             }
                         } else {
                             throw secondaryCorruptException();
@@ -643,7 +800,7 @@ public class SecondaryDatabase extends Database {
         throws DatabaseException {
 
         throw new DatabaseException
-            ("Secondary " + getDatabaseName() + " is corrupt: it refers" +
+            ("Secondary " + getDebugName() + " is corrupt: it refers" +
              " to a missing key in the primary database");
     }
 
@@ -652,4 +809,24 @@ public class SecondaryDatabase extends Database {
         throw new UnsupportedOperationException
             ("Operation not allowed on a secondary");
     }
+
+    /**
+     * Send trace messages to the java.util.logger. Don't rely on the
+     * logger alone to conditionalize whether we send this message,
+     * we don't even want to construct the message if the level is
+     * not enabled.
+     */
+    void trace(Level level,
+               String methodName)
+        throws DatabaseException {
+        Logger logger = envHandle.getEnvironmentImpl().getLogger();
+        if (logger.isLoggable(level)) {
+            StringBuffer sb = new StringBuffer();
+            sb.append(methodName);
+            sb.append(" name=").append(getDebugName());
+            sb.append(" primary=").append(primaryDb.getDebugName());
+
+            logger.log(level, sb.toString());
+        }
+    }
 }
