@@ -1,10 +1,10 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2004
+ * Copyright (c) 2002-2006
  *	Sleepycat Software.  All rights reserved.
  *
- * $Id: TupleFormatTest.java,v 1.19 2004/06/04 14:48:21 mark Exp $
+ * $Id: TupleFormatTest.java,v 1.22 2006/01/03 21:56:05 bostic Exp $
  */
 
 package com.sleepycat.bind.tuple.test;
@@ -59,7 +59,7 @@ public class TupleFormatTest extends TestCase {
 
         DbTestUtil.printTestName("TupleFormatTest." + getName());
         buffer = new DatabaseEntry();
-        out = TupleBinding.newOutput();
+        out = new TupleOutput();
     }
 
     public void tearDown() {
