@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: ApiTest.java,v 1.10 2004/02/25 05:38:14 linda Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: ApiTest.java,v 1.13 2006/01/03 21:56:10 bostic Exp $
+ */
 
 package com.sleepycat.je;
 
@@ -20,10 +20,8 @@ public class ApiTest extends TestCase {
     public void testBasic()
         throws Exception {
 
-
-        Environment env = null;
         try {
-            env = new Environment(null, null);
+            new Environment(null, null);
             fail("Should get exception");
         } catch (NullPointerException e) {
             // expected exception
