@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: CursorTest.java,v 1.66 2004/04/14 01:10:50 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: CursorTest.java,v 1.76 2006/01/03 21:56:10 bostic Exp $
+ */
 
 package com.sleepycat.je;
 
@@ -15,6 +15,7 @@ import java.util.Arrays;
 
 import junit.framework.TestCase;
 
+import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.junit.JUnitThread;
 import com.sleepycat.je.util.TestUtils;
@@ -22,8 +23,11 @@ import com.sleepycat.je.util.TestUtils;
 public class CursorTest extends TestCase {
     private static final boolean DEBUG = false;
     private static final int NUM_RECS = 257;
-    /* Use a ridiculous value because we've seen extreme slowness on
-       ocicat where dbperf is often running. */
+
+    /* 
+     * Use a ridiculous value because we've seen extreme slowness on ocicat
+     * where dbperf is often running.
+     */
     private static final long LOCK_TIMEOUT = 50000000L;
 
     private static final String DUPKEY = "DUPKEY";
@@ -37,7 +41,7 @@ public class CursorTest extends TestCase {
     private volatile int sequence;
 
     public CursorTest() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
@@ -65,7 +69,7 @@ public class CursorTest extends TestCase {
     public void testGetConfig()
 	throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setTransactional(true);
         envConfig.setAllowCreate(true);
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
@@ -101,8 +105,8 @@ public class CursorTest extends TestCase {
     }
 
     /**
-     * Put some data in a database, take it out. Yank the file size down
-     * so we have many files.
+     * Put some data in a database, take it out. Yank the file size down so we
+     * have many files.
      */
     public void testBasic()
 	throws Throwable {
@@ -160,31 +164,33 @@ public class CursorTest extends TestCase {
     }
 
     /**
-     * This series of tests sets up a simple 2 BIN tree with a specific
-     * set of elements (see setupDatabaseAndEnv()).  It creates two threads.
+     * This series of tests sets up a simple 2 BIN tree with a specific set of
+     * elements (see setupDatabaseAndEnv()).  It creates two threads.
      *
-     * Thread 1 positions a cursor on an element on the edge of a BIN
-     * (either the last element on the left BIN or the first element on
-     * the right BIN).  This locks that element.  It throws control to
-     * thread 2.
+     * Thread 1 positions a cursor on an element on the edge of a BIN (either
+     * the last element on the left BIN or the first element on the right BIN).
+     * This locks that element.  It throws control to thread 2.
      *
      * Thread 2 positions a cursor on the adjacent element on the other BIN
-     * (either the first element on the right BIN or the last element on
-     * the left BIN, resp.)  It throws control to thread 1.  After it signals
+     * (either the first element on the right BIN or the last element on the
+     * left BIN, resp.)  It throws control to thread 1.  After it signals
      * thread 1 to continue, thread 2 does either a getNext or getPrev.  This
      * should block because thread 1 has the next/prev element locked.
      *
      * Thread 1 then waits a short time (250ms) so that thread 2 can execute
      * the getNext/getPrev.  Thread 1 then inserts or deletes the "phantom
-     * element" right in between the cursors that were set up in the
-     * previous two steps, sleeps a second, and either commits or aborts.
+     * element" right in between the cursors that were set up in the previous
+     * two steps, sleeps a second, and either commits or aborts.
      *
-     * Thread 2 will then return from the getNext/getPrev.  The returned
-     * key from the getNext/getPrev is then verified.
+     * Thread 2 will then return from the getNext/getPrev.  The returned key
+     * from the getNext/getPrev is then verified.
+     *
+     * The Serializable isolation level is not used for either thread so as to
+     * allow phantoms; otherwise, this test would deadlock.
      *
      * These parameters are all configured through a PhantomTestConfiguration
-     * instance passed to phantomWorker which has the template for the
-     * steps described above.
+     * instance passed to phantomWorker which has the template for the steps
+     * described above.
      */
 
     /**
@@ -301,8 +307,8 @@ public class CursorTest extends TestCase {
     }
 
     /**
-     * Phantom Dup test inserting and committing a phantom while
-     * doing a getNext.
+     * Phantom Dup test inserting and committing a phantom while doing a
+     * getNext.
      */
     public void testPhantomDupInsertGetNextCommit()
 	throws Throwable {
@@ -333,8 +339,8 @@ public class CursorTest extends TestCase {
     }
 
     /**
-     * Phantom Dup test inserting and committing a phantom while
-     * doing a getPrev.
+     * Phantom Dup test inserting and committing a phantom while doing a
+     * getPrev.
      */
     public void testPhantomDupInsertGetPrevCommit()
 	throws Throwable {
@@ -429,11 +435,10 @@ public class CursorTest extends TestCase {
 		    env.beginTransaction(null, TransactionConfig.DEFAULT);
 
 		/*
-		 * Delete the first entry in the second bin so that we
-		 * can reinsert it in tester1 and have it be the first
-		 * entry in that bin.  If we left F and then tried to
-		 * insert something to the left of F, it would end up
-		 * in the first bin.
+		 * Delete the first entry in the second bin so that we can
+		 * reinsert it in tester1 and have it be the first entry in
+		 * that bin.  If we left F and then tried to insert something
+		 * to the left of F, it would end up in the first bin.
 		 */
 		assertEquals(OperationStatus.SUCCESS,
 			     db.delete(txnDel,
@@ -449,8 +454,7 @@ public class CursorTest extends TestCase {
 			Cursor cursor = null;
 			try {
 			    Transaction txn1 =
-				env.beginTransaction
-				(null, TransactionConfig.DEFAULT);
+				env.beginTransaction(null, null);
 			    cursor = db.openCursor(txn1, CursorConfig.DEFAULT);
 			    OperationStatus status =
 				cursor.getSearchKey
@@ -466,11 +470,11 @@ public class CursorTest extends TestCase {
 			    }
 
 			    /*
-			     * Since we can't increment sequence when
-			     * tester2 blocks on the getNext call, all
-			     * we can do is bump sequence right before
-			     * the getNext, and then wait a little in
-			     * this thread for tester2 to block.
+			     * Since we can't increment sequence when tester2
+			     * blocks on the getNext call, all we can do is
+			     * bump sequence right before the getNext, and then
+			     * wait a little in this thread for tester2 to
+			     * block.
 			     */
 			    try {
 				Thread.sleep(250);
@@ -522,8 +526,7 @@ public class CursorTest extends TestCase {
 			Cursor cursor = null;
 			try {
 			    Transaction txn2 =
-				env.beginTransaction
-				(null, TransactionConfig. DEFAULT);
+				env.beginTransaction(null, null);
 			    txn2.setLockTimeout(LOCK_TIMEOUT);
 			    cursor = db.openCursor(txn2, CursorConfig.DEFAULT);
 
@@ -561,7 +564,6 @@ public class CursorTest extends TestCase {
 				System.out.println("t2 caught " + DBE);
 			    }
 			    assertEquals(3, sequence);
-			    byte[] data = nextKey.getData();
 			    assertEquals(config.expectedResult,
 					 new String(nextKey.getData()));
 			    cursor.close();
@@ -605,11 +607,10 @@ public class CursorTest extends TestCase {
 		cursor = db.openCursor(txnDel, CursorConfig.DEFAULT);
 
 		/*
-		 * Delete the first entry in the second bin so that we
-		 * can reinsert it in tester1 and have it be the first
-		 * entry in that bin.  If we left F and then tried to
-		 * insert something to the left of F, it would end up
-		 * in the first bin.
+		 * Delete the first entry in the second bin so that we can
+		 * reinsert it in tester1 and have it be the first entry in
+		 * that bin.  If we left F and then tried to insert something
+		 * to the left of F, it would end up in the first bin.
 		 */
 		assertEquals(OperationStatus.SUCCESS, cursor.getSearchBoth
 			     (new DatabaseEntry(DUPKEY.getBytes()),
@@ -630,8 +631,7 @@ public class CursorTest extends TestCase {
 			Cursor c = null;
 			try {
 			    Transaction txn1 =
-				env.beginTransaction
-				(null, TransactionConfig.DEFAULT);
+				env.beginTransaction(null, null);
 			    cursor = db.openCursor(txn1, CursorConfig.DEFAULT);
 			    OperationStatus status =
 				cursor.getSearchBoth
@@ -649,11 +649,11 @@ public class CursorTest extends TestCase {
 			    }
 
 			    /*
-			     * Since we can't increment sequence when
-			     * tester2 blocks on the getNext call, all
-			     * we can do is bump sequence right before
-			     * the getNext, and then wait a little in
-			     * this thread for tester2 to block.
+			     * Since we can't increment sequence when tester2
+			     * blocks on the getNext call, all we can do is
+			     * bump sequence right before the getNext, and then
+			     * wait a little in this thread for tester2 to
+			     * block.
 			     */
 			    try {
 				Thread.sleep(250);
@@ -714,8 +714,7 @@ public class CursorTest extends TestCase {
 			Cursor cursor = null;
 			try {
 			    Transaction txn2 =
-				env.beginTransaction
-				(null, TransactionConfig.DEFAULT);
+				env.beginTransaction(null, null);
 			    txn2.setLockTimeout(LOCK_TIMEOUT);
 			    cursor = db.openCursor(txn2, CursorConfig.DEFAULT);
 
@@ -784,16 +783,23 @@ public class CursorTest extends TestCase {
     }
 
     /**
-     * Sets up a small database with a tree containing 2 bins, one with
-     * A, B, and C, and the other with F, G, H, and I.
+     * Sets up a small database with a tree containing 2 bins, one with A, B,
+     * and C, and the other with F, G, H, and I.
      */
     private void setupDatabaseAndEnv(boolean writeAsDuplicateData)
 	throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+
+        /* RepeatableRead isolation is required by this test. */
+        TestUtils.clearIsolationLevel(envConfig);
+
+	DbInternal.disableParameterValidation(envConfig);
         envConfig.setTransactional(true);
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(),
                                  "6");
+        envConfig.setConfigParam(EnvironmentParams.NODE_MAX_DUPTREE.getName(),
+                                 "6");
         envConfig.setConfigParam(EnvironmentParams.LOG_FILE_MAX.getName(),
                                  "1024");
         envConfig.setConfigParam(EnvironmentParams.ENV_CHECK_LEAKS.getName(),
@@ -845,8 +851,12 @@ public class CursorTest extends TestCase {
     private void insertMultiDb(int numDbs)
 	throws DatabaseException {
 
-        // Make an environment and open it
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+
+        /* RepeatableRead isolation is required by this test. */
+        TestUtils.clearIsolationLevel(envConfig);
+
+	DbInternal.disableParameterValidation(envConfig);
         envConfig.setTransactional(true);
         envConfig.setConfigParam
 	    (EnvironmentParams.LOG_FILE_MAX.getName(), "1024");
@@ -854,11 +864,12 @@ public class CursorTest extends TestCase {
 	    (EnvironmentParams.ENV_CHECK_LEAKS.getName(), "true");
 	envConfig.setConfigParam
 	    (EnvironmentParams.NODE_MAX.getName(), "6");
+        envConfig.setConfigParam
+	    (EnvironmentParams.NODE_MAX_DUPTREE.getName(), "6");
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
         envConfig.setAllowCreate(true);
         Environment env = new Environment(envHome, envConfig);
 
-        // Make a db and open it
         Database[] myDb = new Database[numDbs];
         Cursor[] cursor = new Cursor[numDbs];
         Transaction txn =
@@ -871,11 +882,10 @@ public class CursorTest extends TestCase {
         for (int i = 0; i < numDbs; i++) {
             myDb[i] = env.openDatabase(txn, "testDB" + i, dbConfig);
 
-            // Setup cursors
             cursor[i] = myDb[i].openCursor(txn, CursorConfig.DEFAULT);
         }
 
-        // Insert data in a round robin fashion to spread over log
+        /* Insert data in a round robin fashion to spread over log. */
         DatabaseEntry key = new DatabaseEntry();
         DatabaseEntry data = new DatabaseEntry();
         for (int i = NUM_RECS; i > 0; i--) {
@@ -890,18 +900,16 @@ public class CursorTest extends TestCase {
             }
         }
 
-        // close the environment
         for (int i = 0; i < numDbs; i++) {
             cursor[i].close();
             myDb[i].close();
         }
         txn.commit();
 
-        env.verify(null, System.err);
+        assertTrue(env.verify(null, System.err));
         env.close();
         env = null;
 
-        // open it again
         envConfig.setAllowCreate(false);
         env = new Environment(envHome, envConfig);
 
@@ -912,10 +920,9 @@ public class CursorTest extends TestCase {
          */
         env.cleanLog();
 
-        // Run verify
         env.verify(null, System.err);
 
-        // Check each db in turn, using null transactions.
+        /* Check each db in turn, using null transactions. */
         dbConfig.setTransactional(false);
         dbConfig.setAllowCreate(false);
         for (int d = 0; d < numDbs; d++) {
