@@ -1,17 +1,18 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: DatabaseConfigTest.java,v 1.9 2004/05/10 18:44:51 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: DatabaseConfigTest.java,v 1.19 2006/01/03 21:56:11 bostic Exp $
+ */
 
 package com.sleepycat.je;
 
 import java.io.File;
 import java.io.IOException;
 import java.util.Comparator;
+import java.util.Set;
 
 import junit.framework.TestCase;
 
@@ -27,7 +28,7 @@ public class DatabaseConfigTest extends TestCase {
     private Environment env;
 
     public DatabaseConfigTest() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
@@ -40,7 +41,7 @@ public class DatabaseConfigTest extends TestCase {
         throws Exception {
 
         try {
-            /* Close in case we hit an exception and didn't close */
+            /* Close in case we hit an exception and didn't close. */
             if (env != null) {
             	env.close();
             } 
@@ -52,14 +53,14 @@ public class DatabaseConfigTest extends TestCase {
     }
 
     /**
-     * Test that we can retrieve a database configuration and that it
-     * clones its configuration appropriately.
+     * Test that we can retrieve a database configuration and that it clones
+     * its configuration appropriately.
      */
     public void testConfig()
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setAllowCreate(true);
             env = new Environment(envHome, envConfig);
 
@@ -75,15 +76,15 @@ public class DatabaseConfigTest extends TestCase {
             dbConfigA.setAllowCreate(false);
             DatabaseConfig getConfig1 = dbA.getConfig();
             assertEquals(true, getConfig1.getAllowCreate());
-            assertEquals(false,getConfig1.getSortedDuplicates());
+            assertEquals(false, getConfig1.getSortedDuplicates());
 
             /* 
-             * Change the retrieved config, ought to have no effect on
-             * what the Database is storing.
+             * Change the retrieved config, ought to have no effect on what the
+             * Database is storing.
              */
             getConfig1.setSortedDuplicates(true);
             DatabaseConfig getConfig2 = dbA.getConfig();
-            assertEquals(false,getConfig2.getSortedDuplicates());
+            assertEquals(false, getConfig2.getSortedDuplicates());
             
             dbA.close();
             env.close();
@@ -93,6 +94,163 @@ public class DatabaseConfigTest extends TestCase {
         }
     }
 
+    public void testConfigMatching()
+        throws Throwable {
+
+        try {
+	    /* DatabaseConfig matching. */
+
+            DatabaseConfig confA = new DatabaseConfig();
+            DatabaseConfig confB = new DatabaseConfig();
+
+	    try {
+		confA.validate(confB);
+	    } catch (Exception E) {
+		fail("expected valid match");
+	    }
+
+	    try {
+		confB.validate(confA);
+	    } catch (Exception E) {
+		fail("expected valid match");
+	    }
+
+	    try {
+		confA.validate(null); // uses the DEFAULT config
+	    } catch (Exception E) {
+		fail("expected valid match");
+	    }
+
+            confA.setReadOnly(true);
+	    try {
+		confA.validate(confB);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+
+            confA.setReadOnly(false);
+	    confA.setSortedDuplicates(true);
+	    try {
+		confA.validate(confB);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confA.setSortedDuplicates(false);
+
+            confA.setOverrideBtreeComparator(true);
+	    confA.setBtreeComparator(TestComparator.class);
+            confB.setOverrideBtreeComparator(true);
+	    confB.setBtreeComparator(TestComparator2.class);
+	    try {
+		confA.validate(confB);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confA.setBtreeComparator(null);
+            confA.setOverrideBtreeComparator(false);
+	    confB.setBtreeComparator(null);
+            confB.setOverrideBtreeComparator(false);
+
+            confA.setOverrideDuplicateComparator(true);
+	    confA.setDuplicateComparator(TestComparator.class);
+            confB.setOverrideDuplicateComparator(true);
+	    confB.setDuplicateComparator(TestComparator2.class);
+	    try {
+		confA.validate(confB);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+
+	    /* SecondaryConfig matching. */
+
+            SecondaryConfig confC = new SecondaryConfig();
+            SecondaryConfig confD = new SecondaryConfig();
+	    confC.setKeyCreator(new SecKeyCreator1());
+	    confD.setKeyCreator(new SecKeyCreator1());
+
+	    try {
+		confC.validate(confD);
+	    } catch (Exception E) {
+		E.printStackTrace();
+		fail("expected valid match");
+	    }
+
+	    try {
+		confD.validate(confC);
+	    } catch (Exception E) {
+		fail("expected valid match");
+	    }
+
+	    try {
+		confC.validate(null);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+
+	    confD.setKeyCreator(new SecKeyCreator2());
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confD.setKeyCreator(new SecKeyCreator1());
+
+	    confD.setMultiKeyCreator(new SecMultiKeyCreator1());
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confD.setMultiKeyCreator(null);
+
+	    confC.setForeignKeyDeleteAction(ForeignKeyDeleteAction.NULLIFY);
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confC.setForeignKeyDeleteAction(ForeignKeyDeleteAction.ABORT);
+
+	    confC.setForeignKeyNullifier(new ForeignKeyNullifier1());
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confC.setForeignKeyNullifier(null);
+
+	    confC.setForeignMultiKeyNullifier(new ForeignMultiKeyNullifier1());
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confC.setForeignMultiKeyNullifier(null);
+
+	    confC.setImmutableSecondaryKey(true);
+	    try {
+		confC.validate(confD);
+		fail("expected exception");
+	    } catch (DatabaseException E) {
+		// ok
+	    }
+	    confC.setImmutableSecondaryKey(false);
+        } catch (Throwable t) {
+            t.printStackTrace();
+            throw t;
+        }
+    }
+
     /**
      * Make sure we can instantiate a comparator at the time it's set.
      */
@@ -133,22 +291,21 @@ public class DatabaseConfigTest extends TestCase {
             throw t;
         }
     }
-    
 
     /**
-     * Test that any conflicts between configuration object settings
-     * and the underlying impl object are detected.
+     * Test that any conflicts between configuration object settings and the
+     * underlying impl object are detected.
      */
     public void testConfigConfict() 
         throws Throwable {
         
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setAllowCreate(true);
             env = new Environment(envHome, envConfig);
 
             /*
-             *  Test conflicts of duplicate allowed configuration
+             * Test conflicts of duplicate allowed configuration.
              */
 
             /* 1a. Create allowing duplicates. */
@@ -161,7 +318,6 @@ public class DatabaseConfigTest extends TestCase {
             DatabaseConfig secondConfig = new DatabaseConfig();
             secondConfig.setSortedDuplicates(false);
             try {
-                Database secondhandle =
                     env.openDatabase(null, "fooDups", secondConfig);
                 fail("Conflict in duplicates allowed should be detected.");
             } catch (DatabaseException e) {
@@ -176,7 +332,6 @@ public class DatabaseConfigTest extends TestCase {
             /* 2b. Try to open w/duplicates. */
             secondConfig.setSortedDuplicates(true);
             try {
-                Database secondhandle =
                     env.openDatabase(null, "fooDups", secondConfig);
                 fail("Conflict in duplicates allowed should be detected.");
             } catch (DatabaseException e) {
@@ -197,7 +352,7 @@ public class DatabaseConfigTest extends TestCase {
 
             /* Open the environment in read only mode. */
             env.close();
-            envConfig = new EnvironmentConfig();
+            envConfig = TestUtils.initEnvConfig();
             envConfig.setReadOnly(true);
             env = new Environment(envHome, envConfig);
             
@@ -206,9 +361,9 @@ public class DatabaseConfigTest extends TestCase {
                                         readOnlyConfig);
             roHandle.close();
 
-            /* Open a read/write database handle, should succeed */
+            /* Open a read/write database handle, should succeed. */
             try {
-                Database rwHandle = env.openDatabase(null, "fooDups", null);
+                env.openDatabase(null, "fooDups", null);
                 fail("Should not be able to open read/write");
             } catch (DatabaseException e) {
                 if (DEBUG) {
@@ -217,7 +372,6 @@ public class DatabaseConfigTest extends TestCase {
             }
             env.close();
 
-
             /*
              * Check comparator changes.
              */
@@ -270,13 +424,12 @@ public class DatabaseConfigTest extends TestCase {
         }
     }
 
-
     public void testIsTransactional()
         throws Throwable {
 
         try {
             /* Open environment in transactional mode.*/
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env = new Environment(envHome, envConfig);
@@ -349,7 +502,7 @@ public class DatabaseConfigTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env = new Environment(envHome, envConfig);
@@ -432,7 +585,7 @@ public class DatabaseConfigTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
 
             /* 
              * Make sure that the database keeps its own copy of the
@@ -450,7 +603,7 @@ public class DatabaseConfigTest extends TestCase {
                                             
             /* Should not succeed, because the database exists. */
             try {
-                Database dbB = env.openDatabase(null, "foo", dbConfig);
+                env.openDatabase(null, "foo", dbConfig);
                 fail("Database already exists");
             } catch (DatabaseException e) {
             }
@@ -461,7 +614,6 @@ public class DatabaseConfigTest extends TestCase {
         }
     }
 
-
     /*
      * This Comparator can't be instantiated because it's private and not
      * static.
@@ -476,8 +628,8 @@ public class DatabaseConfigTest extends TestCase {
     }
 
     /*
-     * This Comparator can't be instantiated because it doesn't have 
-     * zero parameter constructor.
+     * This Comparator can't be instantiated because it doesn't have zero
+     * parameter constructor.
      */
     public static class BadComparator2 implements Comparator {
         public BadComparator2(int i) {
@@ -499,4 +651,90 @@ public class DatabaseConfigTest extends TestCase {
             return 0;
         }
     }
+
+    /*
+     * OK comparator for setting comparators.
+     */
+    public static class TestComparator2 implements Comparator {
+        public TestComparator2() {
+        }
+
+        public int compare(Object o1, Object o2) {
+            return 0;
+        }
+    }
+
+    public static class SecKeyCreator1 implements SecondaryKeyCreator {
+	public boolean createSecondaryKey(SecondaryDatabase secondary,
+					  DatabaseEntry key,
+					  DatabaseEntry data,
+					  DatabaseEntry result)
+	    throws DatabaseException {
+
+	    return true;
+	}
+
+	public boolean equals(Object o) {
+	    if (o == null) {
+		return false;
+	    }
+	    return (o.getClass() == getClass());
+	}
+    }
+
+    public static class SecKeyCreator2 implements SecondaryKeyCreator {
+	public boolean createSecondaryKey(SecondaryDatabase secondary,
+					  DatabaseEntry key,
+					  DatabaseEntry data,
+					  DatabaseEntry result)
+	    throws DatabaseException {
+
+	    return true;
+	}
+
+	public boolean equals(Object o) {
+	    if (o == null) {
+		return false;
+	    }
+	    return (o.getClass() == getClass());
+	}
+    }
+
+    public static class SecMultiKeyCreator1
+        implements SecondaryMultiKeyCreator {
+	public void createSecondaryKeys(SecondaryDatabase secondary,
+					DatabaseEntry key,
+					DatabaseEntry data,
+					Set results)
+	    throws DatabaseException {
+	}
+
+	public boolean equals(Object o) {
+	    if (o == null) {
+		return false;
+	    }
+	    return (o.getClass() == getClass());
+	}
+    }
+
+    public static class ForeignKeyNullifier1 implements ForeignKeyNullifier {
+	public boolean nullifyForeignKey(SecondaryDatabase secondary,
+					 DatabaseEntry data)
+	    throws DatabaseException {
+
+	    return true;
+	}
+    }
+
+    public static class ForeignMultiKeyNullifier1
+        implements ForeignMultiKeyNullifier {
+	public boolean nullifyForeignKey(SecondaryDatabase secondary,
+					 DatabaseEntry key,
+					 DatabaseEntry data,
+					 DatabaseEntry secKey)
+	    throws DatabaseException {
+
+	    return true;
+	}
+    }
 }
