@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: DatabaseTest.java,v 1.79 2004/08/13 03:18:07 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: DatabaseTest.java,v 1.98 2006/01/03 21:56:11 bostic Exp $
+ */
 
 package com.sleepycat.je;
 
@@ -16,7 +16,9 @@ import junit.framework.TestCase;
 
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.INList;
+import com.sleepycat.je.dbi.MemoryBudget;
 import com.sleepycat.je.junit.JUnitThread;
+import com.sleepycat.je.log.LogUtils;
 import com.sleepycat.je.tree.Key;
 import com.sleepycat.je.util.TestUtils;
 
@@ -31,7 +33,7 @@ public class DatabaseTest extends TestCase {
     private Environment env;
 
     public DatabaseTest() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
@@ -69,7 +71,7 @@ public class DatabaseTest extends TestCase {
 
         try {
 
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             txnalEnv = new Environment(envHome, envConfig);
@@ -121,14 +123,69 @@ public class DatabaseTest extends TestCase {
 			     myDb.put(txn, key, data));
                 assertEquals(OperationStatus.SUCCESS,
 			     myDb.get(txn, key, getData, LockMode.DEFAULT));
-                assertEquals(0, Key.compareByteArray(data.getData(),
-                                                     getData.getData()));
+                assertEquals(0, Key.compareKeys(data.getData(),
+						getData.getData(), null));
                 assertEquals(OperationStatus.SUCCESS,
 			     myDb.put(txn, key, data));
                 assertEquals(OperationStatus.SUCCESS, myDb.getSearchBoth
                              (txn, key, getData, LockMode.DEFAULT));
-                assertEquals(0, Key.compareByteArray(data.getData(),
-                                                     getData.getData()));
+                assertEquals(0, Key.compareKeys(data.getData(),
+						getData.getData(), null));
+            }
+	    txn.commit();
+            myDb.close();
+            env.close();
+        } catch (Throwable t) {
+            t.printStackTrace();
+            throw t;
+        }
+    }
+
+    /*
+     * Test that zero length data always returns the same (static) byte[].
+     */
+    public void testZeroLengthData()
+        throws Throwable {
+
+        try {
+            Database myDb = initEnvAndDb(true, false, true, null);
+            DatabaseEntry key = new DatabaseEntry();
+            DatabaseEntry data = new DatabaseEntry();
+            DatabaseEntry getData = new DatabaseEntry();
+	    Transaction txn = env.beginTransaction(null, null);
+	    byte[] appZLBA = new byte[0];
+            for (int i = NUM_RECS; i > 0; i--) {
+                key.setData(TestUtils.getTestArray(i));
+                data.setData(appZLBA);
+                assertEquals(OperationStatus.SUCCESS,
+			     myDb.put(txn, key, data));
+                assertEquals(OperationStatus.SUCCESS,
+			     myDb.get(txn, key, getData, LockMode.DEFAULT));
+		assertFalse(getData.getData() == appZLBA);
+		assertTrue(getData.getData() ==
+			   LogUtils.ZERO_LENGTH_BYTE_ARRAY);
+                assertEquals(0, Key.compareKeys(data.getData(),
+						getData.getData(), null));
+            }
+	    txn.commit();
+            myDb.close();
+            env.close();
+
+	    /*
+	     * Read back from the log.
+	     */
+
+            myDb = initEnvAndDb(true, false, true, null);
+            key = new DatabaseEntry();
+            data = new DatabaseEntry();
+            getData = new DatabaseEntry();
+	    txn = env.beginTransaction(null, null);
+            for (int i = NUM_RECS; i > 0; i--) {
+                key.setData(TestUtils.getTestArray(i));
+                assertEquals(OperationStatus.SUCCESS,
+			     myDb.get(txn, key, getData, LockMode.DEFAULT));
+		assertTrue(getData.getData() ==
+			   LogUtils.ZERO_LENGTH_BYTE_ARRAY);
             }
 	    txn.commit();
             myDb.close();
@@ -550,7 +607,7 @@ public class DatabaseTest extends TestCase {
     /**
      * Exercise the preload method, which warms up the cache.
      */
-    public void testPreload()
+    public void testPreloadByteLimit()
         throws Throwable {
 
         /* Set up a test db */
@@ -572,7 +629,8 @@ public class DatabaseTest extends TestCase {
 	txn.commit();
         myDb.close();
         env.close();
-        myDb = initEnvAndDb(true, false, true, "40000");
+        myDb = initEnvAndDb
+	    (true, false, true, MemoryBudget.MIN_MAX_MEMORY_SIZE_STRING);
 
         /* 
          * Do two evictions, because the first eviction will only strip
@@ -587,7 +645,12 @@ public class DatabaseTest extends TestCase {
         long postEvictResidentNodes = inlist.getSize();
 
         /* Now preload, but not up to the full size of the db */
-        myDb.preload(100000);
+	PreloadConfig conf = new PreloadConfig();
+	conf.setMaxBytes(92000);
+        PreloadStats stats =
+	    myDb.preload(conf); /* Cache size is currently 92160. */
+
+	assertEquals(PreloadStatus.FILLED_CACHE, stats.status);
 
         long postPreloadMemUsage = env.getMemoryUsage();
         long postPreloadResidentNodes = inlist.getSize();
@@ -623,7 +686,9 @@ public class DatabaseTest extends TestCase {
 
         assertTrue(postEvictMemUsage < postCreateMemUsage);
         assertTrue(postEvictMemUsage < postPreloadMemUsage);
-        assertTrue(postPreloadMemUsage < postIterationMemUsage);
+        assertTrue("postPreloadMemUsage=" + postPreloadMemUsage +
+                   " postIterationMemUsage=" + postIterationMemUsage,
+                   postPreloadMemUsage < postIterationMemUsage);
         assertTrue(postIterationMemUsage <= postCreateMemUsage);
         assertTrue(postEvictResidentNodes < postPreloadResidentNodes);
         //assertEquals(postCreateResidentNodes, postIterationResidentNodes);
@@ -633,6 +698,157 @@ public class DatabaseTest extends TestCase {
         env.close();
     }
 
+    public void testPreloadTimeLimit()
+        throws Throwable {
+
+        /* Set up a test db */
+        Database myDb = initEnvAndDb(false, false, true, null);
+        DatabaseEntry key = new DatabaseEntry();
+        DatabaseEntry data = new DatabaseEntry();
+	Transaction txn = env.beginTransaction(null, null);
+        for (int i = 25000; i > 0; i--) {
+            key.setData(TestUtils.getTestArray(i));
+            data.setData(new byte[1]);
+            assertEquals(OperationStatus.SUCCESS,
+			 myDb.put(txn, key, data));
+        }
+
+        /* Recover the database, restart w/no evictor. */
+        long postCreateMemUsage = env.getMemoryUsage();
+        INList inlist = env.getEnvironmentImpl().getInMemoryINs();
+        long postCreateResidentNodes = inlist.getSize();
+	txn.commit();
+        myDb.close();
+        env.close();
+        myDb = initEnvAndDb(true, false, true, null);
+
+        /* 
+         * Do two evictions, because the first eviction will only strip
+         * LNs. We need to actually evict BINS because preload only pulls in
+         * IN/BINs
+         */
+        env.evictMemory(); // first eviction strips LNS.
+        env.evictMemory(); // second one will evict BINS
+
+        long postEvictMemUsage = env.getMemoryUsage();
+        inlist = env.getEnvironmentImpl().getInMemoryINs(); // re-get inList
+        long postEvictResidentNodes = inlist.getSize();
+
+        /* Now preload, but not up to the full size of the db */
+	PreloadConfig conf = new PreloadConfig();
+	conf.setMaxMillisecs(50);
+        PreloadStats stats = myDb.preload(conf);
+	assertEquals(PreloadStatus.EXCEEDED_TIME, stats.status);
+
+        long postPreloadMemUsage = env.getMemoryUsage();
+        long postPreloadResidentNodes = inlist.getSize();
+
+        /* Now iterate to get everything back into memory */
+        Cursor cursor = myDb.openCursor(null, null);
+        int count = 0;
+        OperationStatus status = cursor.getFirst(key, data, LockMode.DEFAULT);
+        while (status == OperationStatus.SUCCESS) {
+            count++;
+            status = cursor.getNext(key, data, LockMode.DEFAULT);
+        }
+	cursor.close();
+
+        long postIterationMemUsage = env.getMemoryUsage();
+        long postIterationResidentNodes = inlist.getSize();
+
+        if (DEBUG) {
+            System.out.println("postCreateMemUsage: " + postCreateMemUsage);
+            System.out.println("postEvictMemUsage: " + postEvictMemUsage);
+            System.out.println("postPreloadMemUsage: " + postPreloadMemUsage);
+            System.out.println("postIterationMemUsage: " +
+                               postIterationMemUsage);
+            System.out.println("postEvictResidentNodes: " +
+                               postEvictResidentNodes);
+            System.out.println("postPreloadResidentNodes: " +
+                               postPreloadResidentNodes);
+            System.out.println("postIterationResidentNodes: " +
+                               postIterationResidentNodes);
+            System.out.println("postCreateResidentNodes: " +
+                               postCreateResidentNodes);
+        }
+
+        assertTrue(postEvictMemUsage < postCreateMemUsage);
+        assertTrue(postEvictMemUsage < postPreloadMemUsage);
+        assertTrue("postPreloadMemUsage=" + postPreloadMemUsage +
+                   " postIterationMemUsage=" + postIterationMemUsage,
+                   postPreloadMemUsage < postIterationMemUsage);
+        assertTrue(postIterationMemUsage <= postCreateMemUsage);
+        assertTrue(postEvictResidentNodes < postPreloadResidentNodes);
+        //assertEquals(postCreateResidentNodes, postIterationResidentNodes);
+        assertTrue(postCreateResidentNodes >= postIterationResidentNodes);
+
+        myDb.close();
+        env.close();
+    }
+
+    /**
+     * Load the entire database with preload.
+     */
+    public void testPreloadEntireDatabase()
+        throws Throwable {
+
+        /* Create a test db with one record */
+        Database myDb = initEnvAndDb(false, false, false, null);
+        DatabaseEntry key = new DatabaseEntry();
+        DatabaseEntry data = new DatabaseEntry();
+        key.setData(TestUtils.getTestArray(0));
+        data.setData(TestUtils.getTestArray(0));
+        assertEquals(OperationStatus.SUCCESS, myDb.put(null, key, data));
+
+        /* Close and reopen. */
+        myDb.close();
+        env.close();
+        myDb = initEnvAndDb(false, false, false, null);
+
+        /*
+         * Preload the entire database.  In JE 2.0.54 this would cause a
+         * NullPointerException.
+         */
+	PreloadConfig conf = new PreloadConfig();
+	conf.setMaxBytes(100000);
+        myDb.preload(conf);
+
+        myDb.close();
+        env.close();
+    }
+
+    /**
+     * Test preload(N, 0) where N > cache size (throws IllArgException).
+     */
+    public void testPreloadBytesExceedsCache()
+        throws Throwable {
+
+        /* Create a test db with one record */
+        Database myDb = initEnvAndDb(false, false, false, "100000");
+        DatabaseEntry key = new DatabaseEntry();
+        DatabaseEntry data = new DatabaseEntry();
+        key.setData(TestUtils.getTestArray(0));
+        data.setData(TestUtils.getTestArray(0));
+        assertEquals(OperationStatus.SUCCESS, myDb.put(null, key, data));
+
+        /* Close and reopen. */
+        myDb.close();
+        env.close();
+        myDb = initEnvAndDb(false, false, false, "100000");
+
+	/* maxBytes > cache size.  Should throw IllegalArgumentException. */
+	try {
+	    PreloadConfig conf = new PreloadConfig();
+	    conf.setMaxBytes(100001);
+	    myDb.preload(conf);
+	    fail("should have thrown IAE");
+	} catch (IllegalArgumentException IAE) {
+	}
+
+        myDb.close();
+        env.close();
+    }
+
     public void testDbClose()
         throws Throwable {
 
@@ -650,7 +866,8 @@ public class DatabaseTest extends TestCase {
 
 	/* Create a cursor, use it, then close db without closing cursor. */
         Cursor cursor = myDb.openCursor(txn, null);
-        OperationStatus status = cursor.getFirst(key, data, LockMode.DEFAULT);
+        assertEquals(OperationStatus.SUCCESS,
+        	     cursor.getFirst(key, data, LockMode.DEFAULT));
 
 	try {
 	    myDb.close();
@@ -664,7 +881,7 @@ public class DatabaseTest extends TestCase {
     public void testDbCloseUnopenedDb()
 	throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setTransactional(true);
         envConfig.setAllowCreate(true);
         env = new Environment(envHome, envConfig);
@@ -693,6 +910,40 @@ public class DatabaseTest extends TestCase {
         }
     }
 
+    public void testBufferOverflowingPut()
+        throws Throwable {
+
+        try {
+
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+            envConfig.setTransactional(true);
+            //envConfig.setConfigParam("je.log.totalBufferBytes", "5000");
+            envConfig.setAllowCreate(true);
+            env = new Environment(envHome, envConfig);
+
+            DatabaseConfig dbConfig = new DatabaseConfig();
+            dbConfig.setSortedDuplicates(true);
+            dbConfig.setAllowCreate(true);
+            dbConfig.setTransactional(true);
+            Database myDb = env.openDatabase(null, "testDB", dbConfig);
+
+            DatabaseEntry key = new DatabaseEntry();
+            DatabaseEntry data = new DatabaseEntry(new byte[10000000]);
+	    try {
+		key.setData(TestUtils.getTestArray(10));
+		myDb.put(null, key, data);
+	    } catch (DatabaseException DE) {
+		fail("unexpected DatabaseException");
+	    }
+            myDb.close();
+            env.close();
+	    env = null;
+        } catch (Throwable t) {
+            t.printStackTrace();
+            throw t;
+        }
+    }
+
     /**
      * Set up the environment and db.
      */
@@ -702,22 +953,26 @@ public class DatabaseTest extends TestCase {
                                   String memSize)
         throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setTransactional(transactional);
         envConfig.setConfigParam
             (EnvironmentParams.ENV_CHECK_LEAKS.getName(), "false");
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "6");
+        envConfig.setConfigParam(EnvironmentParams.NODE_MAX_DUPTREE.getName(),
+				 "6");
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
         if (dontRunEvictor) {
             envConfig.setConfigParam(EnvironmentParams.
                                      ENV_RUN_EVICTOR.getName(),
                                      "false");
-            envConfig.setConfigParam
-		(EnvironmentParams.EVICTOR_EVICTION_BATCH_PERCENTAGE.getName(),
-		 "100");
+
+            /* 
+             * Don't let critical eviction run or it will interfere with the
+             * preload test.
+             */
             envConfig.setConfigParam(EnvironmentParams.
-                                     EVICTOR_NODE_SCAN_PERCENTAGE.getName(),
-                                     "100");
+                                     EVICTOR_CRITICAL_PERCENTAGE.getName(),
+                                     "500");
         }
 
         if (memSize != null) {
