@@ -1,27 +1,34 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: EnvironmentTest.java,v 1.108 2004/08/30 17:02:48 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: EnvironmentTest.java,v 1.156 2006/01/20 13:19:20 cwl Exp $
+ */
 
 package com.sleepycat.je;
 
 import java.io.File;
 import java.io.IOException;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Set;
 
 import junit.framework.TestCase;
 
 import com.sleepycat.je.config.ConfigParam;
 import com.sleepycat.je.config.EnvironmentParams;
+import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.EnvConfigObserver;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.dbi.DbConfigManager;
+import com.sleepycat.je.dbi.MemoryBudget;
+import com.sleepycat.je.txn.LockInfo;
 import com.sleepycat.je.util.StringDbt;
 import com.sleepycat.je.util.TestUtils;
-import com.sleepycat.je.utilint.DaemonThread;
+import com.sleepycat.je.utilint.DaemonRunner;
+import com.sleepycat.je.utilint.DbLsn;
 
 public class EnvironmentTest extends TestCase {
 
@@ -31,7 +38,7 @@ public class EnvironmentTest extends TestCase {
     private File envHome;
 
     public EnvironmentTest() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
@@ -43,8 +50,9 @@ public class EnvironmentTest extends TestCase {
     public void tearDown()
         throws Exception {
 
-        /* Close down environments in case the unit test failed so that
-         * the log files can be removed. 
+        /* 
+	 * Close down environments in case the unit test failed so that the log
+	 * files can be removed.
          */
         try {
             if (env1 != null) {
@@ -81,19 +89,28 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            assertEquals("Checking version", "1.5.3",
+            assertEquals("Checking version", "2.1.30",
                          JEVersion.CURRENT_VERSION.getVersionString());
 
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
-            envConfig.setCacheSize(1200);
-            envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "6");
+            envConfig.setCacheSize(MemoryBudget.MIN_MAX_MEMORY_SIZE);
+            /* Don't track detail with a tiny cache size. */
+            envConfig.setConfigParam
+                (EnvironmentParams.CLEANER_TRACK_DETAIL.getName(), "false");
+            envConfig.setConfigParam
+		(EnvironmentParams.NODE_MAX.getName(), "6");
+            envConfig.setConfigParam
+		(EnvironmentParams.LOG_MEM_SIZE.getName(),
+		 EnvironmentParams.LOG_MEM_SIZE_MIN_STRING);
+            envConfig.setConfigParam
+		(EnvironmentParams.NUM_LOG_BUFFERS.getName(), "2");
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
 
             env1.close();
 
-            // Try to open and close again, now that the environment exists
+            /* Try to open and close again, now that the environment exists. */
             envConfig.setAllowCreate(false);
             env1 = new Environment(envHome, envConfig);
             env1.close();
@@ -110,12 +127,21 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            /* Create two environment handles on the same environment */
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+
+            /* Create two environment handles on the same environment. */
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
-            envConfig.setCacheSize(1200);
+            envConfig.setCacheSize(MemoryBudget.MIN_MAX_MEMORY_SIZE);
+            /* Don't track detail with a tiny cache size. */
+            envConfig.setConfigParam
+                (EnvironmentParams.CLEANER_TRACK_DETAIL.getName(), "false");
             envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(),
                                      "6");
+            envConfig.setConfigParam
+		(EnvironmentParams.LOG_MEM_SIZE.getName(),
+		 EnvironmentParams.LOG_MEM_SIZE_MIN_STRING);
+            envConfig.setConfigParam
+		(EnvironmentParams.NUM_LOG_BUFFERS.getName(), "2");
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
             envConfig.setAllowCreate(false);
@@ -133,8 +159,9 @@ public class EnvironmentTest extends TestCase {
             } catch (DatabaseException DENOE) {
             }
 
-            /* Close both, open a third handle, should get a new
-             * EnvironmentImpl.
+            /* 
+	     * Close both, open a third handle, should get a new
+	     * EnvironmentImpl.
              */
             EnvironmentImpl dbenv1 = env1.getEnvironmentImpl();
             env2.close();
@@ -157,7 +184,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
 
@@ -168,13 +195,12 @@ public class EnvironmentTest extends TestCase {
             } catch (DatabaseException DBE) {
             }
 
-            Database exampleDb = null;
             String databaseName = "simpleDb";
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setAllowCreate(true);
             dbConfig.setTransactional(true);
             try {
-                exampleDb = env1.openDatabase(null, databaseName, dbConfig);
+                env1.openDatabase(null, databaseName, dbConfig);
                 fail("expected DatabaseException since Environment not " +
                      "transactional");
             } catch (DatabaseException DBE) {
@@ -191,7 +217,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setReadOnly(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -200,9 +226,8 @@ public class EnvironmentTest extends TestCase {
             dbConfig.setAllowCreate(true);
             dbConfig.setTransactional(true);
             String databaseName = "simpleDb";
-            Database exampleDb = null;
             try {
-                exampleDb = env1.openDatabase(null, databaseName, dbConfig);
+                env1.openDatabase(null, databaseName, dbConfig);
                 fail("expected DatabaseException since Environment is " +
                      "readonly");
             } catch (DatabaseException DBE) {
@@ -216,6 +241,33 @@ public class EnvironmentTest extends TestCase {
         }
     }
 
+    /**
+     * Tests that opening an environment after a clean close does not add to
+     * the log, but that we do initialize the LastFirstActiveLSN.
+     */
+    public void testOpenWithoutCheckpoint()
+        throws Throwable {
+
+        /* Open, close, open. */
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        env1 = new Environment(envHome, envConfig);
+        env1.close();
+        env1 = new Environment(envHome, null);
+
+        /* Check that no checkpoint was performed. */
+        EnvironmentStats stats = env1.getStats(null);
+        assertEquals(0, stats.getNCheckpoints());
+
+        /* Check that the FirstActiveLSN is available for the cleaner. */
+        long lsn =
+            env1.getEnvironmentImpl().getCheckpointer().getFirstActiveLsn();
+        assertTrue(DbLsn.compareTo(lsn, DbLsn.makeLsn(0, 0)) > 0);
+
+        env1.close();
+        env1 = null;
+    }
+
     /**
      * Test environment configuration.
      */
@@ -223,18 +275,19 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         /* This tests assumes these props are immutable. */
-        assertTrue(!isMutableConfig("je.maxMemory"));
+        assertTrue(!isMutableConfig("je.lock.timeout"));
         assertTrue(!isMutableConfig("je.env.isReadOnly"));
 
         try {
+
             /* 
              * Make sure that the environment keeps its own copy of the
              * configuration object.
              */
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setReadOnly(true);
             envConfig.setAllowCreate(true);
-            envConfig.setCacheSize(7777);
+            envConfig.setLockTimeout(7777);
             env1 = new Environment(envHome, envConfig);
 
             /* 
@@ -246,7 +299,7 @@ public class EnvironmentTest extends TestCase {
             assertTrue(envConfig != retrievedConfig1);
             assertEquals(true, retrievedConfig1.getReadOnly());
             assertEquals(true, retrievedConfig1.getAllowCreate());
-            assertEquals(7777, retrievedConfig1.getCacheSize());
+            assertEquals(7777, retrievedConfig1.getLockTimeout());
             
             /* 
              * Make sure that the environment returns a cloned config
@@ -264,24 +317,22 @@ public class EnvironmentTest extends TestCase {
             env2 = new Environment(envHome, null);
             EnvironmentConfig retrievedConfig3 = env2.getConfig();
             assertEquals(true, retrievedConfig3.getReadOnly());
-            assertEquals(7777, retrievedConfig3.getCacheSize());
+            assertEquals(7777, retrievedConfig3.getLockTimeout());
 
             /*
              * Open an environment handle on an existing environment with
              * mismatching config params.
              */
             try {
-                Environment badEnv = new Environment(envHome, 
-                                                     new EnvironmentConfig());
+                new Environment(envHome, TestUtils.initEnvConfig());
                 fail("Shouldn't open, config param has wrong number of params");
             } catch (IllegalArgumentException e) {
                 /* expected */
             }
 
             try {
-                envConfig.setCacheSize(8888);
-                Environment badEnv = new Environment(envHome, 
-                                                     envConfig);
+                envConfig.setLockTimeout(8888);
+                new Environment(envHome, envConfig);
                 fail("Shouldn't open, cache size doesn't match"); 
             } catch (IllegalArgumentException e) {
                 /* expected */
@@ -304,7 +355,7 @@ public class EnvironmentTest extends TestCase {
             assertTrue(retrievedMutableConfig1 != retrievedMutableConfig2);
 
             /*
-             * Plus check that mutables can be retrieved via the main config
+             * Plus check that mutables can be retrieved via the main config.
              */
             EnvironmentConfig retrievedConfig4 = env1.getConfig();
             assertEquals(true, retrievedConfig4.getTxnNoSync());
@@ -338,8 +389,7 @@ public class EnvironmentTest extends TestCase {
          * Note that during unit testing the shared je.properties is expected
          * to be empty, so we don't test the application of je.properties here.
          */
-
-        final String P1 = EnvironmentParams.ENV_RUN_EVICTOR.getName();
+        final String P1 = EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName();
         final String P2 = EnvironmentParams.ENV_RUN_CLEANER.getName();
         final String P3 = EnvironmentParams.ENV_RUN_CHECKPOINTER.getName();
 
@@ -356,7 +406,7 @@ public class EnvironmentTest extends TestCase {
          * P2 is set to true (the default).
          * P3 is set to false (not the default).
          */
-        config = new EnvironmentConfig();
+        config = TestUtils.initEnvConfig();
         config.setAllowCreate(true);
         config.setConfigParam(P2, "true");
         config.setConfigParam(P3, "false");
@@ -374,7 +424,7 @@ public class EnvironmentTest extends TestCase {
          * P2 is set to false.
          * P3 is set to true.
          */
-        config = new EnvironmentConfig();
+        config = TestUtils.initEnvConfig();
         config.setConfigParam(P1, "false");
         config.setConfigParam(P2, "false");
         config.setConfigParam(P3, "true");
@@ -399,6 +449,35 @@ public class EnvironmentTest extends TestCase {
         env2 = null;
     }
 
+    /**
+     * Checks that je.txn.deadlockStackTrace is mutable and takes effect.
+     */
+    public void testTxnDeadlockStackTrace()
+        throws DatabaseException {
+
+        String name = EnvironmentParams.TXN_DEADLOCK_STACK_TRACE.getName();
+        assertTrue(isMutableConfig(name));
+
+        EnvironmentConfig config = TestUtils.initEnvConfig();
+        config.setAllowCreate(true);
+        config.setConfigParam(name, "true");
+        env1 = new Environment(envHome, config);
+        assertTrue(LockInfo.getDeadlockStackTrace());
+
+        EnvironmentMutableConfig mconfig = env1.getMutableConfig();
+        mconfig.setConfigParam(name, "false");
+        env1.setMutableConfig(mconfig);
+        assertTrue(!LockInfo.getDeadlockStackTrace());
+
+        mconfig = env1.getMutableConfig();
+        mconfig.setConfigParam(name, "true");
+        env1.setMutableConfig(mconfig);
+        assertTrue(LockInfo.getDeadlockStackTrace());
+
+        env1.close();
+        env1 = null;
+    }
+
     /**
      * Checks three config parameter values.
      */
@@ -463,17 +542,22 @@ public class EnvironmentTest extends TestCase {
              * <testdestdir>/propTest/je.properties
              */
             StringBuffer testPropsEnv = new StringBuffer();
-            testPropsEnv.append(System.getProperty("testdestdir"));
+            testPropsEnv.append(System.getProperty(TestUtils.DEST_DIR));
             testPropsEnv.append(File.separatorChar);
             testPropsEnv.append("propTest");
             testEnvHome = new File(testPropsEnv.toString());
             TestUtils.removeLogFiles("testParamLoading start",
                                      testEnvHome, false);
 
-            /* Set some configuration params programatically */
+            /*
+             * Set some configuration params programatically.  Do not use
+             * TestUtils.initEnvConfig since we're counting properties.
+             */
             EnvironmentConfig appConfig = new EnvironmentConfig();
             appConfig.setConfigParam("je.log.numBuffers", "88");
-            appConfig.setConfigParam("je.log.totalBufferBytes", "2003");
+            appConfig.setConfigParam
+		("je.log.totalBufferBytes",
+		 EnvironmentParams.LOG_MEM_SIZE_MIN_STRING + 10);
             appConfig.setAllowCreate(true);
 
             Environment appEnv = new Environment(testEnvHome, appConfig);
@@ -501,8 +585,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -510,7 +593,7 @@ public class EnvironmentTest extends TestCase {
             String databaseName = "simpleDb";
             String newDatabaseName = "newSimpleDb";
 
-            /* Try to rename a non-existent db */
+            /* Try to rename a non-existent db. */
             try {
                 env1.renameDatabase(null, databaseName, newDatabaseName);
                 fail("Rename on non-existent db should fail");
@@ -527,7 +610,7 @@ public class EnvironmentTest extends TestCase {
 
             Transaction txn = env1.beginTransaction(null, null);
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
             txn.commit();
             exampleDb.close();
@@ -538,6 +621,10 @@ public class EnvironmentTest extends TestCase {
             cursor = exampleDb.openCursor(null, null);
             // XXX doSimpleVerification(cursor);
             cursor.close();
+
+            /* Check debug name. */
+            DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(exampleDb);
+            assertEquals(newDatabaseName, dbImpl.getDebugName());
             exampleDb.close();
             try {
                 exampleDb = env1.openDatabase(null, databaseName, dbConfig);
@@ -555,7 +642,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -571,7 +658,7 @@ public class EnvironmentTest extends TestCase {
 						   dbConfig);
 
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
             exampleDb.close();
 
@@ -611,7 +698,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -623,19 +710,19 @@ public class EnvironmentTest extends TestCase {
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(txn, databaseName,
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(txn, databaseName, dbConfig);
 
             /* Put some data in, close the database, commit. */
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
             exampleDb.close();
             txn.commit();
 
             /* 
-             * Rename under another txn, shouldn't be able to open
-             * under the old name.
+             * Rename under another txn, shouldn't be able to open under the
+             * old name.
              */
             txn = env1.beginTransaction(null, null);
             env1.renameDatabase(txn, databaseName, newDatabaseName);
@@ -652,8 +739,8 @@ public class EnvironmentTest extends TestCase {
             }
 
             /* 
-             * Abort the rename, should be able to open under the old
-             * name with empty props (DB_CREATE not set)
+             * Abort the rename, should be able to open under the old name with
+             * empty props (DB_CREATE not set)
              */
             txn.abort();
             exampleDb = new Database(env1);
@@ -683,12 +770,12 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {        
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            /* Set up an environment. */
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
 
-
             String databaseName = "simpleDb";
 
             /* Try to remove a non-existent db */
@@ -702,14 +789,16 @@ public class EnvironmentTest extends TestCase {
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(null, databaseName,
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(null, databaseName, dbConfig);
 
             Transaction txn = env1.beginTransaction(null, null);
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
             txn.commit();
+
+            /* Remove should fail because database is open. */
             try {
                 env1.removeDatabase(null, databaseName);
                 fail("didn't get db open exception");
@@ -718,6 +807,8 @@ public class EnvironmentTest extends TestCase {
             exampleDb.close();
 
             env1.removeDatabase(null, databaseName);
+
+            /* Remove should fail because database does not exist. */
             try {
                 exampleDb = env1.openDatabase(null, databaseName, null);
                 fail("did not catch db does not exist exception");
@@ -734,28 +825,30 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {        
-            /* Set up an environment */
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            /* Set up an environment. */
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
 
-            /* Make a database */
+            /* Make a database. */
             String databaseName = "simpleDb";
             Transaction txn = env1.beginTransaction(null, null);
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(txn, databaseName, 
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(txn, databaseName, dbConfig);
 
             /* Insert and delete data in it. */
 	    Cursor cursor = exampleDb.openCursor(txn, null);
-	    doSimpleCursorPutAndDelete(cursor);
+	    doSimpleCursorPutAndDelete(cursor, false);
 	    cursor.close();
 
-	    /* Try a remove without closing the open Database handle.
-               Should block */
+	    /* 
+	     * Try a remove without closing the open Database handle.  Should
+             * get an exception.
+	     */
             try {
                 env1.removeDatabase(txn, databaseName);
                 fail("didn't get db open exception");
@@ -790,8 +883,8 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {        
-            /* Set up an environment */
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            /* Set up an environment. */
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -802,18 +895,19 @@ public class EnvironmentTest extends TestCase {
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(txn, databaseName, 
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(txn, databaseName, dbConfig);
             txn.commit();
 
             /* Start a new txn and put some data in the created db. */
             txn = env1.beginTransaction(null, null);
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
 
-            /* Try to remove, we should get an exception because the
-             * db is open.
+            /* 
+	     * Try to remove, we should get an exception because the db is
+	     * open.
              */
             try {
                 env1.removeDatabase(txn, databaseName);
@@ -821,6 +915,13 @@ public class EnvironmentTest extends TestCase {
             } catch (DatabaseException DBE) {
             }
             exampleDb.close();
+
+	    /* 
+	     * txn can only be aborted at this point since the removeDatabase()
+	     * timed out.
+	     */
+	    txn.abort();
+            txn = env1.beginTransaction(null, null);
             env1.removeDatabase(txn, databaseName);
 
             try {
@@ -840,10 +941,7 @@ public class EnvironmentTest extends TestCase {
             } catch (DatabaseException DBE) {
                 fail("db does not exist anymore after delete/abort");
             }
-            cursor = exampleDb.openCursor(txn, null);
-            // XXX doSimpleVerification(cursor);
-           
-            cursor.close();
+
             exampleDb.close();
             env1.close();
         } catch (Throwable t) {
@@ -852,17 +950,89 @@ public class EnvironmentTest extends TestCase {
         }
     }
 
+    /**
+     * Provides general testing of getDatabaseNames.  Additionally verifies a
+     * fix for a bug that occurred when the first DB (lowest valued name) was
+     * removed or renamed prior to calling getDatabaseNames.  A NPE occurred
+     * in this case if the compressor had not yet deleted the BIN entry for
+     * the removed/renamed name. [#13377]
+     */
+    public void testGetDatabaseNames()
+        throws DatabaseException {
+
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
+
+        DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+
+        /* Start with no databases. */
+        Set dbNames = new HashSet();
+        env1 = new Environment(envHome, envConfig);
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Add DB1. */
+        dbNames.add("DB1");
+        Database db = env1.openDatabase(null, "DB1", dbConfig);
+        db.close();
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Add DB2. */
+        dbNames.add("DB2");
+        db = env1.openDatabase(null, "DB2", dbConfig);
+        db.close();
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Rename DB2 to DB3 (this caused NPE). */
+        dbNames.remove("DB2");
+        dbNames.add("DB3");
+        env1.renameDatabase(null, "DB2", "DB3");
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Remove DB1. */
+        dbNames.remove("DB1");
+        dbNames.add("DB4");
+        env1.renameDatabase(null, "DB1", "DB4");
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Add DB0. */
+        dbNames.add("DB0");
+        db = env1.openDatabase(null, "DB0", dbConfig);
+        db.close();
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        /* Remove DB0 (this caused NPE). */
+        dbNames.remove("DB0");
+        env1.removeDatabase(null, "DB0");
+        checkDbNames(dbNames, env1.getDatabaseNames());
+
+        env1.close();
+        env1 = null;
+    }
+
+    /**
+     * Checks that the expected set of names equals the list of names returned
+     * from getDatabaseNames.  A list can't be directly compared to a set using
+     * equals().
+     */
+    private void checkDbNames(Set expected, List actual) {
+        assertEquals(expected.size(), actual.size());
+        assertEquals(expected, new HashSet(actual));
+    }
+
     /*
-     * This little test case can only invoke the compressor, since the
-     * evictor, cleaner and checkpointer are all governed by utilization
-     * metrics and are tested elsewhere.
+     * This little test case can only invoke the compressor, since the evictor,
+     * cleaner and checkpointer are all governed by utilization metrics and are
+     * tested elsewhere.
      */
     public void testDaemonManualInvocation()
         throws Throwable {
 
         try {        
-            /* Set up an environment */
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            /* Set up an environment. */
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             String testPropVal = "120000000";
             envConfig.setConfigParam
@@ -871,18 +1041,22 @@ public class EnvironmentTest extends TestCase {
             envConfig.setConfigParam
                 (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
             envConfig.setAllowCreate(true);
+            envConfig.setConfigParam
+		(EnvironmentParams.LOG_MEM_SIZE.getName(), "20000");
+            envConfig.setConfigParam
+		(EnvironmentParams.NUM_LOG_BUFFERS.getName(), "2");
             env1 = new Environment(envHome, envConfig);
 
             String databaseName = "simpleDb";
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(null, databaseName, 
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(null, databaseName, dbConfig);
 
             Transaction txn = env1.beginTransaction(null, null);
             Cursor cursor = exampleDb.openCursor(txn, null);
-            doSimpleCursorPutAndDelete(cursor);
+            doSimpleCursorPutAndDelete(cursor, false);
             cursor.close();
             txn.commit();
             exampleDb.close();
@@ -919,12 +1093,15 @@ public class EnvironmentTest extends TestCase {
             EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(),
         };
 
-        EnvironmentConfig config = new EnvironmentConfig();
+        EnvironmentConfig config = TestUtils.initEnvConfig();
         config.setAllowCreate(true);
 
         config.setConfigParam
             (EnvironmentParams.MAX_MEMORY.getName(),
-             "1024");
+             MemoryBudget.MIN_MAX_MEMORY_SIZE_STRING);
+        /* Don't track detail with a tiny cache size. */
+        config.setConfigParam
+            (EnvironmentParams.CLEANER_TRACK_DETAIL.getName(), "false");
         config.setConfigParam
             (EnvironmentParams.CLEANER_BYTES_INTERVAL.getName(),
              "100");
@@ -934,13 +1111,17 @@ public class EnvironmentTest extends TestCase {
         config.setConfigParam
             (EnvironmentParams.COMPRESSOR_WAKEUP_INTERVAL.getName(),
              "1000000");
+	config.setConfigParam(EnvironmentParams.LOG_MEM_SIZE.getName(),
+			      EnvironmentParams.LOG_MEM_SIZE_MIN_STRING);
+	config.setConfigParam
+	    (EnvironmentParams.NUM_LOG_BUFFERS.getName(), "2");
         setBoolConfigParams(config, runProps,
                             new boolean[] { false, false, false, false });
 
         env1 = new Environment(envHome, config);
         EnvironmentImpl envImpl = env1.getEnvironmentImpl();
 
-        final DaemonThread[] daemons = {
+        final DaemonRunner[] daemons = {
             envImpl.getEvictor(),
             envImpl.getCleaner(),
             envImpl.getCheckpointer(),
@@ -951,8 +1132,10 @@ public class EnvironmentTest extends TestCase {
                              new boolean[] { false, false, false, false });
         doTestDaemonRunPause(env1, daemons, runProps,
                              new boolean[] { true,  false, false, false });
-        doTestDaemonRunPause(env1, daemons, runProps,
-                             new boolean[] { false, true,  false, false });
+	if (!envImpl.isNoLocking()) {
+	    doTestDaemonRunPause(env1, daemons, runProps,
+				 new boolean[] { false, true,  false, false });
+	}
         doTestDaemonRunPause(env1, daemons, runProps,
                              new boolean[] { false, false, true,  false });
         doTestDaemonRunPause(env1, daemons, runProps,
@@ -967,8 +1150,10 @@ public class EnvironmentTest extends TestCase {
     /**
      * Tests a set of daemon on/off settings.
      */
-    private void doTestDaemonRunPause(Environment env, DaemonThread[] daemons,
-                                      String[] runProps, boolean[] runValues)
+    private void doTestDaemonRunPause(Environment env,
+				      DaemonRunner[] daemons,
+                                      String[] runProps,
+				      boolean[] runValues)
         throws DatabaseException, InterruptedException {
 
         /* Set daemon run properties. */
@@ -992,9 +1177,10 @@ public class EnvironmentTest extends TestCase {
         String dbName = "testDaemonRunPause";
         DatabaseConfig dbConfig = new DatabaseConfig();
         dbConfig.setAllowCreate(true);
+	dbConfig.setSortedDuplicates(true);
         Database db = env1.openDatabase(null, dbName, dbConfig);
         Cursor cursor = db.openCursor(null, null);
-        doSimpleCursorPutAndDelete(cursor);
+        doSimpleCursorPutAndDelete(cursor, true);
         cursor.close();
         db.close();
 
@@ -1013,7 +1199,8 @@ public class EnvironmentTest extends TestCase {
     }
 
     private void setBoolConfigParams(EnvironmentMutableConfig config,
-                                     String[] names, boolean[] values) {
+                                     String[] names,
+				     boolean[] values) {
         for (int i = 0; i < names.length; i += 1) {
             config.setConfigParam(names[i],
                                   Boolean.valueOf(values[i]).toString());
@@ -1024,7 +1211,7 @@ public class EnvironmentTest extends TestCase {
         throws Throwable {
 
         try {
-            EnvironmentConfig envConfig = new EnvironmentConfig();
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
             envConfig.setTransactional(true);
             envConfig.setAllowCreate(true);
             env1 = new Environment(envHome, envConfig);
@@ -1038,7 +1225,9 @@ public class EnvironmentTest extends TestCase {
 
             envConfig.setAllowCreate(false);
             env1 = new Environment(envHome, envConfig);
-            Transaction txn = env1.beginTransaction(null, null);
+            
+           /* Create a transaction to prevent the close from succeeding */
+            env1.beginTransaction(null, null);
 	    try {
 		env1.close();
 		fail("Didn't catch DatabaseException for open transactions");
@@ -1057,8 +1246,8 @@ public class EnvironmentTest extends TestCase {
             DatabaseConfig dbConfig = new DatabaseConfig();
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
-            Database exampleDb = env1.openDatabase(null, databaseName,
-						   dbConfig);
+            Database exampleDb =
+		env1.openDatabase(null, databaseName, dbConfig);
 	    try {
 		env1.close();
 		fail("Didn't catch DatabaseException for open dbs");
@@ -1084,7 +1273,7 @@ public class EnvironmentTest extends TestCase {
         "one", "two", "three", "four", "five",
         "six", "seven", "eight", "nine" };
 
-    protected void doSimpleCursorPutAndDelete(Cursor cursor)
+    protected void doSimpleCursorPutAndDelete(Cursor cursor, boolean extras)
         throws DatabaseException {
 
         StringDbt foundKey = new StringDbt();
@@ -1097,6 +1286,17 @@ public class EnvironmentTest extends TestCase {
                 OperationStatus.SUCCESS) {
                 throw new DatabaseException("non-0 return");
             }
+	    /* Need to write some extra out to force eviction to run. */
+	    if (extras) {
+		for (int j = 0; j < 500; j++) {
+		    foundData.setString(Integer.toString(j));
+		    OperationStatus status =
+			cursor.put(foundKey, foundData);
+		    if (status != OperationStatus.SUCCESS) {
+			throw new DatabaseException("non-0 return " + status);
+		    }
+		}
+	    }
         }
 
         OperationStatus status =
