@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: CleanerTest.java,v 1.67 2004/07/28 18:30:35 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: CleanerTest.java,v 1.83 2006/01/04 18:13:01 cwl Exp $
+ */
 
 package com.sleepycat.je.cleaner;
 
@@ -19,22 +19,27 @@ import java.util.Set;
 
 import junit.framework.TestCase;
 
+import com.sleepycat.bind.tuple.IntegerBinding;
 import com.sleepycat.je.CheckpointConfig;
 import com.sleepycat.je.Cursor;
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
+import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
+import com.sleepycat.je.EnvironmentMutableConfig;
 import com.sleepycat.je.EnvironmentStats;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
+import com.sleepycat.je.cleaner.Cleaner;
+import com.sleepycat.je.cleaner.UtilizationProfile;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.EnvironmentImpl;
+import com.sleepycat.je.dbi.MemoryBudget;
 import com.sleepycat.je.log.FileManager;
-import com.sleepycat.je.tree.Key;
 import com.sleepycat.je.util.StringDbt;
 import com.sleepycat.je.util.TestUtils;
 
@@ -42,6 +47,7 @@ public class CleanerTest extends TestCase {
 
     private static final int N_KEYS = 300;
     private static final int N_KEY_BYTES = 10;
+
     /* 
      * Make the log file size small enough to allow cleaning, but large enough
      * not to generate a lot of fsyncing at the log file boundaries.
@@ -53,7 +59,6 @@ public class CleanerTest extends TestCase {
     private Database exampleDb;
     private CheckpointConfig forceConfig;
 
-
     public CleanerTest() {
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
         forceConfig = new CheckpointConfig();
@@ -70,7 +75,8 @@ public class CleanerTest extends TestCase {
     private void initEnv(boolean createDb, boolean allowDups)
         throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+	DbInternal.disableParameterValidation(envConfig);
         envConfig.setTransactional(true);
         envConfig.setAllowCreate(true);
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
@@ -90,6 +96,10 @@ public class CleanerTest extends TestCase {
         envConfig.setConfigParam(EnvironmentParams.BIN_DELTA_PERCENT.getName(),
                                  "75");
 
+        /* Don't use detail tracking in this test. */
+        envConfig.setConfigParam
+            (EnvironmentParams.CLEANER_TRACK_DETAIL.getName(), "false");
+
         exampleEnv = new Environment(envHome, envConfig);
 
         String databaseName = "cleanerDb";
@@ -103,24 +113,20 @@ public class CleanerTest extends TestCase {
     public void tearDown()
         throws IOException, DatabaseException {
 
-        try {
-            exampleDb.close();
-        } catch (DatabaseException e) {
-            System.out.println("tearDown: " + e);
-        }
-        try {
-            exampleEnv.close();
-        } catch (DatabaseException e) {
-            System.out.println("tearDown: " + e);
+        if (exampleEnv != null) {
+            try {
+                exampleEnv.close();
+            } catch (DatabaseException e) {
+                System.out.println("tearDown: " + e);
+            }
         }
         exampleDb = null;
         exampleEnv = null;
                 
-        /*
+        //*
         TestUtils.removeLogFiles("TearDown", envHome, true);
-        TestUtils.removeFiles("TearDown", envHome,
-                              FileManager.DEL_SUFFIX);
-        */
+        TestUtils.removeFiles("TearDown", envHome, FileManager.DEL_SUFFIX);
+        //*/
     }
 
     private void closeEnv()
@@ -133,6 +139,7 @@ public class CleanerTest extends TestCase {
 
         if (exampleEnv != null) {
             exampleEnv.close();
+            exampleEnv = null;
         }
     }
 
@@ -183,13 +190,15 @@ public class CleanerTest extends TestCase {
 	exampleEnv.checkpoint(forceConfig);
 
         for (int i = 0; i < (int) lastNum.longValue(); i++) {
+
             /*
-             * Clean one file aggressively.  Utilization-based cleaning won't
+             * Force clean one file.  Utilization-based cleaning won't
              * work here, since utilization is over 90%.
              */
             DbInternal.envGetEnvironmentImpl(exampleEnv).
-                       getCleaner().
-                       doClean(false, false, true);
+		getCleaner().
+		doClean(false, // cleanMultipleFiles
+			true); // forceCleaning
         }
 
         EnvironmentStats stats = exampleEnv.getStats(TestUtils.FAST_STATS);
@@ -203,10 +212,12 @@ public class CleanerTest extends TestCase {
         checkData(expectedMap);
         assertTrue(fileManager.getLastFileNum().longValue() >
                    lastNum.longValue());
+
+        closeEnv();
     }
 
     /**
-     * Ensure that INs, including BINDeltas, are cleaned.
+     * Ensure that INs are cleaned.
      */
     public void testCleanInternalNodes()
         throws DatabaseException {
@@ -233,7 +244,7 @@ public class CleanerTest extends TestCase {
         modifyData(expectedMap, 10, true);
         checkData(expectedMap);
 
-        /* Checkpoint -- this should create BINDeltas. */
+        /* Checkpoint -- this should obsolete INs. */
         exampleEnv.checkpoint(forceConfig);
         checkData(expectedMap);
 
@@ -248,13 +259,14 @@ public class CleanerTest extends TestCase {
         /* Make sure we really cleaned something.*/
         assertTrue(stats.getNINsCleaned() > 0);
         assertTrue(stats.getNLNsCleaned() > 0);
-        assertTrue(stats.getNDeltasCleaned() > 0);
                 
         closeEnv();
         initEnv(false, true);
         checkData(expectedMap);
         assertTrue(fileManager.getLastFileNum().longValue() >
                    lastNum.longValue());
+
+        closeEnv();
     }
 
     /**
@@ -264,61 +276,136 @@ public class CleanerTest extends TestCase {
         throws Throwable {
 
         initEnv(true, true);
-        try {
-	    int nKeys = 20; // test ends up inserting 2*nKeys
-	    int nDupsPerKey = 30;
 
-	    EnvironmentImpl environment =
-                DbInternal.envGetEnvironmentImpl(exampleEnv);
-	    FileManager fileManager = environment.getFileManager();
+        int nKeys = 20; // test ends up inserting 2*nKeys
+        int nDupsPerKey = 30;
 
-	    /* Insert some non dup data, modify, insert dup data. */
-	    Map expectedMap = new HashMap();
-	    doLargePut(expectedMap, nKeys, 1, true);
-	    modifyData(expectedMap, 10, true);
-	    doLargePut(expectedMap, nKeys, nDupsPerKey, true);
-	    checkData(expectedMap);
+        EnvironmentImpl environment =
+            DbInternal.envGetEnvironmentImpl(exampleEnv);
+        FileManager fileManager = environment.getFileManager();
 
-	    /* 
-	     * Delete all the data, but abort. (Try to fill up the log
-	     * with entries we don't need.
-	     */
-            deleteData(expectedMap, false, false);
-            checkData(expectedMap);
+        /* Insert some non dup data, modify, insert dup data. */
+        Map expectedMap = new HashMap();
+        doLargePut(expectedMap, nKeys, 1, true);
+        modifyData(expectedMap, 10, true);
+        doLargePut(expectedMap, nKeys, nDupsPerKey, true);
+        checkData(expectedMap);
 
-	    /* Do some more insertions, but abort them. */
-            doLargePut(expectedMap, nKeys, nDupsPerKey, false);
-            checkData(expectedMap);
+        /* 
+         * Delete all the data, but abort. (Try to fill up the log
+         * with entries we don't need.
+         */
+        deleteData(expectedMap, false, false);
+        checkData(expectedMap);
 
-	    /* Do some more insertions and commit them. */
-            doLargePut(expectedMap, nKeys, nDupsPerKey, true);
-            checkData(expectedMap);
+        /* Do some more insertions, but abort them. */
+        doLargePut(expectedMap, nKeys, nDupsPerKey, false);
+        checkData(expectedMap);
 
-	    /* Checkpoint */
-	    exampleEnv.checkpoint(forceConfig);
-	    checkData(expectedMap);
+        /* Do some more insertions and commit them. */
+        doLargePut(expectedMap, nKeys, nDupsPerKey, true);
+        checkData(expectedMap);
 
-	    /* Clean */
-	    Long lastNum = fileManager.getLastFileNum();
-            exampleEnv.cleanLog();
-		
-	    /* Validate after cleaning. */
-	    checkData(expectedMap);
-	    EnvironmentStats stats = exampleEnv.getStats(TestUtils.FAST_STATS);
+        /* Checkpoint */
+        exampleEnv.checkpoint(forceConfig);
+        checkData(expectedMap);
 
-	    /* Make sure we really cleaned something.*/
-	    assertTrue(stats.getNINsCleaned() > 0);
-	    assertTrue(stats.getNLNsCleaned() > 0);
+        /* Clean */
+        Long lastNum = fileManager.getLastFileNum();
+        exampleEnv.cleanLog();
+            
+        /* Validate after cleaning. */
+        checkData(expectedMap);
+        EnvironmentStats stats = exampleEnv.getStats(TestUtils.FAST_STATS);
 
-	    closeEnv();
-	    initEnv(false, true);
-	    checkData(expectedMap);
-	    assertTrue(fileManager.getLastFileNum().longValue() >
-		       lastNum.longValue());
-        } catch (Throwable t) {
-            t.printStackTrace();
-            throw t;
-        }
+        /* Make sure we really cleaned something.*/
+        assertTrue(stats.getNINsCleaned() > 0);
+        assertTrue(stats.getNLNsCleaned() > 0);
+
+        closeEnv();
+        initEnv(false, true);
+        checkData(expectedMap);
+        assertTrue(fileManager.getLastFileNum().longValue() >
+                   lastNum.longValue());
+
+        closeEnv();
+    }
+
+    /**
+     * Test for SR13191.  This SR shows a problem where a MapLN is initialized
+     * with a DatabaseImpl that has a null EnvironmentImpl.  When the Database
+     * gets used, a NullPointerException occurs in the Cursor code which
+     * expects there to be an EnvironmentImpl present.  The MapLN gets init'd
+     * by the Cleaner reading through a log file and encountering a MapLN which
+     * is not presently in the DbTree.  As an efficiency, the Cleaner calls
+     * updateEntry on the BIN to try to insert the MapLN into the BIN so that
+     * it won't have to fetch it when it migrates the BIN.  But this is bad
+     * since the MapLN has not been init'd properly.  The fix was to ensure
+     * that the MapLN is init'd correctly by calling postFetchInit on it just
+     * prior to inserting it into the BIN.
+     *
+     * This test first creates an environment and two databases.  The first
+     * database it just adds to the tree with no data.  This will be the MapLN
+     * that eventually gets instantiated by the cleaner.  The second database
+     * is used just to create a bunch of data that will get deleted so as to
+     * create a low utilization for one of the log files.  Once the data for
+     * db2 is created, the log is flipped (so file 0 is the one with the MapLN
+     * for db1 in it), and the environment is closed and reopened.  We insert
+     * more data into db2 until we have enough .jdb files that file 0 is
+     * attractive to the cleaner.  Call the cleaner to have it instantiate the
+     * MapLN and then use the MapLN in a Database.get() call.
+     */
+    public void testSR13191()
+	throws Throwable {
+
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setConfigParam
+	    (EnvironmentParams.ENV_RUN_CLEANER.getName(), "false");
+        Environment env = new Environment(envHome, envConfig);
+	EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
+	FileManager fileManager =
+	    DbInternal.envGetEnvironmentImpl(env).getFileManager();
+
+        DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+        Database db1 =
+	    env.openDatabase(null, "db1", dbConfig);
+
+        Database db2 =
+	    env.openDatabase(null, "db2", dbConfig);
+
+	DatabaseEntry key = new DatabaseEntry();
+	DatabaseEntry data = new DatabaseEntry();
+	IntegerBinding.intToEntry(1, key);
+	data.setData(new byte[100000]);
+        for (int i = 0; i < 50; i++) {
+            assertEquals(OperationStatus.SUCCESS, db2.put(null, key, data));
+	}
+	db1.close();
+	db2.close();
+        assertEquals("Should have 0 as current file", 0L,
+                     fileManager.getCurrentFileNum());
+	envImpl.forceLogFileFlip();
+	env.close();
+
+        env = new Environment(envHome, envConfig);
+	fileManager = DbInternal.envGetEnvironmentImpl(env).getFileManager();
+        assertEquals("Should have 1 as current file", 1L,
+                     fileManager.getCurrentFileNum());
+
+	db2 = env.openDatabase(null, "db2", dbConfig);
+
+        for (int i = 0; i < 250; i++) {
+            assertEquals(OperationStatus.SUCCESS, db2.put(null, key, data));
+	}
+
+	db2.close();
+	env.cleanLog();
+	db1 = env.openDatabase(null, "db1", dbConfig);
+	db1.get(null, key, data, null);
+	db1.close();
+	env.close();
     }
 
     /**
@@ -333,9 +420,9 @@ public class CleanerTest extends TestCase {
 
         Transaction txn = exampleEnv.beginTransaction(null, null);
         for (int i = 0; i < nKeys; i++) {
-            Key key = new Key(new byte[N_KEY_BYTES]);
-            TestUtils.generateRandomAlphaBytes(key.getKey());
-            String keyString = new String(key.getKey());
+            byte[] key = new byte[N_KEY_BYTES];
+            TestUtils.generateRandomAlphaBytes(key);
+            String keyString = new String(key);
 
             /*
 	     * The data map is keyed by key value, and holds a hash
@@ -562,4 +649,141 @@ public class CleanerTest extends TestCase {
             }
         }
     }
+
+    /**
+     * Tests that cleaner mutable configuration parameters can be changed and
+     * that the changes actually take effect.
+     */
+    public void testMutableConfig()
+        throws DatabaseException {
+
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        exampleEnv = new Environment(envHome, envConfig);
+        envConfig = exampleEnv.getConfig();
+        EnvironmentImpl envImpl =
+            DbInternal.envGetEnvironmentImpl(exampleEnv);
+        Cleaner cleaner = envImpl.getCleaner();
+        UtilizationProfile profile = envImpl.getUtilizationProfile();
+        MemoryBudget budget = envImpl.getMemoryBudget();
+        String name;
+        String val;
+
+        /* je.cleaner.minUtilization */
+        name = EnvironmentParams.CLEANER_MIN_UTILIZATION.getName();
+        setParam(name, "33");
+        assertEquals(33, profile.minUtilization);
+
+        /* je.cleaner.minFileUtilization */
+        name = EnvironmentParams.CLEANER_MIN_FILE_UTILIZATION.getName();
+        setParam(name, "7");
+        assertEquals(7, profile.minFileUtilization);
+
+        /* je.cleaner.bytesInterval */
+        name = EnvironmentParams.CLEANER_BYTES_INTERVAL.getName();
+        setParam(name, "1000");
+        assertEquals(1000, cleaner.cleanerBytesInterval);
+
+        /* je.cleaner.deadlockRetry */
+        name = EnvironmentParams.CLEANER_DEADLOCK_RETRY.getName();
+        setParam(name, "7");
+        assertEquals(7, cleaner.nDeadlockRetries);
+
+        /* je.cleaner.lockTimeout */
+        name = EnvironmentParams.CLEANER_LOCK_TIMEOUT.getName();
+        setParam(name, "7000");
+        assertEquals(7, cleaner.lockTimeout);
+
+        /* je.cleaner.expunge */
+        name = EnvironmentParams.CLEANER_REMOVE.getName();
+        val = "false".equals(envConfig.getConfigParam(name)) ?
+            "true" : "false";
+        setParam(name, val);
+        assertEquals(val.equals("true"), cleaner.expunge);
+
+        /* je.cleaner.minAge */
+        name = EnvironmentParams.CLEANER_MIN_AGE.getName();
+        setParam(name, "7");
+        assertEquals(7, profile.minAge);
+
+        /* je.cleaner.cluster */
+        name = EnvironmentParams.CLEANER_CLUSTER.getName();
+        val = "false".equals(envConfig.getConfigParam(name)) ?
+            "true" : "false";
+        setParam(name, val);
+        assertEquals(val.equals("true"), cleaner.clusterResident);
+        /* Cannot set both cluster and clusterAll to true. */
+        setParam(name, "false");
+
+        /* je.cleaner.clusterAll */
+        name = EnvironmentParams.CLEANER_CLUSTER_ALL.getName();
+        val = "false".equals(envConfig.getConfigParam(name)) ?
+            "true" : "false";
+        setParam(name, val);
+        assertEquals(val.equals("true"), cleaner.clusterAll);
+
+        /* je.cleaner.maxBatchFiles */
+        name = EnvironmentParams.CLEANER_MAX_BATCH_FILES.getName();
+        setParam(name, "7");
+        assertEquals(7, cleaner.maxBatchFiles);
+
+        /* je.cleaner.readSize */
+        name = EnvironmentParams.CLEANER_READ_SIZE.getName();
+        setParam(name, "7777");
+        assertEquals(7777, cleaner.readBufferSize);
+
+        /* je.cleaner.detailMaxMemoryPercentage */
+        name = EnvironmentParams.CLEANER_DETAIL_MAX_MEMORY_PERCENTAGE.
+            getName();
+        setParam(name, "7");
+        assertEquals((budget.getMaxMemory() * 7) / 100,
+                     budget.getTrackerBudget());
+
+        /* je.cleaner.threads */
+        name = EnvironmentParams.CLEANER_THREADS.getName();
+        setParam(name, "7");
+        assertEquals((envImpl.isNoLocking() ? 0 : 7),
+		     countCleanerThreads());
+
+        exampleEnv.close();
+        exampleEnv = null;
+    }
+
+    /**
+     * Sets a mutable config param, checking that the given value is not
+     * already set and that it actually changes.
+     */
+    private void setParam(String name, String val)
+        throws DatabaseException {
+
+        EnvironmentMutableConfig config = exampleEnv.getMutableConfig();
+        String myVal = config.getConfigParam(name);
+        assertTrue(!val.equals(myVal));
+
+        config.setConfigParam(name, val);
+        exampleEnv.setMutableConfig(config);
+
+        config = exampleEnv.getMutableConfig();
+        myVal = config.getConfigParam(name);
+        assertTrue(val.equals(myVal));
+    }
+
+    /**
+     * Count the number of threads with the name "Cleaner#".
+     */
+    private int countCleanerThreads() {
+
+        Thread[] threads = new Thread[Thread.activeCount()];
+        Thread.enumerate(threads);
+
+        int count = 0;
+        for (int i = 0; i < threads.length; i += 1) {
+            if (threads[i] != null &&
+		threads[i].getName().startsWith("Cleaner")) {
+                count += 1;
+            }
+        }
+
+        return count;
+    }
 }
