@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: UtilizationTest.java,v 1.3 2004/07/23 02:42:49 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: UtilizationTest.java,v 1.17 2006/01/03 21:56:13 bostic Exp $
+ */
 
 package com.sleepycat.je.cleaner;
 
@@ -25,18 +25,13 @@ import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.DbInternal;
-import com.sleepycat.je.DbTestProxy;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
-import com.sleepycat.je.dbi.CursorImpl;
 import com.sleepycat.je.log.FileManager;
-import com.sleepycat.je.tree.BIN;
-import com.sleepycat.je.tree.ChildReference;
 import com.sleepycat.je.util.TestUtils;
-import com.sleepycat.je.utilint.DbLsn;
 
 public class UtilizationTest extends TestCase {
 
@@ -58,13 +53,8 @@ public class UtilizationTest extends TestCase {
     private Environment env;
     private Database db;
     private boolean dups = false;
-    private int nextKey = 0;
     private DatabaseEntry keyEntry = new DatabaseEntry();
     private DatabaseEntry dataEntry = new DatabaseEntry();
-    private long testFile;
-    private int firstKey;
-    private int lastKey;
-    private int keyCount;
     private String operation;
     private long lastFileSeen;
 
@@ -112,7 +102,7 @@ public class UtilizationTest extends TestCase {
         }
                 
         try {
-            //*
+            /*
             TestUtils.removeLogFiles("tearDown", envHome, true);
             TestUtils.removeFiles("tearDown", envHome, FileManager.DEL_SUFFIX);
             //*/
@@ -133,8 +123,10 @@ public class UtilizationTest extends TestCase {
     private void openEnv()
         throws DatabaseException {
 
-        EnvironmentConfig config = new EnvironmentConfig();
+        EnvironmentConfig config = TestUtils.initEnvConfig();
+	DbInternal.disableParameterValidation(config);
         config.setTransactional(true);
+        config.setTxnNoSync(true);
         config.setAllowCreate(true);
         /* Do not run the daemons. */
         config.setConfigParam
@@ -153,6 +145,11 @@ public class UtilizationTest extends TestCase {
             (EnvironmentParams.LOG_DIRECT_NIO.getName(), "false");
         env = new Environment(envHome, config);
 
+        /* Speed up test that uses lots of very small files. */
+        DbInternal.envGetEnvironmentImpl(env).
+                   getFileManager().
+                   setSyncAtFileEnd(false);
+
         openDb();
     }
 
@@ -185,6 +182,35 @@ public class UtilizationTest extends TestCase {
         }
     }
 
+    public void testReuseSlotAfterDelete()
+        throws DatabaseException {
+
+        openEnv();
+
+        /* Insert and delete without compress to create a knownDeleted slot. */
+        Transaction txn = env.beginTransaction(null, null);
+        long file0 = doPut(0, txn);
+        long file1 = doDelete(0, txn);
+        txn.commit();
+
+        /* Insert key 0 to reuse the knownDeleted slot. */
+        txn = env.beginTransaction(null, null);
+        long file2 = doPut(0, txn);
+        /* Delete and insert to reuse deleted slot in same txn. */
+        long file3 = doDelete(0, txn);
+        long file4 = doPut(0, txn);
+        txn.commit();
+        performRecoveryOperation();
+
+        expectObsolete(file0, true);
+        expectObsolete(file1, true);
+        expectObsolete(file2, true);
+        expectObsolete(file3, true);
+        expectObsolete(file4, false);
+
+        closeEnv(true);
+    }
+
     public void testReuseKnownDeletedSlot()
         throws DatabaseException {
 
@@ -1013,32 +1039,33 @@ public class UtilizationTest extends TestCase {
         closeEnv(true);
     }
 
-    /* Recovery of truncate/remove is not supported yet. */
-
-    public void xtestTruncate()
+    public void testTruncate()
         throws DatabaseException {
 
         truncateOrRemove(true, true);
     }
 
-    public void xtestTruncateAbort()
+    public void testTruncateAbort()
         throws DatabaseException {
 
         truncateOrRemove(true, false);
     }
 
-    public void xtestRemove()
+    public void testRemove()
         throws DatabaseException {
 
         truncateOrRemove(false, true);
     }
 
-    public void xtestRemoveAbort()
+    public void testRemoveAbort()
         throws DatabaseException {
 
         truncateOrRemove(false, false);
     }
 
+    /**
+     * @deprecated use of Database.truncate
+     */
     private void truncateOrRemove(boolean truncate, boolean commit)
         throws DatabaseException {
 
@@ -1178,11 +1205,17 @@ public class UtilizationTest extends TestCase {
         throws DatabaseException {
 
         if (OP_NONE.equals(operation)) {
+            /* Compress to count deleted LNs. */
+            env.compress();
         } else if (OP_CHECKPOINT.equals(operation)) {
+            /* Compress before checkpointing to count deleted LNs. */
+            env.compress();
             env.checkpoint(forceConfig);
         } else if (OP_RECOVER.equals(operation)) {
             closeEnv(false);
             openEnv();
+            /* Compress after recovery to count deleted LNs. */
+            env.compress();
         } else {
             assert false : operation;
         }
@@ -1196,21 +1229,7 @@ public class UtilizationTest extends TestCase {
     private long getFile(Cursor cursor)
         throws DatabaseException {
 
-        CursorImpl impl = DbTestProxy.dbcGetCursorImpl(cursor);
-        int index;
-        BIN bin = impl.getDupBIN();
-        if (bin != null) {
-            index = impl.getDupIndex();
-        } else {
-            bin = impl.getBIN();
-            assertNotNull(bin);
-            index = impl.getIndex();
-        }
-        ChildReference ref = bin.getEntry(index);
-        assertNotNull(ref);
-        DbLsn lsn = ref.getLsn();
-        assertNotNull(lsn);
-        long file = lsn.getFileNumber();
+        long file = CleanerTestUtils.getLogFile(this, cursor);
         assert file > lastFileSeen;
         lastFileSeen = file;
         return file;
