@@ -1,10 +1,10 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2004
+ * Copyright (c) 2002-2006
  *	Sleepycat Software.  All rights reserved.
  *
- * $Id: DbCursorTestBase.java,v 1.80 2004/08/27 21:49:28 linda Exp $
+ * $Id: DbCursorTestBase.java,v 1.91 2006/01/03 21:56:14 bostic Exp $
  */
 
 package com.sleepycat.je.dbi;
@@ -29,10 +29,10 @@ import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.LockStats;
 import com.sleepycat.je.OperationStatus;
+import com.sleepycat.je.VerifyConfig;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.tree.BIN;
 import com.sleepycat.je.tree.DuplicateEntryException;
-import com.sleepycat.je.tree.Key;
 import com.sleepycat.je.tree.Node;
 import com.sleepycat.je.tree.Tree;
 import com.sleepycat.je.util.StringDbt;
@@ -67,7 +67,7 @@ public class DbCursorTestBase extends TestCase {
     public DbCursorTestBase() 
     	throws DatabaseException {
 
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
@@ -104,7 +104,7 @@ public class DbCursorTestBase extends TestCase {
         }
 
         // Set up an environment
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
         envConfig.setTransactional(true);
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "6");
@@ -164,7 +164,11 @@ public class DbCursorTestBase extends TestCase {
 	}
 
 	if (exampleDb != null) {
-	    exampleDb.close();
+            try {
+                exampleDb.close();
+            } catch (Throwable e) {
+                System.out.println("tearDown: " + e);
+            }
 	    exampleDb = null;
 	}
         if (exampleEnv != null) {
@@ -309,9 +313,9 @@ public class DbCursorTestBase extends TestCase {
 		    status = handleEndOfSet(status);
 		}
 	    }
-            TestUtils.validateMemoryUsage(DbInternal.
-                                          envGetEnvironmentImpl(exampleEnv),
-                                          false);
+            TestUtils.validateNodeMemUsage(DbInternal.
+					   envGetEnvironmentImpl(exampleEnv),
+					   false);
 	}
 
 	void perData(String foundKey, String foundData)
@@ -485,9 +489,9 @@ public class DbCursorTestBase extends TestCase {
 	throws DatabaseException {
 
 	for (int i = 0; i < nKeys; i++) {
-	    Key key = new Key(new byte[N_KEY_BYTES]);
-	    TestUtils.generateRandomAlphaBytes(key.getKey());
-	    String keyString = new String(key.getKey());
+	    byte[] key = new byte[N_KEY_BYTES];
+	    TestUtils.generateRandomAlphaBytes(key);
+	    String keyString = new String(key);
 	    String dataString = Integer.toString(i);
 	    putAndVerifyCursor(cursor, new StringDbt(keyString),
                                new StringDbt(dataString), true);
@@ -503,12 +507,12 @@ public class DbCursorTestBase extends TestCase {
     protected void doLargePutPerf(int nKeys)
 	throws DatabaseException {
 
-	Key[] keys = new Key[nKeys];
+	byte[][] keys = new byte[nKeys][];
 	for (int i = 0; i < nKeys; i++) {
-	    Key key = new Key(new byte[20]);
+	    byte[] key = new byte[20];
 	    keys[i] = key;
-	    TestUtils.generateRandomAlphaBytes(key.getKey());
-	    String keyString = new String(key.getKey());
+	    TestUtils.generateRandomAlphaBytes(key);
+	    String keyString = new String(key);
 	    byte[] dataBytes = new byte[120];
 	    TestUtils.generateRandomAlphaBytes(dataBytes);
 	    String dataString = new String(dataBytes);
@@ -583,12 +587,12 @@ public class DbCursorTestBase extends TestCase {
 	    createSomeNonDupes = true;
 	}
 
-        Key[] keys = new Key[nTopKeys];
+        byte[][] keys = new byte[nTopKeys][];
         for (int i = 0; i < nTopKeys; i++) {
-            Key key = new Key(new byte[N_KEY_BYTES]);
+            byte[] key = new byte[N_KEY_BYTES];
             keys[i] = key;
-            TestUtils.generateRandomAlphaBytes(key.getKey());
-	    String keyString = new String(key.getKey());
+            TestUtils.generateRandomAlphaBytes(key);
+	    String keyString = new String(key);
 	    Hashtable ht = new Hashtable();
 	    if (dataMap != null) {
 		dataMap.put(keyString, ht);
@@ -630,20 +634,28 @@ public class DbCursorTestBase extends TestCase {
 	Enumeration e = dataMap.keys();
 	while (e.hasMoreElements()) {
 	    String key = (String) e.nextElement();
-	    if (!retrieveData(tree, new Key(key.getBytes()))) {
+	    if (!retrieveData(tree, key.getBytes())) {
 		System.out.println("Couldn't find: " + key);
 	    }
 	}
     }
 
+    /* Throw assertion if the database is not valid. */
+    protected void validateDatabase() 
+        throws DatabaseException {
+
+        DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(exampleDb);
+        assertTrue(dbImpl.verify(new VerifyConfig(), dbImpl.getEmptyStats()));
+    }
+
     /**
      * Helper routine for above.
      */
-    protected boolean retrieveData(Tree tree, Key key)
+    protected boolean retrieveData(Tree tree, byte[] key)
 	throws DatabaseException {
 
 	TestUtils.checkLatchCount();
-	Node n = tree.search(key, Tree.SearchType.NORMAL, -1);
+	Node n = tree.search(key, Tree.SearchType.NORMAL, -1, null, true);
 	if (!(n instanceof BIN)) {
 	    fail("search didn't return a BIN for key: " + key);
 	}
