@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: FileManagerTest.java,v 1.48 2004/07/20 01:24:02 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: FileManagerTest.java,v 1.62 2006/01/03 21:56:21 bostic Exp $
+ */
 
 package com.sleepycat.je.log;
 
@@ -16,7 +16,12 @@ import java.util.Set;
 
 import junit.framework.TestCase;
 
+import com.sleepycat.je.Database;
+import com.sleepycat.je.DatabaseConfig;
+import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
+import com.sleepycat.je.DbInternal;
+import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.EnvironmentImpl;
@@ -36,7 +41,7 @@ public class FileManagerTest extends TestCase {
 
     public FileManagerTest() {
         super();
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     protected void setUp()
@@ -45,10 +50,11 @@ public class FileManagerTest extends TestCase {
         /* Remove files to start with a clean slate. */
         TestUtils.removeFiles("Setup", envHome, FileManager.JE_SUFFIX);
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+	DbInternal.disableParameterValidation(envConfig);
         envConfig.setConfigParam(EnvironmentParams.LOG_FILE_MAX.getName(),
                                  new Integer(FILE_SIZE).toString());
-        /* Yank the cache size way down */
+        /* Yank the cache size way down. */
         envConfig.setConfigParam
 	    (EnvironmentParams.LOG_FILE_CACHE_SIZE.getName(), "3");
         envConfig.setAllowCreate(true);
@@ -56,12 +62,13 @@ public class FileManagerTest extends TestCase {
 
         /* Make a standalone file manager for this test. */
         envImpl.close();
+        envImpl.open(); /* Just sets state to OPEN. */
         fileManager = new FileManager(envImpl, envHome, false);
 
         /* 
-         * Remove any files after the environment is created again!
-         * We want to remove the files made by recovery, so we can
-         * test the file manager in controlled cases.
+         * Remove any files after the environment is created again!  We want to
+         * remove the files made by recovery, so we can test the file manager
+         * in controlled cases.
          */
         TestUtils.removeFiles("Setup", envHome, FileManager.JE_SUFFIX);
     }
@@ -110,78 +117,79 @@ public class FileManagerTest extends TestCase {
         */
 
         try {
-            // Should start out at lsn 0
+            /* Should start out at LSN 0. */
                           
-            // "add" some entries to the log
+            /* "add" some entries to the log. */
             long hdrSize = FileManager.firstLogEntryOffset();
 
             fileManager.bumpLsn(30L);
-            // item placed here
-            assertEquals(new DbLsn(0, hdrSize),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(0, hdrSize),
                          fileManager.getLastUsedLsn());
-            // prev entry
+            /* prev entry. */
             assertEquals(0, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(50L);
-            // item placed here
-            assertEquals(new DbLsn(0, (hdrSize + 30)),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(0, (hdrSize + 30)),
                          fileManager.getLastUsedLsn());
             assertEquals(hdrSize, fileManager.getPrevEntryOffset());
 
-            // bump over to a file 1
+            /* bump over to a file 1. */
             fileManager.bumpLsn(40L);
-            // item placed here
-            assertEquals(new DbLsn(1, hdrSize),
+            /* item placed here. */
+            assertEquals(DbLsn.makeLsn(1, hdrSize),
                          fileManager.getLastUsedLsn());
             assertEquals(0, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(20L);
-            // item placed here
-            assertEquals(new DbLsn(1,(hdrSize+40)),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(1,(hdrSize+40)),
                          fileManager.getLastUsedLsn());
             assertEquals(hdrSize, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(10L);
-            // item placed here
-            assertEquals(new DbLsn(1,(hdrSize+60)),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(1,(hdrSize+60)),
                          fileManager.getLastUsedLsn());
             assertEquals(hdrSize+40, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(5L);
-            // item placed here
-            assertEquals(new DbLsn(1,(hdrSize+70)),
+            /* item placed here. */
+            assertEquals(DbLsn.makeLsn(1,(hdrSize+70)),
                          fileManager.getLastUsedLsn());
             assertEquals(hdrSize+60, fileManager.getPrevEntryOffset());
 
-            // bump over to file 2
+            /* bump over to file 2. */
             fileManager.bumpLsn(75L);
-            // item placed here
-            assertEquals(new DbLsn(2, hdrSize),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(2, hdrSize),
                          fileManager.getLastUsedLsn());
             assertEquals(0, fileManager.getPrevEntryOffset());
 
-            // Ask for something bigger than a file: bump over to file 3
+            /* Ask for something bigger than a file: bump over to file 3. */
             fileManager.bumpLsn(125L);
-            // item placed here
-            assertEquals(new DbLsn(3, hdrSize),
+            /* item placed here. */
+            assertEquals(DbLsn.makeLsn(3, hdrSize),
                          fileManager.getLastUsedLsn());      
             assertEquals(0, fileManager.getPrevEntryOffset());
 
-            // bump over to file 4
+            /* bump over to file 4. */
             fileManager.bumpLsn(10L);
-            // item placed here
-            assertEquals(new DbLsn(4, hdrSize), fileManager.getLastUsedLsn());
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(4, hdrSize),
+			 fileManager.getLastUsedLsn());
             assertEquals(0, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(20L);
-            // item placed here
-            assertEquals(new DbLsn(4, (hdrSize+10)),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(4, (hdrSize+10)),
 			 fileManager.getLastUsedLsn());
             assertEquals(hdrSize, fileManager.getPrevEntryOffset());
 
             fileManager.bumpLsn(30L);
-            // item placed here
-            assertEquals(new DbLsn(4, (hdrSize+30)),
+            /* Item placed here. */
+            assertEquals(DbLsn.makeLsn(4, (hdrSize+30)),
 			 fileManager.getLastUsedLsn());
             assertEquals((hdrSize+10), fileManager.getPrevEntryOffset());
 
@@ -200,18 +208,19 @@ public class FileManagerTest extends TestCase {
         /*
          * Pretend that the last file is file 79.
          */
-        fileManager.setLastPosition(new DbLsn(79L, 88L),  // next available Lsn
-                                    new DbLsn(79L, 77),
+        fileManager.setLastPosition(// next available LSN
+				    DbLsn.makeLsn(79L, 88L),
+                                    DbLsn.makeLsn(79L, 77),
                                     66L);
 
-        // put an entry down, should fit within file 79
+        /* Put an entry down, should fit within file 79. */
         fileManager.bumpLsn(11L);
-        assertEquals(new DbLsn(79L, 88L), fileManager.getLastUsedLsn());
+        assertEquals(DbLsn.makeLsn(79L, 88L), fileManager.getLastUsedLsn());
         assertEquals(77L, fileManager.getPrevEntryOffset());
 
-        // put another entry in, should go to the next file
+        /* Put another entry in, should go to the next file. */
         fileManager.bumpLsn(22L);
-        assertEquals(new DbLsn(80L, FileManager.firstLogEntryOffset()),
+        assertEquals(DbLsn.makeLsn(80L, FileManager.firstLogEntryOffset()),
                      fileManager.getLastUsedLsn());
         assertEquals(0, fileManager.getPrevEntryOffset());
     }
@@ -242,7 +251,7 @@ public class FileManagerTest extends TestCase {
 
         assertEquals("Should have two files", 2, jeFiles.length);
 
-        // Make a fake files with confusing names.
+        /* Make a fake files with confusing names. */
         File fakeFile1 = new File(envHome, "00000abx.jdb");
         File fakeFile2 = new File(envHome, "10.10.jdb");
         fakeFile1.createNewFile();
@@ -260,7 +269,7 @@ public class FileManagerTest extends TestCase {
         file0Handle.release();
         file1Handle.release();
 
-        // Empty the cache and get them again
+        /* Empty the cache and get them again. */
         fileManager.clear();
         file0Handle = fileManager.getFileHandle(0L);
         file1Handle = fileManager.getFileHandle(1L);
@@ -282,14 +291,14 @@ public class FileManagerTest extends TestCase {
     public void testLastFile()
         throws IOException, DatabaseException {
 
-        // There shouldn't be any files here anymore
+        /* There shouldn't be any files here anymore. */
         String [] jeFiles = fileManager.listFiles(FileManager.JE_SUFFIXES);
         assertTrue(jeFiles.length == 0);
 
-        // No files exist, should get null
+        /* No files exist, should get null. */
         assertNull("No last file", fileManager.getLastFileNum());
 
-        // Create some files, ask for the largest
+        /* Create some files, ask for the largest. */
         File fakeFile1 = new File(envHome, "108.cif");
         fakeFile1.createNewFile();
         FileManagerTestUtils.createLogFile(fileManager, envImpl, FILE_SIZE);
@@ -307,17 +316,17 @@ public class FileManagerTest extends TestCase {
     public void testFollowingFile()
         throws IOException, DatabaseException {
 
-        // There shouldn't be any files here anymore
+        /* There shouldn't be any files here anymore. */
         String [] jeFiles = fileManager.listFiles(FileManager.JE_SUFFIXES);
         assertTrue(jeFiles.length == 0);
 
-        // No files exist, should get null
+        /* No files exist, should get null. */
         assertNull("No last file", fileManager.getFollowingFileNum(0, true));
         assertNull("No last file", fileManager.getFollowingFileNum(0, false));
         assertNull("No last file", fileManager.getFollowingFileNum(1, true));
         assertNull("No last file", fileManager.getFollowingFileNum(-1, false));
 
-        // Create some files
+        /* Create some files. */
         File okFile1 = new File(envHome, "00000001.jdb");
         okFile1.createNewFile();
 
@@ -363,7 +372,7 @@ public class FileManagerTest extends TestCase {
     public void testBadHeader()
         throws IOException, DatabaseException {
 
-        // First try a bad environment r/w
+        /* First try a bad environment r/w. */
         try {
             FileManager test =
                 new FileManager(envImpl, new File("xxyy"), true);
@@ -372,7 +381,7 @@ public class FileManagerTest extends TestCase {
             /* should throw */
         }
         
-        // Next try a bad environment r/o
+        /* Next try a bad environment r/o. */
         try {
             FileManager test =
                 new FileManager(envImpl, new File("xxyy"), false);
@@ -381,16 +390,15 @@ public class FileManagerTest extends TestCase {
             /* should throw */
         }
         
-        // Now create a file, but mess up the header
+        /* Now create a file, but mess up the header. */
         FileManagerTestUtils.createLogFile(fileManager, envImpl, FILE_SIZE);
 
         byte [] badData = new byte[]{1,1};
         int headerSize =  FileManager.firstLogEntryOffset();
-        int middleOfHeader = headerSize/2;
         RandomAccessFile file0 = 
             new RandomAccessFile
 		(fileManager.getFullFileName(0, FileManager.JE_SUFFIX),
-		 FileManager.READWRITE_MODE);
+		 FileManager.FileMode.READWRITE_MODE.getModeValue());
         file0.write(badData);
         file0.close();
         fileManager.clear();
@@ -412,7 +420,7 @@ public class FileManagerTest extends TestCase {
         RandomAccessFile file0 = 
             new RandomAccessFile
 		(fileManager.getFullFileName(0, FileManager.JE_SUFFIX),
-		 FileManager.READWRITE_MODE);
+		 FileManager.FileMode.READWRITE_MODE.getModeValue());
         file0.getChannel().truncate(FileManager.firstLogEntryOffset()/2);
         file0.close();
 
@@ -432,8 +440,7 @@ public class FileManagerTest extends TestCase {
         try {
 
             /* 
-             * Make five log files. The file descriptor cache should
-             * be empty.
+             * Make five log files. The file descriptor cache should be empty.
              */
             FileManagerTestUtils.createLogFile
 		(fileManager, envImpl, FILE_SIZE);
@@ -488,8 +495,8 @@ public class FileManagerTest extends TestCase {
             assertTrue(keySet.contains(f3));
 
             /* 
-             * Now ask for another file. The cache should not grow,
-             * because no handles are locked and there's room to evict one.
+             * Now ask for another file. The cache should not grow, because no
+             * handles are locked and there's room to evict one.
              */
             f0Handle.release();
             f1Handle.release();
@@ -511,6 +518,37 @@ public class FileManagerTest extends TestCase {
         }
     }
 
+    public void testFlipFile()
+	throws Throwable {
+
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setTransactional(true);
+        Environment env = new Environment(envHome, envConfig);
+	EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
+	FileManager fileManager = envImpl.getFileManager();
+
+        DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+        Database exampleDb =
+	    env.openDatabase(null, "simpleDb", dbConfig);
+
+        assertEquals("Should have 0 as current file", 0L,
+                     fileManager.getCurrentFileNum());
+	long flipLsn = envImpl.forceLogFileFlip();
+	assertEquals("LSN should be 1 post-flip", 1L,
+		     DbLsn.getFileNumber(flipLsn));
+	DatabaseEntry key = new DatabaseEntry();
+	DatabaseEntry data = new DatabaseEntry();
+	key.setData("key".getBytes());
+	data.setData("data".getBytes());
+	exampleDb.put(null, key, data);
+        assertEquals("Should have 1 as last file", 1L,
+                     fileManager.getCurrentFileNum());
+	exampleDb.close();
+	env.close();
+    }	
+
     class CachingThread extends Thread {
         private FileManager fileManager;
         private long fileNum;
@@ -520,7 +558,6 @@ public class FileManagerTest extends TestCase {
         CachingThread(FileManager fileManager, long fileNum) {
             this.fileManager = fileManager;
             this.fileNum = fileNum;
-    
         }
 
         public void run() {
