@@ -1,10 +1,10 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2004
+ * Copyright (c) 2002-2006
  *      Sleepycat Software.  All rights reserved.
  *
- * $Id: RecoveryTestBase.java,v 1.96 2004/07/30 15:53:36 cwl Exp $
+ * $Id: RecoveryTestBase.java,v 1.107 2006/01/03 21:56:24 bostic Exp $
  */
 
 package com.sleepycat.je.recovery;
@@ -36,6 +36,7 @@ import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
+import com.sleepycat.je.XAEnvironment;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.log.FileManager;
@@ -69,9 +70,9 @@ public class RecoveryTestBase extends TestCase {
     }
 
     private void init() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
         Key.DUMP_BINARY = true;
-        envConfig = new EnvironmentConfig();
+        envConfig = TestUtils.initEnvConfig();
         forceConfig = new CheckpointConfig();
         forceConfig.setForce(true);
     }
@@ -102,13 +103,28 @@ public class RecoveryTestBase extends TestCase {
 
     /**
      * Make an environment and databases, commit the db creation by default.
-     * Running with or without the checkpoint daemon changes how recovery
-     * is exercised.
+     * Running with or without the checkpoint daemon changes how recovery is
+     * exercised.
      */
     protected void createEnv(int fileSize, boolean runCheckpointDaemon)
         throws DatabaseException {
 
-        /* Make an environment and open it */
+	createEnvInternal(fileSize, runCheckpointDaemon, false);
+    }
+
+    protected void createXAEnv(int fileSize, boolean runCheckpointDaemon)
+        throws DatabaseException {
+
+	createEnvInternal(fileSize, runCheckpointDaemon, true);
+    }
+
+    private void createEnvInternal(int fileSize,
+				   boolean runCheckpointDaemon,
+				   boolean createXAEnv)
+        throws DatabaseException {
+
+        /* Make an environment and open it. */
+	DbInternal.disableParameterValidation(envConfig);
         envConfig.setTransactional(true);
         envConfig.setAllowCreate(true);
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
@@ -128,7 +144,11 @@ public class RecoveryTestBase extends TestCase {
 		(EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
         }
         setExtraProperties();
-        env = new Environment(envHome, envConfig);
+	if (createXAEnv) {
+	    env = new XAEnvironment(envHome, envConfig);
+	} else {
+	    env = new Environment(envHome, envConfig);
+	}
     }
 
     /*
@@ -145,7 +165,7 @@ public class RecoveryTestBase extends TestCase {
     protected void createDbs(Transaction txn, int numDbs)
         throws DatabaseException {
 
-        // Make a db and open it
+        /* Make a db and open it. */
         dbs = new Database[numDbs];
 
         DatabaseConfig dbConfig = new DatabaseConfig();
@@ -168,7 +188,26 @@ public class RecoveryTestBase extends TestCase {
                                    int numDbs) 
         throws DatabaseException {
 
-        createEnv(fileSize, runCheckpointerDaemon);
+	createEnvAndDbsInternal(fileSize, runCheckpointerDaemon,
+				numDbs, false);
+    }
+
+    protected void createXAEnvAndDbs(int fileSize,
+				     boolean runCheckpointerDaemon,
+				     int numDbs) 
+        throws DatabaseException {
+
+	createEnvAndDbsInternal(fileSize, runCheckpointerDaemon,
+				numDbs, true);
+    }
+
+    protected void createEnvAndDbsInternal(int fileSize,
+					   boolean runCheckpointerDaemon,
+					   int numDbs,
+					   boolean createXAEnv)
+        throws DatabaseException {
+
+        createEnvInternal(fileSize, runCheckpointerDaemon, createXAEnv);
         Transaction txn = env.beginTransaction(null, null);
         createDbs(txn, numDbs);
         txn.commit();
@@ -180,7 +219,7 @@ public class RecoveryTestBase extends TestCase {
     protected void closeEnv()
         throws DatabaseException {
 
-        TestUtils.validateMemoryUsage(DbInternal.envGetEnvironmentImpl(env),
+        TestUtils.validateNodeMemUsage(DbInternal.envGetEnvironmentImpl(env),
                                       false);
 
         /* Close the environment. */
@@ -191,10 +230,16 @@ public class RecoveryTestBase extends TestCase {
                 }
             }
         }
+	forceCloseEnvOnly();
+    }
 
-        /* Close w/out checkpointing, in order to exercise recovery better.*/
-        DbInternal.envGetEnvironmentImpl(env).close(false);
-        env = null;
+    /* Force the environment to be closed even if with outstanding handles.*/
+    protected void forceCloseEnvOnly() 
+	throws DatabaseException {
+
+	/* Close w/out checkpointing, in order to exercise recovery better.*/
+	DbInternal.envGetEnvironmentImpl(env).close(false);
+	env = null;
     }
 
     /*
@@ -204,18 +249,64 @@ public class RecoveryTestBase extends TestCase {
     protected List recoverAndVerify(Hashtable expectedData, int numDbs)
         throws DatabaseException {
 
-        List infoList = new ArrayList();
+	return recoverAndVerifyInternal(expectedData, numDbs, 
+                                        false,  // XA
+                                        false); // readOnly
+    }
 
-        // open it again, which will run recovery
-        EnvironmentConfig recoveryConfig = new EnvironmentConfig();
+    protected List recoverROAndVerify(Hashtable expectedData, int numDbs)
+        throws DatabaseException {
+
+	return recoverAndVerifyInternal(expectedData, numDbs, 
+                                        false,  // XA
+                                        true);  // readOnly
+    }
+
+    /*
+     * Recover the databases and check the data. Return a list of the
+     * RecoveryInfos generated by each recovery.
+     */
+    protected List xaRecoverAndVerify(Hashtable expectedData, int numDbs)
+        throws DatabaseException {
+
+	return recoverAndVerifyInternal(expectedData, numDbs, 
+			                true,   // XA
+					false); // readOnly
+    }
+
+    private List recoverAndVerifyInternal(Hashtable expectedData,
+					  int numDbs,
+					  boolean createXAEnv,
+                                          boolean readOnlyMode)
+        throws DatabaseException {
+
+	List infoList = recoverOnlyInternal(numDbs, createXAEnv, readOnlyMode);
+        verifyData(expectedData, numDbs);
+	TestUtils.validateNodeMemUsage(DbInternal.envGetEnvironmentImpl(env),
+                                      false);	
+        /* Run verify again. */
+        DbInternal.envGetEnvironmentImpl(env).close(false);
+        env = new Environment(envHome, getRecoveryConfig(readOnlyMode));
+        EnvironmentImpl envImpl = 
+	    DbInternal.envGetEnvironmentImpl(env);
+        infoList.add(envImpl.getLastRecoveryInfo());
+        verifyData(expectedData, numDbs);
+        TestUtils.validateNodeMemUsage(envImpl, false);
+        env.close();
+	return infoList;
+    }
+
+    private EnvironmentConfig getRecoveryConfig(boolean readOnlyMode) {
+        EnvironmentConfig recoveryConfig = TestUtils.initEnvConfig();
         recoveryConfig.setConfigParam
             (EnvironmentParams.NODE_MAX.getName(), "6");
 	recoveryConfig.setConfigParam(EnvironmentParams.MAX_MEMORY.getName(),
 				      new Long(1 << 24).toString());
+        recoveryConfig.setReadOnly(readOnlyMode);
 
-        /* 
-         * Don't run checkLeaks, because verify is running while the
-         * system is not quiescent. The other daemons are running.
+        /*
+         * Don't run checkLeaks, because verify is running while the system is
+         * not quiescent. The other daemons are running.
          */
         recoveryConfig.setConfigParam
 	    (EnvironmentParams.ENV_CHECK_LEAKS.getName(), "false");
@@ -230,31 +321,52 @@ public class RecoveryTestBase extends TestCase {
             recoveryConfig.setConfigParam
                 (EnvironmentParams.JE_LOGGING_LEVEL.getName(), "FINE");
         }
-        env = new Environment(envHome, recoveryConfig);
-        TestUtils.validateMemoryUsage(DbInternal.envGetEnvironmentImpl(env),
+
+	recoveryConfig.setTransactional(true);
+	return recoveryConfig;
+    }
+
+    protected List recoverOnly(int numDbs)
+	throws DatabaseException {
+
+	return recoverOnlyInternal(numDbs, 
+                                   false,   // XA
+                                   false);  // read only
+    }
+
+    protected List xaRecoverOnly(int numDbs)
+	throws DatabaseException {
+
+	return recoverOnlyInternal(numDbs, 
+                                   true,   // XA
+                                   false); // read only
+    }
+
+    private List recoverOnlyInternal(int numDbs,
+                                     boolean createXAEnv,
+                                     boolean readOnlyMode)
+	throws DatabaseException {
+
+        List infoList = new ArrayList();
+
+        /* Open it again, which will run recovery. */
+	if (createXAEnv) {
+	    env = new XAEnvironment(envHome, getRecoveryConfig(readOnlyMode));
+	} else {
+	    env = new Environment(envHome, getRecoveryConfig(readOnlyMode));
+	}
+        TestUtils.validateNodeMemUsage(DbInternal.envGetEnvironmentImpl(env),
                                       false);
 
         infoList.add
             (DbInternal.envGetEnvironmentImpl(env).getLastRecoveryInfo());
-        verifyData(expectedData, numDbs);
-	TestUtils.validateMemoryUsage(DbInternal.envGetEnvironmentImpl(env),
-                                      false);	
-        // Run verify again
-        DbInternal.envGetEnvironmentImpl(env).close(false);
-        env = new Environment(envHome, recoveryConfig);
-        EnvironmentImpl envImpl = 
-	    DbInternal.envGetEnvironmentImpl(env);
-        infoList.add(envImpl.getLastRecoveryInfo());
-        verifyData(expectedData, numDbs);
-        TestUtils.validateMemoryUsage(envImpl, false);
-        env.close();
         
         return infoList;
     }
 
     /**
-     * Compare the data in the databases agains the data in the expected
-     * data set.
+     * Compare the data in the databases agains the data in the expected data
+     * set.
      */
     protected void verifyData(Hashtable expectedData, int numDbs) 
         throws DatabaseException  {
@@ -263,19 +375,28 @@ public class RecoveryTestBase extends TestCase {
     }
 
     /**
-     * Compare the data in the databases against the data in the expected
-     * data set.
+     * Compare the data in the databases against the data in the expected data
+     * set.
      */
     protected void verifyData(Hashtable expectedData,
                               boolean checkInList,
                               int numDbs)
         throws DatabaseException  {
 
-        // Run verify
+	verifyData(expectedData, checkInList, 0, numDbs);
+    }
+
+    protected void verifyData(Hashtable expectedData,
+                              boolean checkInList,
+			      int startDb,
+                              int endDb)
+        throws DatabaseException  {
+
+        /* Run verify. */
         if (checkInList) {
-            env.verify(null, System.err);
+            assertTrue(env.verify(null, System.err));
         } else {
-            env.verify(null, System.err);
+            assertTrue(env.verify(null, System.err));
         }
 
         /*
@@ -294,14 +415,15 @@ public class RecoveryTestBase extends TestCase {
         /* Generate an expected count map. */
         Map countMap = generateCountMap(expectedData);
 
-        /* Check each db in turn */
+        /* Check each db in turn. */
         DatabaseConfig dbConfig = new DatabaseConfig();
 	if (btreeComparisonFunction != null) {
 	    dbConfig.setBtreeComparator(btreeComparisonFunction.getClass());
 	}
         dbConfig.setTransactional(env.getConfig().getTransactional());
         dbConfig.setSortedDuplicates(true);
-        for (int d = 0; d < numDbs; d++) {
+        dbConfig.setReadOnly(true);
+        for (int d = startDb; d < endDb; d++) {
             Database checkDb = env.openDatabase(null, DB_NAME + d,
 						dbConfig);
             Cursor myCursor = checkDb.openCursor(null, null);
@@ -314,7 +436,7 @@ public class RecoveryTestBase extends TestCase {
 
             while (status == OperationStatus.SUCCESS) {
 
-                /* The key should have been in the expected data set */
+                /* The key should have been in the expected data set. */
                 removeExpectedData(useData, d, key, data, true);
 
                 /* The count should be right. */
@@ -387,13 +509,30 @@ public class RecoveryTestBase extends TestCase {
                               boolean addToExpectedData,
                               int numDbs)
         throws DatabaseException {
+
         insertData(txn, startVal, endVal, expectedData,
                    nDuplicatesPerKey, false, addToExpectedData,
-                   numDbs);
+                   0, numDbs);
+    }
+
+    protected void insertData(Transaction txn,
+                              int startVal,
+                              int endVal,
+                              Map expectedData,
+                              int nDuplicatesPerKey,
+                              boolean addToExpectedData,
+                              int startDb,
+			      int endDb)
+        throws DatabaseException {
+
+        insertData(txn, startVal, endVal, expectedData,
+                   nDuplicatesPerKey, false, addToExpectedData,
+                   startDb, endDb);
     }
 
     /**
      * Insert data over many databases.
+     *
      * @param toggle if true, insert every other value.
      */
     protected void insertData(Transaction txn,
@@ -406,20 +545,40 @@ public class RecoveryTestBase extends TestCase {
                               int numDbs)
         throws DatabaseException {
 
-        Cursor[] cursors = getCursors(txn, numDbs);
+	insertData(txn, startVal, endVal, expectedData, nDuplicatesPerKey,
+		   toggle, addToExpectedData, 0, numDbs);
+    }
 
-        // Make sure this test inserts something!
+    /**
+     * Insert data over many databases.
+     *
+     * @param toggle if true, insert every other value.
+     */
+    protected void insertData(Transaction txn,
+                              int startVal,
+                              int endVal,
+                              Map expectedData,
+                              int nDuplicatesPerKey,
+                              boolean toggle,
+                              boolean addToExpectedData, 
+                              int startDb,
+			      int endDb)
+        throws DatabaseException {
+
+        Cursor[] cursors = getCursors(txn, startDb, endDb);
+
+        /* Make sure this test inserts something! */
         assertTrue(endVal - startVal > -1);
 
-        // are we inserting in an ascending or descending way?
+        /* Are we inserting in an ascending or descending way? */
         int incVal = (toggle) ? 2 : 1;
         if (startVal < endVal) {
-            for (int i = startVal; i <= endVal; i+=incVal) {
+            for (int i = startVal; i <= endVal; i += incVal) {
                 insertOneRecord(cursors, i, expectedData,
                                 nDuplicatesPerKey, addToExpectedData);
             }
         } else {
-            for (int i = startVal; i >= endVal; i-=incVal) {
+            for (int i = startVal; i >= endVal; i -= incVal) {
                 insertOneRecord(cursors, i, expectedData,
                                 nDuplicatesPerKey, addToExpectedData);
             }
@@ -431,9 +590,9 @@ public class RecoveryTestBase extends TestCase {
     }
 
     /**
-     * Add to the set of expected results. ExpectedData is keyed by
-     * a TestData object that wraps db number and key, and points to
-     * sets of TestData objects that wrap db number, key, and data.
+     * Add to the set of expected results. ExpectedData is keyed by a TestData
+     * object that wraps db number and key, and points to sets of TestData
+     * objects that wrap db number, key, and data.
      */
     protected void addExpectedData(Map expectedData,
 				   int dbNum,
@@ -453,7 +612,7 @@ public class RecoveryTestBase extends TestCase {
     }
 
     /**
-     * Remove from the set of expected results
+     * Remove from the set of expected results.
      */
     private void removeExpectedData(Map expectedData,
 				    int dbNum,
@@ -477,12 +636,12 @@ public class RecoveryTestBase extends TestCase {
     /**
      * @return a set of cursors for the test databases.
      */
-    private Cursor[] getCursors(Transaction txn, int numDbs)
+    private Cursor[] getCursors(Transaction txn, int startDb, int endDb)
         throws DatabaseException {
 
-        Cursor[] cursors = new Cursor[numDbs];
+        Cursor[] cursors = new Cursor[endDb - startDb];
         for (int i = 0; i < cursors.length; i++) {
-            cursors[i] = dbs[i].openCursor(txn, null);
+            cursors[i] = dbs[startDb + i].openCursor(txn, null);
         }
         return cursors;
     }
@@ -503,8 +662,8 @@ public class RecoveryTestBase extends TestCase {
         for (int c = 0; c < cursors.length; c++) {
 
             int testVal = val + c;
-            byte [] keyData = TestUtils.getTestArray(testVal);
-            byte [] dataData = TestUtils.byteArrayCopy(keyData);
+            byte[] keyData = TestUtils.getTestArray(testVal);
+            byte[] dataData = TestUtils.byteArrayCopy(keyData);
             key.setData(keyData);
             for (int d = 0; d < nDuplicatesPerKey; d++) {
                 dataData = TestUtils.byteArrayCopy(dataData);
@@ -527,7 +686,8 @@ public class RecoveryTestBase extends TestCase {
     protected void deleteData(Transaction txn, Map expectedData,
                               boolean all, boolean expectCommit, int numDbs)
         throws DatabaseException {
-        Cursor[] cursors = getCursors(txn, numDbs);
+
+        Cursor[] cursors = getCursors(txn, 0, numDbs);
         DatabaseEntry key = new DatabaseEntry();
         DatabaseEntry data = new DatabaseEntry();
 
@@ -548,7 +708,7 @@ public class RecoveryTestBase extends TestCase {
                 }
                 status = cursors[d].getNext(key, data, LockMode.DEFAULT);
             }
-            // Make sure the test deletes something!
+            /* Make sure the test deletes something! */
             assertTrue(deleteCount > 0);
         }
 
@@ -571,22 +731,22 @@ public class RecoveryTestBase extends TestCase {
                               boolean expectCommit, int numDbs) 
         throws DatabaseException {
 
-        Cursor[] cursors = getCursors(txn, numDbs);
+        Cursor[] cursors = getCursors(txn, 0, numDbs);
         DatabaseEntry key = new DatabaseEntry();
         DatabaseEntry data = new DatabaseEntry();
 
         for (int d = 0; d < cursors.length; d++) {
 
-            // position cursor at the start value.
+            /* Position cursor at the start value. */
             OperationStatus status =
 		cursors[d].getFirst(key, data, LockMode.DEFAULT);
 
-            // for each record within the range, change the data.
+            /* For each record within the range, change the data. */
             int modCount = 0;
             int keyVal = TestUtils.getTestVal(key.getData());
             while ((status == OperationStatus.SUCCESS) && (keyVal <= endVal)) {
 
-                // Change the data
+                /* Change the data. */
                 removeExpectedData(expectedData, d, key, data, expectCommit);
                 data.setData(TestUtils.getTestArray(keyVal + increment));
                 cursors[d].delete();
@@ -600,7 +760,7 @@ public class RecoveryTestBase extends TestCase {
                     keyVal = TestUtils.getTestVal(key.getData());
                 }
             }
-            // Make sure we modify something!
+            /* Make sure we modify something! */
             assertTrue(modCount > 0);
         }
 
@@ -666,8 +826,8 @@ public class RecoveryTestBase extends TestCase {
 
     protected class TestData {
         public int dbNum;
-        public byte [] key;
-        public byte [] data;
+        public byte[] key;
+        public byte[] data;
 
         TestData(int dbNum, DatabaseEntry keyDbt, DatabaseEntry dataDbt) {
             this.dbNum = dbNum;
