@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: SecondaryTest.java,v 1.24 2004/08/18 14:41:57 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: SecondaryTest.java,v 1.34 2006/01/04 21:49:53 linda Exp $
+ */
 
 package com.sleepycat.je.test;
 
@@ -19,7 +19,6 @@ import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
-import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
@@ -31,12 +30,12 @@ import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.util.TestUtils;
 
-public class SecondaryTest extends TxnTestCase {
+public class SecondaryTest extends MultiKeyTxnTestCase {
 
     private static final int NUM_RECS = 5;
     private static final int KEY_OFFSET = 100;
 
-    private static EnvironmentConfig envConfig = new EnvironmentConfig();
+    private static EnvironmentConfig envConfig = TestUtils.initEnvConfig();
     static {
         envConfig.setConfigParam(EnvironmentParams.ENV_CHECK_LEAKS.getName(),
                                  "false");
@@ -47,7 +46,8 @@ public class SecondaryTest extends TxnTestCase {
     }
 
     public static Test suite() {
-        return new TxnTestSuite(SecondaryTest.class, envConfig, null);
+
+        return multiKeyTxnTestSuite(SecondaryTest.class, envConfig, null);
     }
 
     public void testPutAndDelete()
@@ -201,6 +201,11 @@ public class SecondaryTest extends TxnTestCase {
         OperationStatus status;
         Transaction txn = txnBegin();
 
+        /*
+         * For parameters that do not require initialization with a non-null
+         * data array, we set them to null to make sure this works. [#12121]
+         */
+
         /* Add one record for each key with one data/duplicate. */
         for (int i = 0; i < NUM_RECS; i += 1) {
             status = priDb.put(txn, entry(i), entry(i));
@@ -210,23 +215,27 @@ public class SecondaryTest extends TxnTestCase {
         /* SecondaryDatabase.get() */
         for (int i = 0; i < NUM_RECS; i += 1) {
 
+            data.setData(null);
             status = secDb.get(txn, entry(i + KEY_OFFSET), key,
                                data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(i), key);
             assertDataEquals(entry(i), data);
         }
+        data.setData(null);
         status = secDb.get(txn, entry(NUM_RECS + KEY_OFFSET), key,
                            data, LockMode.DEFAULT);
         assertSame(OperationStatus.NOTFOUND, status);
 
         /* SecondaryDatabase.getSearchBoth() */
         for (int i = 0; i < NUM_RECS; i += 1) {
+            data.setData(null);
             status = secDb.getSearchBoth(txn, entry(i + KEY_OFFSET), entry(i),
                                          data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(i), data);
         }
+        data.setData(null);
         status = secDb.getSearchBoth(txn, entry(NUM_RECS + KEY_OFFSET),
                                      entry(NUM_RECS), data, LockMode.DEFAULT);
         assertSame(OperationStatus.NOTFOUND, status);
@@ -238,17 +247,26 @@ public class SecondaryTest extends TxnTestCase {
         SecondaryCursor cursor = secDb.openSecondaryCursor(txn, null);
         try {
             /* SecondaryCursor.getFirst()/getNext() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getFirst(secKey, key, data, LockMode.DEFAULT);
             for (int i = 0; i < NUM_RECS; i += 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getNext(secKey, key, data, LockMode.DEFAULT);
             }
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getCurrent() (last) */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getCurrent(secKey, key, data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(NUM_RECS - 1 + KEY_OFFSET), secKey);
@@ -256,17 +274,26 @@ public class SecondaryTest extends TxnTestCase {
             assertDataEquals(entry(NUM_RECS - 1), data);
 
             /* SecondaryCursor.getLast()/getPrev() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getLast(secKey, key, data, LockMode.DEFAULT);
             for (int i = NUM_RECS - 1; i >= 0; i -= 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getPrev(secKey, key, data, LockMode.DEFAULT);
             }
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getCurrent() (first) */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getCurrent(secKey, key, data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(0 + KEY_OFFSET), secKey);
@@ -274,64 +301,82 @@ public class SecondaryTest extends TxnTestCase {
             assertDataEquals(entry(0), data);
 
             /* SecondaryCursor.getSearchKey() */
+            key.setData(null);
+            data.setData(null);
             status = cursor.getSearchKey(entry(KEY_OFFSET - 1), key,
                                          data, LockMode.DEFAULT);
             assertSame(OperationStatus.NOTFOUND, status);
             for (int i = 0; i < NUM_RECS; i += 1) {
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getSearchKey(entry(i + KEY_OFFSET), key,
                                              data, LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
             }
+            key.setData(null);
+            data.setData(null);
             status = cursor.getSearchKey(entry(NUM_RECS + KEY_OFFSET), key,
                                          data, LockMode.DEFAULT);
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getSearchBoth() */
+            data.setData(null);
             status = cursor.getSearchKey(entry(KEY_OFFSET - 1), entry(0),
                                          data, LockMode.DEFAULT);
             assertSame(OperationStatus.NOTFOUND, status);
             for (int i = 0; i < NUM_RECS; i += 1) {
+                data.setData(null);
                 status = cursor.getSearchBoth(entry(i + KEY_OFFSET), entry(i),
                                               data, LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i), data);
             }
+            data.setData(null);
             status = cursor.getSearchBoth(entry(NUM_RECS + KEY_OFFSET),
                                           entry(NUM_RECS), data,
                                           LockMode.DEFAULT);
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getSearchKeyRange() */
+            key.setData(null);
+            data.setData(null);
             status = cursor.getSearchKeyRange(entry(KEY_OFFSET - 1), key,
                                               data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(0), key);
             assertDataEquals(entry(0), data);
             for (int i = 0; i < NUM_RECS; i += 1) {
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getSearchKeyRange(entry(i + KEY_OFFSET), key,
                                                   data, LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
             }
+            key.setData(null);
+            data.setData(null);
             status = cursor.getSearchKeyRange(entry(NUM_RECS + KEY_OFFSET),
                                               key, data, LockMode.DEFAULT);
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getSearchBothRange() */
+            data.setData(null);
             status = cursor.getSearchBothRange(entry(1 + KEY_OFFSET), entry(1),
                                                data, LockMode.DEFAULT);
             assertSame(OperationStatus.SUCCESS, status);
             assertDataEquals(entry(1), data);
             for (int i = 0; i < NUM_RECS; i += 1) {
+                data.setData(null);
                 status = cursor.getSearchBothRange(entry(i + KEY_OFFSET),
                                                    entry(i), data,
                                                    LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i), data);
             }
+            data.setData(null);
             status = cursor.getSearchBothRange(entry(NUM_RECS + KEY_OFFSET),
                                                entry(NUM_RECS), data,
                                                LockMode.DEFAULT);
@@ -349,64 +394,100 @@ public class SecondaryTest extends TxnTestCase {
             }
 
             /* SecondaryCursor.getNextDup() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getFirst(secKey, key, data, LockMode.DEFAULT);
             for (int i = 0; i < NUM_RECS; i += 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getNextDup(secKey, key, data,
                                            LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i + KEY_OFFSET), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getNextDup(secKey, key, data,
                                            LockMode.DEFAULT);
                 assertSame(OperationStatus.NOTFOUND, status);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getNext(secKey, key, data, LockMode.DEFAULT);
             }
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getNextNoDup() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getFirst(secKey, key, data, LockMode.DEFAULT);
             for (int i = 0; i < NUM_RECS; i += 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getNextNoDup(secKey, key, data,
                                              LockMode.DEFAULT);
             }
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getPrevDup() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getLast(secKey, key, data, LockMode.DEFAULT);
             for (int i = NUM_RECS - 1; i >= 0; i -= 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i + KEY_OFFSET), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getPrevDup(secKey, key, data,
                                            LockMode.DEFAULT);
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getPrevDup(secKey, key, data,
                                            LockMode.DEFAULT);
                 assertSame(OperationStatus.NOTFOUND, status);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getPrev(secKey, key, data, LockMode.DEFAULT);
             }
             assertSame(OperationStatus.NOTFOUND, status);
 
             /* SecondaryCursor.getPrevNoDup() */
+            secKey.setData(null);
+            key.setData(null);
+            data.setData(null);
             status = cursor.getLast(secKey, key, data, LockMode.DEFAULT);
             for (int i = NUM_RECS - 1; i >= 0; i -= 1) {
                 assertSame(OperationStatus.SUCCESS, status);
                 assertDataEquals(entry(i + KEY_OFFSET), secKey);
                 assertDataEquals(entry(i + KEY_OFFSET), key);
                 assertDataEquals(entry(i), data);
+                secKey.setData(null);
+                key.setData(null);
+                data.setData(null);
                 status = cursor.getPrevNoDup(secKey, key, data,
                                              LockMode.DEFAULT);
             }
@@ -552,7 +633,6 @@ public class SecondaryTest extends TxnTestCase {
         throws DatabaseException {
 
         Database priDb = openDatabase(false, "testDB", false);
-        OperationStatus status;
         Transaction txn = txnBegin();
 
         /* Test population of newly created secondary database. */
@@ -599,7 +679,6 @@ public class SecondaryTest extends TxnTestCase {
 
         SecondaryDatabase secDb = initDb();
         Database priDb = secDb.getPrimaryDatabase();
-        OperationStatus status;
         Transaction txn = txnBegin();
         
         for (int i = 0; i < NUM_RECS; i += 1) {
@@ -607,11 +686,23 @@ public class SecondaryTest extends TxnTestCase {
         }
         verifyRecords(txn, priDb, NUM_RECS, false);
         verifyRecords(txn, secDb, NUM_RECS, true);
-        assertEquals(NUM_RECS, priDb.truncate(txn, true));
+        txnCommit(txn);
+        secDb.close();
+        priDb.close();
+
+        txn = txnBegin();
+        assertEquals(NUM_RECS, env.truncateDatabase(txn, "testDB", true));
+        assertEquals(NUM_RECS, env.truncateDatabase(txn, "testSecDB", true));
+        txnCommit(txn);
+
+        secDb = initDb();
+        priDb = secDb.getPrimaryDatabase();
+
+        txn = txnBegin();
         verifyRecords(txn, priDb, 0, false);
         verifyRecords(txn, secDb, 0, true);
-
         txnCommit(txn);
+
         secDb.close();
         priDb.close();
     }
@@ -728,6 +819,9 @@ public class SecondaryTest extends TxnTestCase {
         priDb.close();
     }
 
+    /**
+     * @deprecated use of Database.truncate
+     */
     public void testOperationsNotAllowed()
         throws DatabaseException {
 
@@ -745,6 +839,16 @@ public class SecondaryTest extends TxnTestCase {
                                       new SecondaryConfig());
             fail();
         } catch (NullPointerException expected) { }
+
+        /* Open secondary with both single and multi key creators. */
+        SecondaryConfig config = new SecondaryConfig();
+        config.setKeyCreator(new MyKeyCreator());
+        config.setMultiKeyCreator
+            (new SimpleMultiKeyCreator(new MyKeyCreator()));
+        try {
+            env.openSecondaryDatabase(txn, "xxx", priDb, config);
+            fail();
+        } catch (IllegalArgumentException expected) { }
         
         /* Database operations. */
 
@@ -837,6 +941,18 @@ public class SecondaryTest extends TxnTestCase {
         } catch (IllegalArgumentException expected) {}
 
         priDb.close();
+
+        /* Single secondary with two primaries.*/
+        Database pri1 = openDatabase(false, "pri1", false);
+        Database pri2 = openDatabase(false, "pri2", false);
+        Database sec1 = openSecondary(pri1, false, "sec", false, false);
+        try {
+            openSecondary(pri2, false, "sec", false, false);
+            fail();
+        } catch (IllegalArgumentException expected) {}
+        sec1.close();
+        pri1.close();
+        pri2.close();
     }
 
     /**
@@ -1238,7 +1354,12 @@ public class SecondaryTest extends TxnTestCase {
         dbConfig.setReadOnly(readOnly);
         dbConfig.setAllowPopulate(allowPopulate);
         if (!readOnly) {
-            dbConfig.setKeyCreator(new MyKeyCreator());
+            if (useMultiKey) {
+                dbConfig.setMultiKeyCreator
+                    (new SimpleMultiKeyCreator(new MyKeyCreator()));
+            } else {
+                dbConfig.setKeyCreator(new MyKeyCreator());
+            }
         }
         Transaction txn = txnBegin();
         SecondaryDatabase secDb;
@@ -1269,7 +1390,7 @@ public class SecondaryTest extends TxnTestCase {
     }
 
     private void assertDataEquals(DatabaseEntry e1, DatabaseEntry e2) {
-        assertTrue(DbInternal.dataEquals(e1, e2));
+        assertTrue(e1.equals(e2));
     }
 
     private static class MyKeyCreator implements SecondaryKeyCreator {
