@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: GetParentNodeTest.java,v 1.26 2004/08/21 02:05:39 linda Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: GetParentNodeTest.java,v 1.39 2006/01/03 21:56:29 bostic Exp $
+ */
 
 package com.sleepycat.je.tree;
 
@@ -23,10 +23,9 @@ import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.DatabaseImpl;
-import com.sleepycat.je.latch.Latch;
+import com.sleepycat.je.latch.LatchSupport;
 import com.sleepycat.je.util.StringDbt;
 import com.sleepycat.je.util.TestUtils;
-import com.sleepycat.je.utilint.DbLsn;
 
 public class GetParentNodeTest extends TestCase {
     static private final boolean DEBUG = false;
@@ -40,7 +39,7 @@ public class GetParentNodeTest extends TestCase {
     private DBIN firstDBIN;
 
     public GetParentNodeTest() {
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp() 
@@ -62,7 +61,8 @@ public class GetParentNodeTest extends TestCase {
 
     private void initEnv()
         throws DatabaseException {
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "4");
         envConfig.setTransactional(true);
         envConfig.setAllowCreate(true);
@@ -79,17 +79,16 @@ public class GetParentNodeTest extends TestCase {
     }
 
     /**
-     * Test getParentINForChildIN and GetParentBINForChildLN
-     * painstakingly on a hand constructed tree.
+     * Test getParentINForChildIN and GetParentBINForChildLN painstakingly on a
+     * hand constructed tree.
      */
     public void testBasic()
         throws Exception {
 
         try {
             /* 
-             * Make a tree w/3 levels in the main tree and a single
-             * dup tree. The dupTree has two levels. The tree looks
-             * like this:
+             * Make a tree w/3 levels in the main tree and a single dup
+             * tree. The dupTree has two levels. The tree looks like this:
 	     *
              *            root(key=a)
              *             |
@@ -140,8 +139,8 @@ public class GetParentNodeTest extends TestCase {
                          db.put(null, new StringDbt("k"),
                                 new StringDbt("data1")));
 
-            /* Add one dup tree */
-            Key dupKey = new Key(new String("b").getBytes());
+            /* Add one dup tree. */
+            byte[] dupKey = "b".getBytes();
             assertEquals(OperationStatus.SUCCESS,
                          db.put(null, new StringDbt("b"),
                                 new StringDbt("data2")));
@@ -162,31 +161,31 @@ public class GetParentNodeTest extends TestCase {
 				new StringDbt("data7")));
 
             /* 
-             * Test exact matches 
+             * Test exact matches.
              */
             checkTreeUsingExistingNodes(dupKey, true);
             checkTreeUsingExistingNodes(dupKey, false);
 
-            /* Test potential matches */
+            /* Test potential matches. */
             checkTreeUsingPotentialNodes();
 
-            /* Test deletes */
+            /* Test deletes. */
 	    checkTreeWithDeletedBins(true);
 	    checkTreeWithDeletedBins(false);
 
-            /* Should be no latches held */
-            assertEquals(0, Latch.countLatchesHeld());
+            /* Should be no latches held. */
+            assertEquals(0, LatchSupport.countLatchesHeld());
         } catch (Throwable t) {
             t.printStackTrace();
             throw new Exception(t);
         }
     }
 
-    private void checkTreeUsingExistingNodes(Key dupKey,
+    private void checkTreeUsingExistingNodes(byte[] dupKey,
                                              boolean requireExactMatch) 
         throws DatabaseException {
 
-        /* Start at the root */
+        /* Start at the root. */
         DatabaseImpl database = DbInternal.dbGetDatabaseImpl(db);
         Tree tree = database.getTree();
 
@@ -194,22 +193,21 @@ public class GetParentNodeTest extends TestCase {
             tree.dump();
         }
 
-        rootIN = tree.withRootLatched(new GetRoot(DbInternal.dbGetDatabaseImpl(db)));
+        rootIN = tree.withRootLatchedShared
+	    (new GetRoot(DbInternal.dbGetDatabaseImpl(db)));
         rootIN.latch();
-        SearchResult result = tree.getParentINForChildIN(rootIN, true);
+        SearchResult result = tree.getParentINForChildIN(rootIN, true, true);
         assertFalse(result.exactParentFound);
         assertEquals(rootIN.getNEntries(), 2);
    
         /* Second and third level. */
         BIN dupTreeBin = null;
         DIN dupTreeDINRoot = null;
-        Key dupBinKey = null;
         firstBIN = null;
         int dupIndex = -1;
         for (int i = 0; i < rootIN.getNEntries(); i++) {
             /* Each level 2 IN. */
-            ChildReference ref = rootIN.getEntry(i);
-            IN in = (IN) ref.fetchTarget(database, rootIN);
+            IN in = (IN) rootIN.fetchTarget(i);
             if (i == 0) {
                 firstLevel2IN = in;
             }
@@ -217,17 +215,17 @@ public class GetParentNodeTest extends TestCase {
 
             /* For each BIN, find its parent, and then find its LNs. */
             for (int j = 0; j < in.getNEntries(); j++) {
-                BIN bin = (BIN) in.getEntry(j).fetchTarget(database, in);
+                BIN bin = (BIN) in.fetchTarget(j);
                 if (firstBIN == null) {
                     firstBIN = bin;
                 }
                 checkMatch(tree, bin, in, j, requireExactMatch);
 
                 for (int k = 0; k < bin.getNEntries(); k++) {
-                    Node n = bin.getEntry(k).fetchTarget(database, bin);
+                    Node n = bin.fetchTarget(k);
                     if (n instanceof LN) {
-                        checkMatch(tree, (LN) n, bin, bin.getEntry(k).getKey(),
-                                   null, k, bin.getEntry(k).getLsn());
+                        checkMatch(tree, (LN) n, bin, bin.getKey(k),
+                                   null, k, bin.getLsn(k));
                     }
                 }
                     
@@ -235,7 +233,7 @@ public class GetParentNodeTest extends TestCase {
                 if (findIndex > 0) {
                     dupIndex = findIndex;
                     dupTreeDINRoot =
-                        (DIN) bin.getEntry(dupIndex).fetchTarget(database, bin);
+                        (DIN) bin.fetchTarget(dupIndex);
                     dupTreeBin = bin;
                 }
             }
@@ -249,22 +247,19 @@ public class GetParentNodeTest extends TestCase {
         assertTrue(dupTreeDINRoot.getNEntries() > 0);
 
         for (int i = 0; i < dupTreeDINRoot.getNEntries(); i++) {
-            ChildReference ref = dupTreeDINRoot.getEntry(i);
-            IN in = (IN) ref.fetchTarget(database, dupTreeDINRoot);
+            IN in = (IN) dupTreeDINRoot.fetchTarget(i);
             checkMatch(tree, in, dupTreeDINRoot, i, requireExactMatch);
             if (firstDBIN == null) {
                 firstDBIN = (DBIN)in;
             }
 
             for (int k = 0; k < in.getNEntries(); k++) {
-                Node n = in.getEntry(k).fetchTarget(database, in);
+                Node n = in.fetchTarget(k);
                 LN ln = (LN) n;
                 checkMatch(tree, ln, (BIN)in, dupKey,
-                           new Key(ln.getData()),
-                           k, in.getEntry(k).getLsn());
-                
+                           ln.getData(),
+                           k, in.getLsn(k));
             }
-
         }
     }
             
@@ -279,8 +274,8 @@ public class GetParentNodeTest extends TestCase {
         throws DatabaseException {
 
         target.latch();
-        SearchResult result = tree.getParentINForChildIN(target,
-                                                         requireExactMatch);
+        SearchResult result = tree.getParentINForChildIN
+            (target, requireExactMatch, true);
         assertTrue(result.exactParentFound);
         assertEquals("Target=" + target + " parent=" + parent,
                      index, result.index);
@@ -291,28 +286,36 @@ public class GetParentNodeTest extends TestCase {
     /*
      * Search for the BIN for this LN.
      */
-    private void checkMatch(Tree tree, LN target, BIN parent, Key mainKey,
-                            Key dupKey, int index, DbLsn expectedLsn) 
+    private void checkMatch(Tree tree,
+			    LN target,
+			    BIN parent,
+			    byte[] mainKey,
+                            byte[] dupKey,
+			    int index,
+			    long expectedLsn) 
         throws DatabaseException {
         TreeLocation location = new TreeLocation();
 
         
-        assertTrue(tree.getParentBINForChildLN(location, mainKey, dupKey, target,
-                                               false, true, false));
+        assertTrue
+	    (tree.getParentBINForChildLN(location, mainKey, dupKey, target,
+					 false, true, false, true));
         location.bin.releaseLatch();
         assertEquals(parent, location.bin);
         assertEquals(index, location.index);
         assertEquals(expectedLsn, location.childLsn);
 
-        assertTrue(tree.getParentBINForChildLN(location, mainKey, dupKey, target,
-                                               true, false, true));
+        assertTrue
+	    (tree.getParentBINForChildLN(location, mainKey, dupKey, target,
+					 true, false, true, true));
         location.bin.releaseLatch();
         assertEquals(parent, location.bin);
         assertEquals(index, location.index);
         assertEquals(expectedLsn, location.childLsn);
 
-        assertTrue(tree.getParentBINForChildLN(location, mainKey, dupKey, target,
-                                               true, true, false));
+        assertTrue
+	    (tree.getParentBINForChildLN(location, mainKey, dupKey, target,
+					 true, true, false, true));
         location.bin.releaseLatch();
         assertEquals(parent, location.bin);
         assertEquals(index, location.index);
@@ -343,48 +346,47 @@ public class GetParentNodeTest extends TestCase {
 
         DatabaseImpl database = DbInternal.dbGetDatabaseImpl(db);
         Tree tree = database.getTree();
+
         /* 
-         * Make an IN with the key "ab". Its potential parent should
-         * be the first level 2 IN.
+         * Make an IN with the key "ab". Its potential parent should be the
+         * first level 2 IN.
          */
-        IN inAB = new IN(database, new Key(new String("ab").getBytes()), 4, 2);
+        IN inAB = new IN(database, "ab".getBytes(), 4, 2);
         checkPotential(tree, inAB, firstLevel2IN);
 
         /* 
-         * Make an BIN with the key "x". Its potential parent should
-         * be the first level 2 IN.
+         * Make an BIN with the key "x". Its potential parent should be the
+         * first level 2 IN.
          */
         BIN binAB =
-	    new BIN(database, new Key(new String("ab").getBytes()), 4, 1);
+	    new BIN(database, "ab".getBytes(), 4, 1);
         checkPotential(tree, binAB, firstLevel2IN);
 
         /*
-         * Make a DIN with the key "a". Its potential parent should be
-         * BIN(c).
+         * Make a DIN with the key "a". Its potential parent should be BIN(c).
          */
         DIN dinA = new DIN(database,
-                           new Key("data1".getBytes()),
+                           "data1".getBytes(),
                            4,
-                           new Key("a".getBytes()),
+                           "a".getBytes(),
                            null, 3);
         checkPotential(tree, dinA, firstBIN);
 
         /*
          * Make an LN with the key "ab". It's potential parent should be the 
-         * BINa
+         * BINa.
          */
         LN LNab = new LN("foo".getBytes());
-        Key mainKey = new Key("ab".getBytes());
+        byte[] mainKey = "ab".getBytes();
         checkPotential(tree, LNab, firstBIN, mainKey,
-                       new Key(LNab.getData()), mainKey);
+                       LNab.getData(), mainKey);
 
         /**
-         * Make a dup LN with the key b. Its potential parent should be
-         * DBINb.
+         * Make a dup LN with the key b. Its potential parent should be DBINb.
          */
         LN LNdata = new LN("data".getBytes());
-        mainKey = new Key("b".getBytes());
-        Key dupKey = new Key(LNdata.getData());
+        mainKey = "b".getBytes();
+        byte[] dupKey = LNdata.getData();
         checkPotential(tree, LNdata, firstDBIN, mainKey, dupKey, dupKey);
     }
 
@@ -393,12 +395,13 @@ public class GetParentNodeTest extends TestCase {
 
         /* Try an exact match, expect a failure, then try an inexact match. */
         potential.latch();
-        SearchResult result = tree.getParentINForChildIN(potential, true);
+        SearchResult result = tree.getParentINForChildIN
+            (potential, true, true);
         assertFalse(result.exactParentFound);
         assertTrue(result.parent == null);
 
         potential.latch();
-        result = tree.getParentINForChildIN(potential, false);
+        result = tree.getParentINForChildIN(potential, false, true);
         assertFalse(result.exactParentFound);
         assertEquals("expected = " + expectedParent.getNodeId() +
                      " got" + result.parent.getNodeId(),
@@ -407,15 +410,15 @@ public class GetParentNodeTest extends TestCase {
     }
 
     private void checkPotential(Tree tree, LN potential, BIN expectedParent,
-                                Key mainKey, Key dupKey, Key expectedKey)
+                                byte[] mainKey, byte[] dupKey, byte[] expectedKey)
         throws DatabaseException {
 
         /* Try an exact match, expect a failure, then try an inexact match. */
         TreeLocation location = new TreeLocation();
         assertFalse(tree.getParentBINForChildLN(location, 
-                                                  mainKey, dupKey,
-                                                  potential, false,
-                                                  false, true));
+						mainKey, dupKey,
+						potential, false,
+						false, true, true));
         location.bin.releaseLatch();
         assertEquals(location.bin, expectedParent);
         assertEquals(expectedKey, location.lnKey);
@@ -425,22 +428,22 @@ public class GetParentNodeTest extends TestCase {
         throws DatabaseException {
 
 	/**
-	 * Mark all refs from the IN's to the BIN's as "known deleted".
+	 * Mark all refs from the IN's to the BIN's as "known deleted".  Start
+	 * at the root.
 	 */
-        /* Start at the root */
         DatabaseImpl database = DbInternal.dbGetDatabaseImpl(db);
         Tree tree = database.getTree();
 
-        rootIN = tree.withRootLatched(new GetRoot(DbInternal.dbGetDatabaseImpl(db)));
+        rootIN = tree.withRootLatchedShared
+	    (new GetRoot(DbInternal.dbGetDatabaseImpl(db)));
    
         /* Second and third level. */
         for (int i = 0; i < rootIN.getNEntries(); i++) {
             /* Each level 2 IN. */
-            IN in = (IN) rootIN.getEntry(i).fetchTarget(database, rootIN);
+            IN in = (IN) rootIN.fetchTarget(i);
             for (int j = 0; j < in.getNEntries(); j++) {
-		ChildReference ref = in.getEntry(j);
-                BIN bin = (BIN) ref.getTarget();
-		ref.setKnownDeleted();
+                BIN bin = (BIN) in.getTarget(j);
+		in.setKnownDeleted(j);
 		checkDeletedMatch(tree, bin, in, j, requireExactMatch);
             }
 	}
@@ -457,7 +460,8 @@ public class GetParentNodeTest extends TestCase {
         throws DatabaseException {
 
         target.latch();
-        SearchResult result = tree.getParentINForChildIN(target, requireExactMatch);
+        SearchResult result = tree.getParentINForChildIN
+            (target, requireExactMatch, true);
         assertFalse(result.exactParentFound);
         assertEquals("Target=" + target + " parent=" + parent,
                      index, result.index);
