@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: LockManagerTest.java,v 1.33 2004/05/25 03:03:35 linda Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: LockManagerTest.java,v 1.42 2006/01/10 00:05:00 mark Exp $
+ */
 
 package com.sleepycat.je.txn;
 
@@ -14,11 +14,12 @@ import java.io.File;
 import junit.framework.TestCase;
 
 import com.sleepycat.je.DatabaseException;
+import com.sleepycat.je.DeadlockException;
 import com.sleepycat.je.EnvironmentConfig;
-import com.sleepycat.je.LockNotGrantedException;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.junit.JUnitThread;
+import com.sleepycat.je.util.TestUtils;
 
 public class LockManagerTest extends TestCase {
     
@@ -34,15 +35,16 @@ public class LockManagerTest extends TestCase {
     private File envHome;
 
     public LockManagerTest() {
-        envHome =  new File(System.getProperty("testdestdir"));
+        envHome =  new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp()
 	throws DatabaseException {
 
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "6");
         envConfig.setAllowCreate(true);
+	envConfig.setTransactional(true);
         env = new EnvironmentImpl(envHome, envConfig);
 
         TxnManager txnManager = env.getTxnManager();
@@ -71,14 +73,14 @@ public class LockManagerTest extends TestCase {
 	try {
 	    assertFalse(lockManager.isOwner(nid, txn1, LockType.READ));
 	    assertFalse(lockManager.isOwner(nid, txn1, LockType.WRITE));
-	    assertFalse(lockManager.isOwner(nid, txn1, null));
 	    assertFalse(lockManager.isLocked(nid));
 	    assertFalse(lockManager.isWaiter(nid, txn1));
-	    lockManager.lock(1, txn1, LockType.READ, 0, false);
+	    lockManager.lock(1, txn1, LockType.READ, 0, false, null);
 
 	    /* already holds this lock */
 	    assertEquals(LockGrantType.EXISTING,
-                         lockManager.lock(1, txn1, LockType.READ, 0, false));
+                         lockManager.lock(1, txn1, LockType.READ, 0,
+					  false, null));
 	    assertFalse(lockManager.isOwner(nid, txn2, LockType.READ));
 	    assertFalse(lockManager.isOwner(nid, txn2, LockType.WRITE));
 	    assertTrue(lockManager.isLocked(nid));
@@ -101,11 +103,12 @@ public class LockManagerTest extends TestCase {
             assertFalse(lockManager.isOwner(nid, txn1, LockType.READ));
 	    assertFalse(lockManager.nOwners(nid) == 1);
 
-	    lockManager.lock(1, txn1, LockType.WRITE, 0, false);
+	    lockManager.lock(1, txn1, LockType.WRITE, 0, false, null);
 	    /* holds write and subsequent request for READ is ok */
-	    lockManager.lock(1, txn1, LockType.READ, 0, false);
+	    lockManager.lock(1, txn1, LockType.READ, 0, false, null);
 	    /* already holds this lock */
-	    assertTrue(lockManager.lock(1, txn1, LockType.WRITE, 0, false) ==
+	    assertTrue(lockManager.lock(1, txn1, LockType.WRITE,
+					0, false, null) ==
 		       LockGrantType.EXISTING);
 	    assertFalse(lockManager.isWaiter(nid, txn1));
 	} catch (Exception e) {
@@ -124,11 +127,10 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.READ, 0, false);
+			lockManager.lock(1, txn1, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.READ));
-			assertTrue
-			    (lockManager.isOwner(nid, txn1, null));
 			sequence++;
 			while (sequence < 3) {
 			    Thread.yield();
@@ -145,7 +147,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders2") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn2, LockType.READ, 0, false);
+			lockManager.lock(1, txn2, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
 			sequence++;
@@ -164,7 +167,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders3") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn3, LockType.READ, 0, false);
+			lockManager.lock(1, txn3, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn3, LockType.READ));
 			sequence++;
@@ -198,7 +202,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.READ, 0, false);
+			lockManager.lock(1, txn1, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.READ));
 			while (lockManager.nWaiters(nid) < 1) {
@@ -220,7 +225,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders2") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn2, LockType.READ, 0, false);
+			lockManager.lock(1, txn2, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
 			while (lockManager.nWaiters(nid) < 1) {
@@ -244,7 +250,8 @@ public class LockManagerTest extends TestCase {
 			while (lockManager.nOwners(nid) < 2) {
 			    Thread.yield();
 			}
-			lockManager.lock(1, txn3, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn3, LockType.WRITE, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn3, LockType.WRITE));
 		    } catch (DatabaseException DBE) {
@@ -275,7 +282,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.READ, 0, false);
+			lockManager.lock(1, txn1, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.READ));
 			while (lockManager.nWaiters(nid) < 2) {
@@ -293,7 +301,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testMultipleReaders2") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn2, LockType.READ, 0, false);
+			lockManager.lock(1, txn2, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
 			while (lockManager.nWaiters(nid) < 2) {
@@ -314,7 +323,8 @@ public class LockManagerTest extends TestCase {
 			while (lockManager.nOwners(nid) < 2) {
 			    Thread.yield();
 			}
-			lockManager.lock(1, txn3, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn3, LockType.WRITE, 0,
+					 false, null);
 			while (lockManager.nWaiters(nid) < 1) {
 			    Thread.yield();
 			}
@@ -335,7 +345,8 @@ public class LockManagerTest extends TestCase {
 			while (lockManager.nWaiters(nid) < 1) {
 			    Thread.yield();
 			}
-			lockManager.lock(1, txn4, LockType.READ, 0, false);
+			lockManager.lock(1, txn4, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn4, LockType.READ));
 			lockManager.release(1L, txn4);
@@ -370,7 +381,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testUpgradeLock1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.READ, 0, false);
+			lockManager.lock(1, txn1, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.READ));
 			while (lockManager.nWaiters(nid) < 2) {
@@ -388,13 +400,15 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testUpgradeLock2") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn2, LockType.READ, 0, false);
+			lockManager.lock(1, txn2, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
 			while (lockManager.nWaiters(nid) < 1) {
 			    Thread.yield();
 			}
-			lockManager.lock(1, txn2, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn2, LockType.WRITE, 0,
+					 false, null);
 			assertTrue(lockManager.nWaiters(nid) == 1);
 			lockManager.release(1L, txn2);
 		    } catch (DatabaseException DBE) {
@@ -411,7 +425,8 @@ public class LockManagerTest extends TestCase {
 			while (lockManager.nOwners(nid) < 2) {
 			    Thread.yield();
 			}
-			lockManager.lock(1, txn3, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn3, LockType.WRITE, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn3, LockType.WRITE));
 			lockManager.release(1L, txn3);
@@ -441,7 +456,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testNonBlocking1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.READ, 0, false);
+			lockManager.lock(1, txn1, LockType.READ, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.READ));
 			while (sequence < 1) {
@@ -463,10 +479,9 @@ public class LockManagerTest extends TestCase {
 			while (lockManager.nOwners(nid) < 1) {
 			    Thread.yield();
 			}
-			try {
-			    lockManager.lock(1, txn2, LockType.WRITE, 0, true);
-                            fail();
-                        } catch (LockNotGrantedException expected) {}
+                        LockGrantType grant = lockManager.lock
+                            (1, txn2, LockType.WRITE, 0, true, null);
+                        assertSame(LockGrantType.DENIED, grant);
 			assertFalse
 			    (lockManager.isOwner(nid, txn2, LockType.WRITE));
 			assertFalse
@@ -480,7 +495,7 @@ public class LockManagerTest extends TestCase {
 			}
 			assertTrue
 			    (lockManager.lock(1, txn2, LockType.WRITE, 0,
-                                              false) ==
+                                              false, null) ==
 			     LockGrantType.NEW);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.WRITE));
@@ -513,7 +528,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testNonBlocking1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn1, LockType.WRITE, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.WRITE));
 			sequence++;
@@ -536,10 +552,9 @@ public class LockManagerTest extends TestCase {
 			while (sequence < 1) {
 			    Thread.yield();
 			}
-                        try {
-			    lockManager.lock(1, txn2, LockType.READ, 0, true);
-                            fail();
-                        } catch (LockNotGrantedException expected) {}
+                        LockGrantType grant = lockManager.lock
+                            (1, txn2, LockType.READ, 0, true, null);
+                        assertSame(LockGrantType.DENIED, grant);
 			assertFalse
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
 			assertFalse
@@ -553,7 +568,7 @@ public class LockManagerTest extends TestCase {
 			}
 			assertTrue
 			    (lockManager.lock(1, txn2, LockType.READ, 0,
-                                              false) ==
+                                              false, null) ==
 			     LockGrantType.NEW);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
@@ -586,7 +601,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testBlocking1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn1, LockType.WRITE, 0,
+					 false, null);
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.WRITE));
 			sequence++;
@@ -611,9 +627,10 @@ public class LockManagerTest extends TestCase {
 			}
 			try {
 			    lockManager.lock(1, txn2, LockType.READ, 500,
-                                             false);
+                                             false, null);
 			    fail("didn't time out");
-			} catch (LockTimeOutException e) {
+			} catch (DeadlockException e) {
+                            assertTrue(e.getMessage().startsWith("Lock "));
 			}
 			assertFalse
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
@@ -628,7 +645,7 @@ public class LockManagerTest extends TestCase {
 			}
 			assertTrue
 			    (lockManager.lock(1, txn2, LockType.READ, 0,
-                                              false) ==
+                                              false, null) ==
 			     LockGrantType.NEW);
 			assertTrue
 			    (lockManager.isOwner(nid, txn2, LockType.READ));
@@ -657,7 +674,8 @@ public class LockManagerTest extends TestCase {
 	    new JUnitThread("testDeadlock1") {
 		public void testBody() {
 		    try {
-			lockManager.lock(1, txn1, LockType.WRITE, 0, false);
+			lockManager.lock(1, txn1, LockType.WRITE, 0,
+					 false, null);
 			System.out.println("t1 has locked 1");
 			assertTrue
 			    (lockManager.isOwner(nid, txn1, LockType.WRITE));
@@ -668,7 +686,8 @@ public class LockManagerTest extends TestCase {
 			    Thread.yield();
 			}
 
-			lockManager.lock(2, txn1, LockType.READ, 1000, false);
+			lockManager.lock(2, txn1, LockType.READ, 1000,
+					 false, null);
 			System.out.println("t1 about to sleep");
 			Thread.sleep(5000);
 
@@ -692,13 +711,15 @@ public class LockManagerTest extends TestCase {
 			    Thread.yield();
 			}
 
-			lockManager.lock(2, txn2, LockType.WRITE, 0, false);
+			lockManager.lock(2, txn2, LockType.WRITE, 0,
+					 false, null);
 			System.out.println("t2 has locked 2");
 
 			sequence++;   // bump to 2
 
 			System.out.println("t2 about to lock 1");
-			lockManager.lock(1, txn2, LockType.READ, 1000, false);
+			lockManager.lock(1, txn2, LockType.READ, 1000,
+					 false, null);
 			System.out.println("t2 about to sleep");
 			Thread.sleep(5000);
 
