@@ -1,11 +1,11 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: TxnEndTest.java,v 1.55 2004/08/13 03:20:46 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2006
+ *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: TxnEndTest.java,v 1.63 2006/01/03 21:56:31 bostic Exp $
+ */
 
 package com.sleepycat.je.txn;
 
@@ -31,6 +31,7 @@ import com.sleepycat.je.Transaction;
 import com.sleepycat.je.TransactionStats;
 import com.sleepycat.je.VerifyConfig;
 import com.sleepycat.je.config.EnvironmentParams;
+import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.junit.JUnitThread;
 import com.sleepycat.je.log.FileManager;
 import com.sleepycat.je.util.TestUtils;
@@ -48,7 +49,7 @@ public class TxnEndTest extends TestCase {
     public TxnEndTest()
 	throws DatabaseException {
 
-        envHome = new File(System.getProperty("testdestdir"));
+        envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
     public void setUp() 
@@ -60,7 +61,7 @@ public class TxnEndTest extends TestCase {
          * Run environment without in compressor on so we can check the
          * compressor queue in a deterministic way.
          */
-        EnvironmentConfig envConfig = new EnvironmentConfig();
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setTransactional(true);
         envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(), "6");
         envConfig.setConfigParam(EnvironmentParams.
@@ -73,7 +74,15 @@ public class TxnEndTest extends TestCase {
     public void tearDown()
         throws IOException, DatabaseException {
 
-        env.close();
+        if (env != null) {
+            try {
+                env.close();
+            } catch (Exception e) {
+                System.out.println("tearDown: " + e);
+            }
+        }
+        env = null;
+
         TestUtils.removeFiles("TearDown", envHome, FileManager.JE_SUFFIX);
     }
 
@@ -92,12 +101,15 @@ public class TxnEndTest extends TestCase {
         }
     }
 
-    private void closeDbs()
+    private void closeAll()
         throws DatabaseException {	
 
         for (int i = 0; i < NUM_DBS; i++) {
             dbs[i].close();
         }
+        dbs = null;
+        env.close();
+        env = null;
     }
 
     /**
@@ -223,18 +235,19 @@ public class TxnEndTest extends TestCase {
         throws DatabaseException {
 
         for (int i = 0; i < NUM_DBS; i++) {
-            // Run validate
-            DbInternal.dbGetDatabaseImpl(dbs[i]).
-		verify(new VerifyConfig());
+            /* Run verify */
+            DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(dbs[i]);
+            assertTrue(dbImpl.verify(new VerifyConfig(),
+                                      dbImpl.getEmptyStats()));
 
             Cursor verifyCursor =
-		dbs[i].openCursor(null, CursorConfig.DIRTY_READ);
+		dbs[i].openCursor(null, CursorConfig.READ_UNCOMMITTED);
             DatabaseEntry key = new DatabaseEntry();
             DatabaseEntry data = new DatabaseEntry();
             OperationStatus status =
 		verifyCursor.getFirst(key, data, LockMode.DEFAULT);
             for (int d = 0; d < numKeys; d++) {
-                assertEquals(OperationStatus.SUCCESS, status);
+                assertEquals("key=" + d, OperationStatus.SUCCESS, status);
                 byte[] expected = TestUtils.getTestArray(d + valueOffset);
                 assertTrue(Arrays.equals(expected, key.getData()));
                 assertTrue("Expected= " + TestUtils.dumpByteArray(expected) +
@@ -274,6 +287,7 @@ public class TxnEndTest extends TestCase {
             closeCursors();
             txn.abort();
             verifyData(numKeys*2, 0);
+
             /*
              * Check the in compressor queue, we should have some number of
              * bins on. If the queue size is 0, then check the processed stats,
@@ -315,7 +329,7 @@ public class TxnEndTest extends TestCase {
             envStat = env.getStats(TestUtils.FAST_STATS);
             assertTrue(envStat.getInCompQueueSize() > queueSize);
 
-            closeDbs();
+            closeAll();
 
         } catch (Throwable t) {
             // print stacktrace before attempt to run tearDown
@@ -339,7 +353,7 @@ public class TxnEndTest extends TestCase {
         } catch (DatabaseException e) {
         }
 	closeCursors();
-        closeDbs();
+        closeAll();
     }
 
     class CascadingAbortTestJUnitThread extends JUnitThread {
@@ -441,7 +455,7 @@ public class TxnEndTest extends TestCase {
 	    if (false) {
 		db.close();
 		env.close();
-		EnvironmentConfig envConfig = new EnvironmentConfig();
+		EnvironmentConfig envConfig = TestUtils.initEnvConfig();
 		envConfig.setTransactional(true);
 		envConfig.setConfigParam(EnvironmentParams.NODE_MAX.getName(),
 					 "6");
@@ -504,7 +518,7 @@ public class TxnEndTest extends TestCase {
             verifyData(numKeys, 0);  // verify w/dirty read    
             txn.abort();
             
-            closeDbs();
+            closeAll();
         } catch (Throwable t) {
             t.printStackTrace();
             throw t;
@@ -534,11 +548,12 @@ public class TxnEndTest extends TestCase {
         try {
             txnB.setLockTimeout(1000);
 
-            Database dbB =
 		env.openDatabase(txnB, "foo", dbConfig);
             fail("Shouldn't be able to open foo");
         } catch (DatabaseException e) {
         }
+	/* txnB must be aborted since openDatabase timed out. */
+	txnB.abort();
 
         // Open this database with the same txn and another handle
         Database dbC =
@@ -546,6 +561,7 @@ public class TxnEndTest extends TestCase {
         
         // Now commit txnA and txnB should be able to open this.
         txnA.commit();
+	txnB = env.beginTransaction(null, null);
         Database dbB =
 	    env.openDatabase(txnB, "foo", dbConfig);
         txnB.commit();
@@ -556,4 +572,19 @@ public class TxnEndTest extends TestCase {
         dbB.close();
         dbC.close();
     }
+
+    /* Test that the transaction is unsable about a close. */
+    public void testClose()
+        throws DatabaseException {
+
+        Transaction txnA = env.beginTransaction(null, null);
+        txnA.commit();
+
+        try {
+            env.openDatabase(txnA, "foo", null);
+            fail("Should not be able to use a closed exception");
+        } catch (DatabaseException expected) {
+        }
+    }
+
 }
